local private = {}

function CDTL2:CreateBarFrames()
	local frame1Enabled = CDTL2.db.profile.barFrames["frame1"]["enabled"]
	local frame2Enabled = CDTL2.db.profile.barFrames["frame2"]["enabled"]
	local frame3Enabled = CDTL2.db.profile.barFrames["frame3"]["enabled"]
	
	if frame1Enabled then
		if CDTL2_BarFrame_1 then
			CDTL2:RefreshBarFrame(1)
			--CDTL2:Print("BarFrame 1 already created")
		else
			private.CreateBarFrame(1)
			--CDTL2:Print("Created BarFrame 1")
		end
	end
	
	if frame2Enabled then
		if CDTL2_BarFrame_2 then
			CDTL2:RefreshBarFrame(2)
			--CDTL2:Print("BarFrame 2 already created")
		else
			private.CreateBarFrame(2)
			--CDTL2:Print("Created BarFrame 2")
		end
	end
	
	if frame3Enabled then
		if CDTL2_BarFrame_3 then
			CDTL2:RefreshBarFrame(3)
			--CDTL2:Print("BarFrame 3 already created")
		else
			private.CreateBarFrame(3)
			--CDTL2:Print("Created BarFrame 3")
		end
	end
end

function CDTL2:RefreshBarFrame(i)
	local f = nil
	local s = nil
	if i == 1 then
		f = CDTL2_BarFrame_1
		s = CDTL2.db.profile.barFrames["frame1"]
	elseif i == 2 then
		f = CDTL2_BarFrame_2
		s = CDTL2.db.profile.barFrames["frame2"]
	elseif i == 3 then
		f = CDTL2_BarFrame_3
		s = CDTL2.db.profile.barFrames["frame3"]
	end
	
	f.name = s["name"]

	local anchor = "CENTER"
	if s["grow"] == "DOWN" then
		anchor = "TOP"
	elseif  s["grow"] == "UP" then
		anchor = "BOTTOM"
	end
	
	f:ClearAllPoints()
	f:SetPoint(s["relativeTo"], s["posX"], s["posY"])
	f:SetSize(s["width"], s["height"])

	f.mf:ClearAllPoints()
	f.mf:SetPoint("CENTER", 0, 0)
	f.mf:SetSize(s["width"], s["height"])
	
	-- BACKGROUND
	f.bg:SetTexture(CDTL2.LSM:Fetch("statusbar", s["bgTexture"]))
	f.bg:SetAllPoints(true)
	f.bg:SetVertexColor(1, 0, 1, 0)
	
	-- MF BACKGROUND
	f.mf.bg:SetTexture(CDTL2.LSM:Fetch("statusbar", s["bgTexture"]))
	f.mf.bg:SetAllPoints(true)
	f.mf.bg:SetVertexColor(
		s["bgTextureColor"]["r"],
		s["bgTextureColor"]["g"],
		s["bgTextureColor"]["b"],
		s["bgTextureColor"]["a"]
	)
	
	-- BORDER
	--CDTL2:SetBorder(f.bd, s["border"])
	--f.bd:SetFrameLevel(f:GetFrameLevel() + 1)
	CDTL2:SetBorder(f.mf.bd, s["border"])
	f.mf.bd:SetFrameLevel(f.mf:GetFrameLevel() + 1)
	
	-- DEBUG/UNLOCK
	f.db:ClearAllPoints()
	f.db:SetPoint("CENTER", 0, 0)
	f.db.text:SetFont(CDTL2.LSM:Fetch("font", "Fira Sans Condensed"), 12, "NONE")
	f.db.text:ClearAllPoints()
	f.db.text:SetPoint("CENTER", 0, 0)
	f.db.text:SetText(f.name)
	f.db:SetSize(s["width"], s["height"])
	f.db.bg:SetAllPoints(true)
	f.db.bg:SetColorTexture( 
		CDTL2.colors["db"]["r"],
		CDTL2.colors["db"]["g"],
		CDTL2.colors["db"]["b"],
		CDTL2.colors["db"]["a"]
	)
	
	if CDTL2.db.profile.global["unlockFrames"] then
		CDTL2:FrameUnlock(f)
	else
		CDTL2:FrameLock(f)
	end
	
	if CDTL2.db.profile.global["debugMode"] then
		CDTL2:DebugOn(f)
	else
		CDTL2:DebugOff(f)
	end
end

private.CreateBarFrame = function(frameNumber)
	local frameName = "CDTL2_BarFrame_"..frameNumber
	local f = CreateFrame("Frame", frameName, UIParent, BackdropTemplateMixin and "BackdropTemplate" or nil)
	
	f.number = frameNumber
	f.childCount = 0
	
	f.mf = CreateFrame("Frame", frameName.."_MF", UIParent, BackdropTemplateMixin and "BackdropTemplate" or nil)
	f.mf:SetParent(f)
	f.mf.bg = f.mf:CreateTexture(nil, "BACKGROUND")
	f.mf.bd = CreateFrame("Frame", frameName.."_BD", UIParent, BackdropTemplateMixin and "BackdropTemplate" or nil)
	f.mf.bd:SetParent(f.mf)

	f.bg = f:CreateTexture(nil, "BACKGROUND")
	--f.bd = CreateFrame("Frame", frameName.."_BD", UIParent, BackdropTemplateMixin and "BackdropTemplate" or nil)
	--f.bd:SetParent(f)

	f.db = CreateFrame("Frame", frameName.."_DB", UIParent, BackdropTemplateMixin and "BackdropTemplate" or nil)
	f.db:SetParent(f)
	f.db.bg = f.db:CreateTexture(nil, "BACKGROUND")
	f.db.text = f.db:CreateFontString(nil,"ARTWORK")
	
	-- ON UPDATE
	f:HookScript("OnUpdate", function(self, elapsed)
		private.BarFrameUpdate(self, elapsed)
	end)
	
	-- DRAG AND DROP MOVEMENT
	f:RegisterForDrag("LeftButton")
	f:SetScript("OnDragStart", f.StartMoving)
	f:SetScript("OnDragStop", f.StopMovingOrSizing)
	
	CDTL2:RefreshBarFrame(frameNumber)
	table.insert(CDTL2.barFrames, f)
end

private.BarFrameUpdate = function(f, elapsed)
	local s = nil
	if f.number == 1 then
		s = CDTL2.db.profile.barFrames["frame1"]
	elseif f.number == 2 then
		s = CDTL2.db.profile.barFrames["frame2"]
	elseif f.number == 3 then
		s = CDTL2.db.profile.barFrames["frame3"]
	end
	
	local children = { f.mf:GetChildren() }
	local validChildren = {}
	
	local count = 0
	for _, child in ipairs(children) do
		if child.valid then
			if child:GetAlpha() ~= 0 then
				count = count + 1
				table.insert(validChildren, child)
			end
		end
	end
	
	--[[if count == 1 then
		validChildren[1]:ClearAllPoints()	
		validChildren[1]:SetPoint("CENTER", 0, 0)
	elseif count > 1 then
		for k, child in ipairs(validChildren) do
			
		end
	end]]--
	
	local bWidth = s["width"]
	local bHeight = s["height"]
	local bPadX = s["bar"]["xPadding"]
	local bPadY = s["bar"]["yPadding"]
	local fPad = s["padding"]
	
	local mfSizeX = bWidth + (fPad * 2)
	local mfSizeY = bHeight + (fPad * 2)
	
	local mfPosX = 0
	local mfPosY = 0
	
	local xInset = 0
	local yInset = 0
	
	local xOffset = 0
	local yOffset = 0
	
	local xMod = 0
	local yMod = 0
	
	if count == 1 then
		validChildren[1]:ClearAllPoints()	
		validChildren[1]:SetPoint("CENTER", 0, 0)
	elseif count > 1 then
		local anchor = "TOP"
		
		if s["grow"] == "UP" then
			yInset = -fPad
			xMod = 0
			yMod = -1
			
			mfSizeY = (bHeight * count) + (fPad * 2) + (bPadY + (count - 1))
			yOffset = (mfSizeY - (fPad * 2)) / 2
			yOffset = yOffset - (bHeight / 2)
		
		elseif s["grow"] == "DOWN" then
			yInset = -fPad
			xMod = 0
			yMod = -1
			
			mfSizeY = (bHeight * count) + (fPad * 2) + (bPadY + (count - 1))
			yOffset = (mfSizeY - (fPad * 2)) / -2
			yOffset = yOffset + (bHeight / 2)
		
		elseif s["grow"] == "LEFT" then
			anchor = "LEFT"
			xInset = fPad
			xMod = 1
			yMod = 0
			
			mfSizeX = (bWidth * count) + (fPad * 2) + (bPadX + (count - 1))
			xOffset = (mfSizeX - (fPad * 2)) / -2
			xOffset = xOffset + (bWidth / 2)
		
		elseif s["grow"] == "RIGHT" then
			anchor = "LEFT"
			xInset = fPad
			xMod = 1
			yMod = 0
			
			mfSizeX = (bWidth * count) + (fPad * 2) + (bPadX + (count - 1))
			xOffset = (mfSizeX - (fPad * 2)) / 2
			xOffset = xOffset - (bWidth / 2)
		
		elseif s["grow"] == "CENTER_V" then
			yInset = -fPad
			xMod = 0
			yMod = -1
			
			mfSizeY = (bHeight * count) + (fPad * 2) + (bPadY + (count - 1))
		
		elseif s["grow"] == "CENTER_H" then
			anchor = "LEFT"
			xInset = fPad
			xMod = 1
			yMod = 0
			
			mfSizeX = (bWidth * count) + (fPad * 2) + (bPadX + (count - 1))
		end
		
		if s["sorting"] == "ASCENDING" then
			table.sort(validChildren, function(a, b)
				local a = a.currentCD
				local b = b.currentCD
				
				return a < b
			end)
		elseif s["sorting"] == "DESCENDING" then
			table.sort(validChildren, function(a, b)
				local a = a.currentCD
				local b = b.currentCD
				
				return a > b
			end)
		end
		
		for k, child in ipairs(validChildren) do
			local x = (bWidth * (k - 1) * xMod) + (bPadX * (k - 1))
			local y = (bHeight * (k - 1) * yMod) - (bPadY * (k - 1))
						
			child:ClearAllPoints()		
			child:SetPoint(anchor, x + xInset, y + yInset)
		end
	end
	
	f.mf:SetSize(mfSizeX, mfSizeY)
	f.mf:ClearAllPoints()
	f.mf:SetPoint("CENTER", mfPosX + xOffset, mfPosY + yOffset)
	
	-- FRAME (UN)LOCKING
	if CDTL2.db.profile.global["unlockFrames"] then
		local _, _, relativeTo, xOfs, yOfs = f:GetPoint()
		--CDTL2:Print(f.name.." - "..relativeTo.." - "..xOfs.." - "..yOfs)
		
		if f.number == 1 then
			CDTL2.db.profile.barFrames["frame1"]["relativeTo"] = relativeTo
			CDTL2.db.profile.barFrames["frame1"]["posX"] = xOfs -- xOffset
			CDTL2.db.profile.barFrames["frame1"]["posY"] = yOfs -- yOffset
		elseif f.number == 2 then
			CDTL2.db.profile.barFrames["frame2"]["relativeTo"] = relativeTo
			CDTL2.db.profile.barFrames["frame2"]["posX"] = xOfs -- xOffset
			CDTL2.db.profile.barFrames["frame2"]["posY"] = yOfs -- yOffset
		elseif f.number == 3 then
			CDTL2.db.profile.barFrames["frame3"]["relativeTo"] = relativeTo
			CDTL2.db.profile.barFrames["frame3"]["posX"] = xOfs -- xOffset
			CDTL2.db.profile.barFrames["frame3"]["posY"] = yOfs -- yOffset
		end
	else
		--f:SetSize(s["width"], s["height"])
		--f:ClearAllPoints()
		--f:SetPoint("CENTER", UIParent, s["relativeTo"], xPos, yPos)
	end
	
	f.childCount = count
	--CDTL2:Autohide(f)
	
	if s["enabled"] then
		CDTL2:Autohide(f)
	else
		f:SetAlpha(0)
	end
end