local private = {}

function CDTL2:CreateLanes()
	local lane1Enabled = CDTL2.db.profile.lanes["lane1"]["enabled"]
	local lane2Enabled = CDTL2.db.profile.lanes["lane2"]["enabled"]
	local lane3Enabled = CDTL2.db.profile.lanes["lane3"]["enabled"]
	
	if lane1Enabled then
		if CDTL2_Lane_1 then
			CDTL2:RefreshLane(1)
			--CDTL2:Print("Lane 1 already created")
		else
			private.CreateLane(1)
			--CDTL2:Print("Created Lane 1")
		end
	end
	
	if lane2Enabled then
		if CDTL2_Lane_2 then
			CDTL2:RefreshLane(2)
			--CDTL2:Print("Lane 2 already created")
		else
			private.CreateLane(2)
			--CDTL2:Print("Created Lane 2")
		end
	end
	
	if lane3Enabled then
		if CDTL2_Lane_3 then
			CDTL2:RefreshLane(3)
			--CDTL2:Print("Lane 3 already created")
		else
			private.CreateLane(3)
			--CDTL2:Print("Created Lane 3")
		end
	end
end

function CDTL2:RefreshLane(i)
	local f = nil
	local s = nil
	if i == 1 then
		f = CDTL2_Lane_1
		s = CDTL2.db.profile.lanes["lane1"]
	elseif i == 2 then
		f = CDTL2_Lane_2
		s = CDTL2.db.profile.lanes["lane2"]
	elseif i == 3 then
		f = CDTL2_Lane_3
		s = CDTL2.db.profile.lanes["lane3"]
	end
	
	f.name = s["name"]

	f:ClearAllPoints()
	f:SetPoint(s["relativeTo"], s["posX"], s["posY"])
	f:SetSize(s["width"], s["height"])
	
	-- FOREGROUND
	f:SetMinMaxValues(0, 1)
	f:SetValue(1)
	f:SetStatusBarTexture(CDTL2.LSM:Fetch("statusbar", s["fgTexture"]))
	f:GetStatusBarTexture():SetHorizTile(false)
	f:GetStatusBarTexture():SetVertTile(false)
	f:SetStatusBarColor(
		s["fgTextureColor"]["r"],
		s["fgTextureColor"]["g"],
		s["fgTextureColor"]["b"],
		s["fgTextureColor"]["a"]
	)
	
	-- BACKGROUND
	f.bg:SetTexture(CDTL2.LSM:Fetch("statusbar", s["bgTexture"]))
	f.bg:SetAllPoints(true)
	f.bg:SetVertexColor(
		s["bgTextureColor"]["r"],
		s["bgTextureColor"]["g"],
		s["bgTextureColor"]["b"],
		s["bgTextureColor"]["a"]
	)
	
	-- SECONDARY TRACKING
	f.st:ClearAllPoints()
	f.st:SetSize(s["tracking"]["stWidth"], s["tracking"]["stHeight"])
	f.st.bg:SetTexture(CDTL2.LSM:Fetch("statusbar", s["tracking"]["stTexture"]))
	f.st.bg:SetAllPoints(true)
	f.st.bg:SetVertexColor(
		s["tracking"]["stTextureColor"]["r"],
		s["tracking"]["stTextureColor"]["g"],
		s["tracking"]["stTextureColor"]["b"],
		s["tracking"]["stTextureColor"]["a"]
	)
	
	-- BORDER
	CDTL2:SetBorder(f.bd, s["border"])
	f.bd:SetFrameLevel(f:GetFrameLevel() + 1)
	
	-- TEXT
	private.SetModeText(f, s)
	private.RefreshText(f, s)
	
	-- DEBUG/UNLOCK
	f.db:ClearAllPoints()
	f.db:SetPoint("CENTER", 0, 0)
	f.db.text:SetFont(CDTL2.LSM:Fetch("font", "Fira Sans Condensed"), 12, "NONE")
	f.db.text:ClearAllPoints()
	f.db.text:SetPoint("CENTER", 0, 0)
	f.db.text:SetText(f.name)
	f.db:SetSize(s["width"], s["height"])
	f.db.bg:SetAllPoints(true)
	f.db.bg:SetColorTexture( 
		CDTL2.colors["db"]["r"],
		CDTL2.colors["db"]["g"],
		CDTL2.colors["db"]["b"],
		CDTL2.colors["db"]["a"]
	)
	
	if CDTL2.db.profile.global["unlockFrames"] then
		CDTL2:FrameUnlock(f)
	else
		CDTL2:FrameLock(f)
	end
	
	if CDTL2.db.profile.global["debugMode"] then
		CDTL2:DebugOn(f)
	else
		CDTL2:DebugOff(f)
	end
	
	--[[if s["enabled"] then
		f:Show()
		--f:SetAlpha(1)
		CDTL2:Print("Enabled")
	else
		f:Hide()
		--f:SetAlpha(0)
		CDTL2:Print("Disabled")
	end]]--
end

private.RefreshText = function(f, s)
	-- MODE TEXT
	for i = 1, 10, 1 do
		local tSettings = nil
		local tObject = nil
	   
		if i == 1 then
			tSettings = s["modeText"]["text1"]
			tObject = f.t1
		elseif i == 2 then
			tSettings = s["modeText"]["text2"]
			tObject = f.t2
		elseif i == 3 then
			tSettings = s["modeText"]["text3"]
			tObject = f.t3
		elseif i == 4 then
			tSettings = s["modeText"]["text4"]
			tObject = f.t4
		elseif i == 5 then
			tSettings = s["modeText"]["text5"]
			tObject = f.t5
		elseif i == 6 then
			tSettings = s["customText"]["text1"]
			tObject = f.c1
		elseif i == 7 then
			tSettings = s["customText"]["text2"]
			tObject = f.c2
		elseif i == 8 then
			tSettings = s["customText"]["text3"]
			tObject = f.c3
		elseif i == 9 then
			tSettings = s["customText"]["text4"]
			tObject = f.c4
		elseif i == 10 then
			tSettings = s["customText"]["text5"]
			tObject = f.c5
		end
		
		tObject:ClearAllPoints()
		tObject:SetPoint(
				tSettings["align"],
				f,
				tSettings["anchor"],
				tSettings["offX"] + (s["width"] * tSettings["pos"]),
				tSettings["offY"]
			)
		tObject:SetFont(CDTL2.LSM:Fetch("font", tSettings["font"]), tSettings["size"], tSettings["outline"])
		tObject:SetTextColor(
				tSettings["color"]["r"],
				tSettings["color"]["g"],
				tSettings["color"]["b"],
				tSettings["color"]["a"]
			)
		tObject:SetShadowColor(
				tSettings["shadColor"]["r"],
				tSettings["shadColor"]["g"],
				tSettings["shadColor"]["b"],
				tSettings["shadColor"]["a"]
			)
		tObject:SetShadowOffset(tSettings["shadX"], tSettings["shadY"])
		tObject:SetNonSpaceWrap(false)
		
		--tObject:SetText(tSettings["text"])
		tObject:SetText(CDTL2:ConvertTextTags(tSettings["text"], f))
		
		if tSettings["enabled"] == true and tSettings["used"] == true then
			tObject:SetAlpha(1)
		else
			tObject:SetAlpha(0)
		end
	end
end

private.CalcStacking = function(f, s, validChildren, count)
	local baseLevel = 5
	local levelMultiplier = 5
	--local tOffset = 0

	if count >= 2 and s["stacking"]["enabled"] then
		local style = s["stacking"]["style"]
		
		if style == "GROUPED" then
			local height = s["stacking"]["height"]
			local grow = s["stacking"]["grow"]
			local iconSize = s["icons"]["size"]
			
			table.sort(validChildren, function(a, b)
				local aX, _ = a:GetCenter()
				local bX, _ = b:GetCenter()
				
				return aX < bX
			end)
			
			--if count > 1 then
				local indicies = {}
				local currentIndex = 1
				
				for k, iconA in pairs(validChildren) do
					if k == 1 then
						table.insert(indicies, currentIndex)
					end
					
					if k + 1 <= count then
						local iconB = validChildren[k + 1]
						local aX, _ = iconA:GetCenter()
						local bX, _ = iconB:GetCenter()
						local distance = math.abs(aX - bX)
						
						if distance <= iconSize then
							table.insert(indicies, currentIndex)
						else
							currentIndex = currentIndex + 1
							table.insert(indicies, currentIndex)
						end
					end
					
					for i = 1, currentIndex, 1 do
						local currentStack = {}
						local stackCount = 0
						-- Count how many we have in the i'nth stack
						for k, v in pairs(indicies) do
							if v == i then
								stackCount = stackCount + 1
								table.insert(currentStack, validChildren[k])
							end
						end
						
						table.sort(currentStack, function(a, b)
								return a.currentCD > b.currentCD
							end)
						
						-- Then set the correct data for the i'nth stack
						for k, icon in pairs(currentStack) do
							icon.stack = i
							local verticalOffset = height / stackCount
							local iconOffset = verticalOffset * (k - 1)
							local stackOffset = 0
							--local stackOffset = (height / 2)
							--icon:SetPoint("CENTER", 0, iconOffset - stackOffset  + (verticalOffset / 2) + generalIconOffset)						
							--icon.yOffset = iconOffset - stackOffset  + (verticalOffset / 2)
							
							if grow == "DOWN" then
								iconOffset = -iconOffset
							elseif grow == "CENTER" then
								--iconOffset = iconOffset - (height / 2)
								if stackCount == 1 then
									iconOffset = 0
								else
									local centerOffset = iconSize / stackCount
									iconOffset = iconOffset - (height / 2) + centerOffset
								end
								
								--CDTL2:Print(tostring(iconOffset))
							end
							
							icon.yOffset = iconOffset
							
							local levelMultiplier = 5
							local baseLevel = k * levelMultiplier
							
							icon:SetFrameLevel(baseLevel)
							icon.bd:SetFrameLevel(baseLevel + 1)
							icon.hl:SetFrameLevel(baseLevel + 2)
						end
					end
				end
			--else
				--validChildren[1].stackOffset = 0
			--end
		elseif style == "OFFSET" then
			--[[table.sort(validChildren, function(a, b)
				--local _, _, _, ax, ay = a:GetPoint()
				--local _, _, _, bx, by = b:GetPoint()
				--return ax > bx
				
				local aX, _ = a:GetCenter()
				local bX, _ = b:GetCenter()
				return aX < bX
			end)]]--
			
			table.sort(validChildren, function(a, b)
				return a.currentCD > b.currentCD
			end)
			
			local offset = s["stacking"]["height"] / (count - 1)
			local height = s["stacking"]["height"]
			local grow = s["stacking"]["grow"]
			
			for k, child in ipairs(validChildren) do
				--if count > 1 then
					local o = (offset * k) - offset
					
					
					if grow == "DOWN" then
						o = -o
					elseif grow == "CENTER" then
						o = o - (height / 2)
					end
					
					child.yOffset = o
				--else
					--child.yOffset = 0
				--end
				
				child:SetFrameLevel(baseLevel * k)
				child.bd:SetFrameLevel(baseLevel * k + 1)
				child.hl:SetFrameLevel(baseLevel * k + 2)
			end
			
		elseif style == "STAGGERED" then
			
		else
			table.sort(validChildren, function(a, b)
				return a.currentCD > b.currentCD
			end)
		
			for k, child in ipairs(validChildren) do
				child:SetFrameLevel(baseLevel * k)
				child.bd:SetFrameLevel(baseLevel * k + 1)
				child.hl:SetFrameLevel(baseLevel * k + 2)
			end
		end
	else
		for k, child in ipairs(validChildren) do
			child.stack = 1
			child.yOffset = 0
			child:SetFrameLevel(baseLevel * k)
			child.bd:SetFrameLevel(baseLevel * k + 1)
			child.hl:SetFrameLevel(baseLevel * k + 2)
		end
	end
end

private.CalcTracking = function(f, s, t)	
	local position = 0

	if t == "GCD" then
		local start, duration, _, _ = GetSpellCooldown(8921)	-- Spell ID for Moonfire, but any spell without a cooldown can be used
		local cooldownMS, gcdMS = GetSpellBaseCooldown(8921)	-- Spell ID for Moonfire, but any spell without a cooldown can be used
		
		local timeLeft = (start + duration) - GetTime()
		position = ( timeLeft / (gcdMS / 1000))
		
	elseif t == "HEALTH" then
		local hpMax = UnitHealthMax("player")
		local hpCurrent = UnitHealth("player")
		
		if hpCurrent ~= 0 then
			local percent = hpCurrent / hpMax
			position = percent
		end	
		
	elseif t == "CLASS_POWER" then
		local pType = CDTL2.player["classPower"]
		local pMax = UnitPowerMax("player", pType)
		local pCurrent = UnitPower("player", pType)
		
		if pType then
			if pCurrent ~= 0 then
				local percent = pCurrent / pMax
				position = percent
			end
		end
		
	end
	
	return position
end

private.CreateLane = function(laneNumber)	
	local frameName = "CDTL2_Lane_"..laneNumber
	local f = CreateFrame("StatusBar", frameName, UIParent, BackdropTemplateMixin and "BackdropTemplate" or nil)
	
	f.number = laneNumber
	f.childCount = 0

	f.bg = f:CreateTexture(nil, "BACKGROUND")
	f.st = CreateFrame("Frame", frameName.."_ST", f, BackdropTemplateMixin and "BackdropTemplate" or nil)
	f.st:SetParent(f)
	f.st.bg = f.st:CreateTexture(nil, "BACKGROUND")
	f.bd = CreateFrame("Frame", frameName.."_BD", UIParent, BackdropTemplateMixin and "BackdropTemplate" or nil)
	f.bd:SetParent(f)
	
	f.db = CreateFrame("Frame", frameName.."_DB", UIParent, BackdropTemplateMixin and "BackdropTemplate" or nil)
	f.db:SetParent(f)
	f.db.bg = f.db:CreateTexture(nil, "BACKGROUND")
	f.db.text = f.db:CreateFontString(nil,"ARTWORK")
	
	-- MODE TEXT
	f.t1 = f:CreateFontString(nil,"ARTWORK")
	f.t2 = f:CreateFontString(nil,"ARTWORK")
	f.t3 = f:CreateFontString(nil,"ARTWORK")
	f.t4 = f:CreateFontString(nil,"ARTWORK")
	f.t5 = f:CreateFontString(nil,"ARTWORK")
	
	f.c1 = f:CreateFontString(nil,"ARTWORK")
	f.c2 = f:CreateFontString(nil,"ARTWORK")
	f.c3 = f:CreateFontString(nil,"ARTWORK")
	f.c4 = f:CreateFontString(nil,"ARTWORK")
	f.c5 = f:CreateFontString(nil,"ARTWORK")
	
	-- ON UPDATE
	f:HookScript("OnUpdate", function(self, elapsed)
		private.LaneUpdate(self, elapsed)
	end)
	
	-- DRAG AND DROP MOVEMENT
	f:RegisterForDrag("LeftButton")
	f:SetScript("OnDragStart", f.StartMoving)
	f:SetScript("OnDragStop", f.StopMovingOrSizing)
	
	CDTL2:RefreshLane(laneNumber)
	table.insert(CDTL2.lanes, f)
end

private.LaneUpdate = function(f, elapsed)
	local s = nil
	if f.number == 1 then
		s = CDTL2.db.profile.lanes["lane1"]
	elseif f.number == 2 then
		s = CDTL2.db.profile.lanes["lane2"]
	elseif f.number == 3 then
		s = CDTL2.db.profile.lanes["lane3"]
	end

	local children = { f:GetChildren() }
	local validChildren = {}
	
	local count = 0
	for _, child in ipairs(children) do
		if child.valid then
			count = count + 1
			table.insert(validChildren, child)
		end
	end
	
	--table.sort(validChildren, function(a, b)
	--	return a.currentCD > b.currentCD
	--end)
	
	private.CalcStacking(f, s, validChildren, count)
	
	-- STACKING
	--local baseLevel = 5
	--local tOffset = 0
	
	-- DYNAMIC
	--if s["stacking"]["style"] == "DYNAMIC" then
		
		
	-- STAGGERED
	--[[elseif s["stacking"]["style"] == "STAGGERED" then
		for k, child in ipairs(validChildren) do
			child.yOffset = tOffset
		
			-- FRAME LEVELS
			child:SetFrameLevel(baseLevel * k)
			child.bd:SetFrameLevel(baseLevel * k + 1)
			--child.hl:SetFrameLevel(baseLevel * k + 2)
		end]]--
	
	-- NONE
	--[[else
		for k, child in ipairs(validChildren) do
			child:SetFrameLevel(baseLevel * k)
			child.bd:SetFrameLevel(baseLevel * k + 1)
			--child.hl:SetFrameLevel(baseLevel * k + 2)
		end
	end]]--
	
	--[[for k, child in ipairs(validChildren) do
		if s["stacking"]["enabled"] then
			local tOffset = 0
			local sOffset = s["stacking"]["height"] / 2
			local vOffset = s["stacking"]["height"] / count
			
			-- DYNAMIC
			if s["stacking"]["style"] == "DYNAMIC" then
				
				
			-- STAGGERED
			elseif s["stacking"]["style"] == "STAGGERED" then
				local iOffset = vOffset * (k - 1)
				
				tOffset = iOffset - sOffset + (vOffset / 2)
			end
			
			child.yOffset = tOffset
			
			-- FRAME LEVELS
			child:SetFrameLevel(baseLevel * k)
			child.bd:SetFrameLevel(baseLevel * k + 1)
			--child.hl:SetFrameLevel(baseLevel * k + 2)
		end
	end]]--
	
	private.UpdateText(f, s)
	
	-- PRIMARY TRACKING
	if s["tracking"]["primaryTracking"] ~= "NONE" then
		local tValue = private.CalcTracking(f, s, s["tracking"]["primaryTracking"])
		f:SetValue(tValue)
	else
		f:SetValue(1)
	end
	
	-- SECONDARY TRACKING
	if s["tracking"]["secondaryTracking"] ~= "NONE" then
		local tValue = private.CalcTracking(f, s, s["tracking"]["secondaryTracking"])
		
		if tValue < 1 and tValue > 0 then
			local tPosition = (s["width"] - s["tracking"]["stWidth"]) * tValue
			
			f.st:ClearAllPoints()
			f.st:SetPoint("LEFT", tPosition, 0)
			
			f.st:SetAlpha(1)
		else
			f.st:SetAlpha(0)
		end
	else
		f.st:SetAlpha(0)
	end

	-- FRAME (UN)LOCKING
	if CDTL2.db.profile.global["unlockFrames"] then
		local _, _, relativeTo, xOfs, yOfs = f:GetPoint()
		
		s["relativeTo"] = relativeTo
		s["posX"] = xOfs
		s["posY"] = yOfs
	end
	
	f.childCount = count
	
	if s["enabled"] then
		CDTL2:Autohide(f)
	else
		f:SetAlpha(0)
	end
end

private.SetModeText = function(f, s)
	local st = s["modeText"]
	local mode = s["mode"]["type"]
	
	if mode == "LINEAR" then
		st["text1"]["text"] = "END"
		st["text1"]["pos"] = 0
		st["text1"]["used"] = true
		
		st["text2"]["text"] = "25%"
		st["text2"]["pos"] = 0.25
		st["text2"]["used"] = true
		
		st["text3"]["text"] = "50%"
		st["text3"]["pos"] = 0.5
		st["text3"]["used"] = true
		
		st["text4"]["text"] = "75%"
		st["text4"]["pos"] = 0.75
		st["text4"]["used"] = true
		
		st["text5"]["text"] = "100%"
		st["text5"]["pos"] = 1
		st["text5"]["used"] = true	
	elseif mode == "LINEAR_ABS" then
		local max = s["mode"]["linearAbs"]["max"]
		st["text1"]["text"] = "END"
		st["text1"]["pos"] = 0
		st["text1"]["used"] = true
		
		st["text2"]["text"] = CDTL2:GetReadableTime(max * 0.25)
		st["text2"]["pos"] = 0.25
		st["text2"]["used"] = true
		
		st["text3"]["text"] = CDTL2:GetReadableTime(max * 0.5)
		st["text3"]["pos"] = 0.5
		st["text3"]["used"] = true
		
		st["text4"]["text"] = CDTL2:GetReadableTime(max * 0.75)
		st["text4"]["pos"] = 0.75
		st["text4"]["used"] = true
		
		st["text5"]["text"] = CDTL2:GetReadableTime(max)
		st["text5"]["pos"] = 1
		st["text5"]["used"] = true
	elseif mode == "SPLIT" then
		st["text1"]["text"] = "END"
		st["text2"]["text"] = s["mode"]["split"]["s1v"].."%"
		st["text2"]["pos"] = s["mode"]["split"]["s1p"]
		st["text3"]["text"] = s["mode"]["split"]["s2v"].."%"
		st["text3"]["pos"] = s["mode"]["split"]["s2p"]
		st["text4"]["text"] = s["mode"]["split"]["s3v"].."%"
		st["text4"]["pos"] = s["mode"]["split"]["s3p"]
		st["text5"]["text"] = "100%"
		
		if s["mode"]["split"]["count"] == 1 then
			st["text2"]["used"] = true
			st["text3"]["used"] = false
			st["text4"]["used"] = false
		elseif s["mode"]["split"]["count"] == 2 then
			st["text2"]["used"] = true
			st["text3"]["used"] = true
			st["text4"]["used"] = false
		elseif s["mode"]["split"]["count"] == 3 then
			st["text2"]["used"] = true
			st["text3"]["used"] = true
			st["text4"]["used"] = true
		end
	elseif mode == "SPLIT_ABS" then
		local max = s["mode"]["splitAbs"]["max"]
		st["text1"]["text"] = "END"
		st["text2"]["text"] = CDTL2:GetReadableTime(s["mode"]["splitAbs"]["s1v"])
		st["text2"]["pos"] = s["mode"]["splitAbs"]["s1p"]
		st["text3"]["text"] = CDTL2:GetReadableTime(s["mode"]["splitAbs"]["s2v"])
		st["text3"]["pos"] = s["mode"]["splitAbs"]["s2p"]
		st["text4"]["text"] = CDTL2:GetReadableTime(s["mode"]["splitAbs"]["s3v"])
		st["text4"]["pos"] = s["mode"]["splitAbs"]["s3p"]
		st["text5"]["text"] = CDTL2:GetReadableTime(max)
		
		if s["mode"]["splitAbs"]["count"] == 1 then
			st["text2"]["used"] = true
			st["text3"]["used"] = false
			st["text4"]["used"] = false
		elseif s["mode"]["splitAbs"]["count"] == 2 then
			st["text2"]["used"] = true
			st["text3"]["used"] = true
			st["text4"]["used"] = false
		elseif s["mode"]["splitAbs"]["count"] == 3 then
			st["text2"]["used"] = true
			st["text3"]["used"] = true
			st["text4"]["used"] = true
		end
	end
end

private.UpdateText = function(f, s)
	for i = 1, 10, 1 do
		local tSettings = nil
		local tObject = nil
	   
		if i == 1 then
			tSettings = s["modeText"]["text1"]
			tObject = f.t1
		elseif i == 2 then
			tSettings = s["modeText"]["text2"]
			tObject = f.t2
		elseif i == 3 then
			tSettings = s["modeText"]["text3"]
			tObject = f.t3
		elseif i == 4 then
			tSettings = s["modeText"]["text4"]
			tObject = f.t4
		elseif i == 5 then
			tSettings = s["modeText"]["text5"]
			tObject = f.t5
		elseif i == 6 then
			tSettings = s["customText"]["text1"]
			tObject = f.c1
		elseif i == 7 then
			tSettings = s["customText"]["text2"]
			tObject = f.c2
		elseif i == 8 then
			tSettings = s["customText"]["text3"]
			tObject = f.c3
		elseif i == 9 then
			tSettings = s["customText"]["text4"]
			tObject = f.c4
		elseif i == 10 then
			tSettings = s["customText"]["text5"]
			tObject = f.c5
		end
		
		tObject:SetText(CDTL2:ConvertTextTags(tSettings["text"], f))
	end
end