local private = {}

private.GetChangelog = function()
	local changeLog = ""
	changeLog = changeLog.."\n"
	changeLog = changeLog.."*This new version is very experimental right now, expect bugs/weird behaviour*\n\n"
	changeLog = changeLog.."This is a testing version, and it should run independently of CooldownTimeine\n"
	changeLog = changeLog.."Type /CDTL2 for options\n\n\n"
	changeLog = changeLog.."Changelog 2.5.4-1b1:\n\n"
	changeLog = changeLog.."  - Almost completely re-written the addon\n"
	changeLog = changeLog.."  - Now comes with a few default sounds and textures, and a new default font\n"
	changeLog = changeLog.."  - You can enable/disable 3 of each lanes(previously timeline)/ready/bars\n"
	changeLog = changeLog.."  - Each can be configured independently to show exactly what you want, how you want it\n"
	changeLog = changeLog.."  - Filters have been completely re-designed to aid with this\n"
	changeLog = changeLog.."  - Support for Classic and Classic Era at the same time\n"
	changeLog = changeLog.."\n\n"
	changeLog = changeLog.."Changelog 2.5.4-1b2:\n\n"
	changeLog = changeLog.."  - Fixed an issue in which debug info showed up on icons\n"
	changeLog = changeLog.."  - Fixed an issue in which debug disabled bar/ready frames would not disappear until /reload\n"
	changeLog = changeLog.."  - Fixed an issue in which tag info would not show up when mousing over text inputs\n"
	changeLog = changeLog.."  - Added short format name tag to the description when mousing over text inputs\n"
	changeLog = changeLog.."  - I somehow forgot to add the options for bar foreground/background textures and colours, so they are now there\n"
	changeLog = changeLog.."\n\n"
	changeLog = changeLog.."Known Issues:\n\n"
	changeLog = changeLog.."  - Alignments of bars is slightly out, and becomes pronounced with multiple bars\n"
	changeLog = changeLog.."  - Grouped icon stacking when 'centered' isnt working correctly\n"
	changeLog = changeLog.."  - Cooldowns that only start after aura disappears(eg Presence of Mind) are not detected on /reload if the aura is still active\n"
		
	return changeLog
end

function CDTL2:GetMainOptions()
	local options = {
		name = "CDTL2",
		type = "group",
		childGroups  = "tab",
		args = {
			global = {
				name = "Global",
				type = "group",
				order = 100,
				childGroups  = "tab",
				args = {
					unlockFrames = {
						name = "Unlock Frames",
						desc = "Unlock frames to allow drag and drop movement",
						order = 100,
						type = "toggle",
						width = "full",
						get = function(info, index)
								return CDTL2.db.profile.global["unlockFrames"]
							end,
						set = function(info, val)
								CDTL2:ToggleFrameLock()
							end,
					},
					autohide = {
						name = "Auto-hide Frames",
						desc = "If selected frames will hide themselves when there is no active bar/icon in it",
						order = 200,
						type = "toggle",
						width = "full",
						get = function(info, index)
								return CDTL2.db.profile.global["autohide"]
							end,
						set = function(info, val)
								CDTL2.db.profile.global["autohide"] = val
							end,
					},
					enableTooltip = {
						name = "Enable tooltips",
						desc = "If selected mousing over icons will display a tooltip",
						order = 300,
						type = "toggle",
						width = "full",
						get = function(info, index)
								return CDTL2.db.profile.global["enableTooltip"]
							end,
						set = function(info, val)
								CDTL2.db.profile.global["enableTooltip"] = val
							end,
					},
					spacer500 = {
						name = "\n\n",
						type = "description",
						order = 500,
					},
					zoom = {
						name = "Icon Zoom",
						desc = "Sets the zoom level of icons",
						order = 501,
						type = "range",
						softMin = 1,
						softMax = 1.5,
						get = function(info, index)
								return CDTL2.db.profile.global["zoom"]
							end,
						set = function(info, val)
								CDTL2.db.profile.global["zoom"] = val
								CDTL2:RefreshAllIcons()
								CDTL2:RefreshAllBars()
							end,
					},
				},
			},
			changelog = {
				name = "Changelog",
				type = "group",
				order = 200,
				childGroups  = "tab",
				args = {
					spacer100 = {
						name = function() return "CDTL2 v."..CDTL2.version.."\n\n" end,
						fontSize = "large",
						type = "description",
						order = 100,
					},
					spacer200 = {
						name = function(info)
							return private.GetChangelog()
						end,
						type = "description",
						order = 200,
					},
				},
			},
		},
	}
	
	return options
end

function CDTL2:GetFilterOptions()
	CDTL2.filterList = {}
	CDTL2.currentFilter = {
		default = "",
		spells = "",
		items = "",
		buffs = "",
		debuffs = "",
		offensives = "",
		petSpells = "",
	}
	CDTL2.currentFilterHidden = {
		default = true,
		spells = true,
		items = true,
		buffs = true,
		debuffs = true,
		offensives = true,
		petSpells = true,
	}

	local options = {
		name = "Filters",
		--handler = CDTL2,
		type = "group",
		childGroups  = "tab",
		args = {
			defaults = {
				name = "Defaults",
				type = "group",
				order = 100,
				args = {
					spacer100 = {
						name = "\n\nSelect a type to edit its default settings\n",
						type = "description",
						order = 100,
					},
					list = {
						name = "",
						desc = "",
						order = 101,
						type = "select",
						values = {
								["SPELLS"] = "Spells",
								["ITEMS"] = "Items",
								["BUFFS"] = "Buffs",
								["DEBUFFS"] = "Debufs",
								["OFFENSIVES"] = "Offensives",
								["PETSPELLS"] = "Pet Spells",
							},
						get = function(info)
								return CDTL2.currentFilter["default"]
							end,
						set = function(info, val)
								CDTL2.currentFilter["default"] = val
								CDTL2.currentFilterHidden["default"] = false
							end,
					},
					spacer200 = {
						name = "\n\n",
						type = "description",
						order = 200,
					},
					enable = {
						name = "Enabled",
						desc = "If unselected cooldowns of this type will be ignored",
						order = 201,
						type = "toggle",
						width = "full",
						hidden = function(info)
								return CDTL2.currentFilterHidden["default"]
							end,
						get = function(info, index)
								local default = CDTL2.currentFilter["default"]
								if default == "SPELLS" then
									return CDTL2.db.profile.global["spells"]["enabled"]
								elseif default == "ITEMS" then
									return CDTL2.db.profile.global["items"]["enabled"]
								elseif default == "BUFFS" then
									return CDTL2.db.profile.global["buffs"]["enabled"]
								elseif default == "DEBUFFS" then
									return CDTL2.db.profile.global["debuffs"]["enabled"]
								elseif default == "OFFENSIVES" then
									return CDTL2.db.profile.global["offensives"]["enabled"]
								elseif default == "PETSPELLS" then
									return CDTL2.db.profile.global["petSpells"]["enabled"]
								end
								
								return nil
							end,
						set = function(info, val)
								CDTL2.db.profile.global["spells"]["enabled"] = val
								
								local default = CDTL2.currentFilter["default"]
								if default == "SPELLS" then
									CDTL2.db.profile.global["spells"]["enabled"] = val
								elseif default == "ITEMS" then
									CDTL2.db.profile.global["items"]["enabled"] = val
								elseif default == "BUFFS" then
									CDTL2.db.profile.global["buffs"]["enabled"] = val
								elseif default == "DEBUFFS" then
									CDTL2.db.profile.global["debuffs"]["enabled"] = val
								elseif default == "OFFENSIVES" then
									CDTL2.db.profile.global["offensives"]["enabled"] = val
								elseif default == "PETSPELLS" then
									CDTL2.db.profile.global["petSpells"]["enabled"] = val
								end
								
								for _, cd in pairs(CDTL2.cooldowns) do
									if cd.data["type"] == "spells" then
										cd.data["enabled"] = val
									end
								end
							end,
					},
					--[[spacer300 = {
						name = "",
						type = "description",
						order = 300,
					},]]
					showSpellsDefault = {
						name = "Show by Default",
						desc = "If unselected you will need to manually enable each newly detected cooldown in order for them to show as bars/icons",
						order = 202,
						type = "toggle",
						width = "full",
						hidden = function(info)
								return CDTL2.currentFilterHidden["default"]
							end,
						get = function(info, index)
								local default = CDTL2.currentFilter["default"]
								if default == "SPELLS" then
									return CDTL2.db.profile.global["spells"]["showByDefault"]
								elseif default == "ITEMS" then
									return CDTL2.db.profile.global["items"]["showByDefault"]
								elseif default == "BUFFS" then
									return CDTL2.db.profile.global["buffs"]["showByDefault"]
								elseif default == "DEBUFFS" then
									return CDTL2.db.profile.global["debuffs"]["showByDefault"]
								elseif default == "OFFENSIVES" then
									return CDTL2.db.profile.global["offensives"]["showByDefault"]
								elseif default == "PETSPELLS" then
									return CDTL2.db.profile.global["petSpells"]["showByDefault"]
								end
								
								return nil
							end,
						set = function(info, val)
								CDTL2.db.profile.global["spells"]["showByDefault"] = val
								
								local default = CDTL2.currentFilter["default"]
								if default == "SPELLS" then
									CDTL2.db.profile.global["spells"]["showByDefault"] = val
								elseif default == "ITEMS" then
									CDTL2.db.profile.global["items"]["showByDefault"] = val
								elseif default == "BUFFS" then
									CDTL2.db.profile.global["buffs"]["showByDefault"] = val
								elseif default == "DEBUFFS" then
									CDTL2.db.profile.global["debuffs"]["showByDefault"] = val
								elseif default == "OFFENSIVES" then
									CDTL2.db.profile.global["offensives"]["showByDefault"] = val
								elseif default == "PETSPELLS" then
									CDTL2.db.profile.global["petSpells"]["showByDefault"] = val
								end
							end,
					},
					spacer300 = {
						name = "\n\n",
						type = "description",
						order = 300,
						hidden = function(info)
								return CDTL2.currentFilterHidden["default"]
							end,
					},
					defaultLane = {
						name = "Default Lane",
						desc = function(info)
								return "Set the default lane all new "..string.lower(CDTL2.currentFilter["default"]).." icons will use"
							end,
						order = 301,
						type = "select",
						hidden = function(info)
								return CDTL2.currentFilterHidden["default"]
							end,
						width = "half",
						values = { [0] = "Hide", [1] = "1", [2] = "2", [3] = "3" },
						get = function(info)
								local default = CDTL2.currentFilter["default"]
								if default == "SPELLS" then
									return CDTL2.db.profile.global["spells"]["defaultLane"]
								elseif default == "ITEMS" then
									return CDTL2.db.profile.global["items"]["defaultLane"]
								elseif default == "BUFFS" then
									return CDTL2.db.profile.global["buffs"]["defaultLane"]
								elseif default == "DEBUFFS" then
									return CDTL2.db.profile.global["debuffs"]["defaultLane"]
								elseif default == "OFFENSIVES" then
									return CDTL2.db.profile.global["offensives"]["defaultLane"]
								elseif default == "PETSPELLS" then
									return CDTL2.db.profile.global["petSpells"]["defaultLane"]
								end
								
								return nil
							end,
						set = function(info, val)
								local default = CDTL2.currentFilter["default"]
								if default == "SPELLS" then
									CDTL2.db.profile.global["spells"]["defaultLane"] = val
								elseif default == "ITEMS" then
									CDTL2.db.profile.global["items"]["defaultLane"] = val
								elseif default == "BUFFS" then
									CDTL2.db.profile.global["buffs"]["defaultLane"] = val
								elseif default == "DEBUFFS" then
									CDTL2.db.profile.global["debuffs"]["defaultLane"] = val
								elseif default == "OFFENSIVES" then
									CDTL2.db.profile.global["offensives"]["defaultLane"] = val
								elseif default == "PETSPELLS" then
									CDTL2.db.profile.global["petSpells"]["defaultLane"] = val
								end
							end,
					},
					setDefaultLane = {
						name = "Set All",
						desc = function(info)
								return "This will override current settings, and set all "..string.lower(CDTL2.currentFilter["default"]).." cooldowns to the default value"
							end,
						confirm = true,
						order = 302,
						type = "execute",hidden = function(info)
								return CDTL2.currentFilterHidden["default"]
							end,
						width = "half",
						func = function(info)
								local default = CDTL2.currentFilter["default"]
								
								for _, spell in pairs(CDTL2.cooldowns) do
									if spell.data["type"] == string.lower(default) then
										local default = CDTL2.currentFilter["default"]
										if default == "SPELLS" then
											spell.data["type"] = CDTL2.db.profile.global["spells"]["defaultLane"]
										elseif default == "ITEMS" then
											spell.data["type"] = CDTL2.db.profile.global["items"]["defaultLane"]
										elseif default == "BUFFS" then
											spell.data["type"] = CDTL2.db.profile.global["buffs"]["defaultLane"]
										elseif default == "DEBUFFS" then
											spell.data["type"] = CDTL2.db.profile.global["debuffs"]["defaultLane"]
										elseif default == "OFFENSIVES" then
											spell.data["type"] = CDTL2.db.profile.global["offensives"]["defaultLane"]
										elseif default == "PETSPELLS" then
											spell.data["type"] = CDTL2.db.profile.global["petSpells"]["defaultLane"]
										end
									end
								end
								
								for _, spell in pairs(CDTL2.db.profile.tables[string.lower(default)]) do
									spell["lane"] = CDTL2.db.profile.global[string.lower(default)]["defaultLane"]
								end
							end
					},
					spacer310 = {
						name = "\n",
						type = "description",
						order = 310,
						hidden = function(info)
								return CDTL2.currentFilterHidden["default"]
							end,
					},
					defaultBar = {
						name = "Default Bar",
						desc = function(info)
								return "Set the default bar frame all new "..string.lower(CDTL2.currentFilter["default"]).." will use"
							end,
						order = 311,
						type = "select",
						hidden = function(info)
								return CDTL2.currentFilterHidden["default"]
							end,
						width = "half",
						values = { [1] = "1", [2] = "2", [3] = "3" },
						get = function(info)
								local default = CDTL2.currentFilter["default"]
								if default == "SPELLS" then
									return CDTL2.db.profile.global["spells"]["defaultBar"]
								elseif default == "ITEMS" then
									return CDTL2.db.profile.global["items"]["defaultBar"]
								elseif default == "BUFFS" then
									return CDTL2.db.profile.global["buffs"]["defaultBar"]
								elseif default == "DEBUFFS" then
									return CDTL2.db.profile.global["debuffs"]["defaultBar"]
								elseif default == "OFFENSIVES" then
									return CDTL2.db.profile.global["offensives"]["defaultBar"]
								elseif default == "PETSPELLS" then
									return CDTL2.db.profile.global["petSpells"]["defaultBar"]
								end
								
								return nil
							end,
						set = function(info, val)
								local default = CDTL2.currentFilter["default"]
								if default == "SPELLS" then
									CDTL2.db.profile.global["spells"]["defaultBar"] = val
								elseif default == "ITEMS" then
									CDTL2.db.profile.global["items"]["defaultBar"] = val
								elseif default == "BUFFS" then
									CDTL2.db.profile.global["buffs"]["defaultBar"] = val
								elseif default == "DEBUFFS" then
									CDTL2.db.profile.global["debuffs"]["defaultBar"] = val
								elseif default == "OFFENSIVES" then
									CDTL2.db.profile.global["offensives"]["defaultBar"] = val
								elseif default == "PETSPELLS" then
									CDTL2.db.profile.global["petSpells"]["defaultBar"] = val
								end
							end,
					},
					setDefaultBar = {
						name = "Set All",
						desc = function(info)
								return "This will override current settings, and set all "..string.lower(CDTL2.currentFilter["default"]).." cooldowns to the default value"
							end,
						confirm = true,
						order = 312,
						type = "execute",
						hidden = function(info)
								return CDTL2.currentFilterHidden["default"]
							end,
						width = "half",
						func = function(info)
								local default = CDTL2.currentFilter["default"]
								
								for _, spell in pairs(CDTL2.cooldowns) do
									if spell.data["type"] == string.lower(default) then
										local default = CDTL2.currentFilter["default"]
										if default == "SPELLS" then
											spell.data["type"] = CDTL2.db.profile.global["spells"]["defaultBar"]
										elseif default == "ITEMS" then
											spell.data["type"] = CDTL2.db.profile.global["items"]["defaultBar"]
										elseif default == "BUFFS" then
											spell.data["type"] = CDTL2.db.profile.global["buffs"]["defaultBar"]
										elseif default == "DEBUFFS" then
											spell.data["type"] = CDTL2.db.profile.global["debuffs"]["defaultBar"]
										elseif default == "OFFENSIVES" then
											spell.data["type"] = CDTL2.db.profile.global["offensives"]["defaultBar"]
										elseif default == "PETSPELLS" then
											spell.data["type"] = CDTL2.db.profile.global["petSpells"]["defaultBar"]
										end
									end
								end
								
								for _, spell in pairs(CDTL2.db.profile.tables[string.lower(default)]) do
									spell["barFrame"] = CDTL2.db.profile.global[string.lower(default)]["defaultBar"]
								end
							end
					},
					spacer320 = {
						name = "\n",
						type = "description",
						order = 320,
						hidden = function(info)
								return CDTL2.currentFilterHidden["default"]
							end,
					},
					defaultReady = {
						name = "Default Ready",
						desc = function(info)
								return "Set the default ready frame all new "..string.lower(CDTL2.currentFilter["default"]).." icons will use"
							end,
						order = 321,
						hidden = function(info)
								return CDTL2.currentFilterHidden["default"]
							end,
						type = "select",
						hidden = function(info)
								return CDTL2.currentFilterHidden["default"]
							end,
						width = "half",
						values = { [1] = "1", [2] = "2", [3] = "3" },
						get = function(info)
								local default = CDTL2.currentFilter["default"]
								if default == "SPELLS" then
									return CDTL2.db.profile.global["spells"]["defaultReady"]
								elseif default == "ITEMS" then
									return CDTL2.db.profile.global["items"]["defaultReady"]
								elseif default == "BUFFS" then
									return CDTL2.db.profile.global["buffs"]["defaultReady"]
								elseif default == "DEBUFFS" then
									return CDTL2.db.profile.global["debuffs"]["defaultReady"]
								elseif default == "OFFENSIVES" then
									return CDTL2.db.profile.global["offensives"]["defaultReady"]
								elseif default == "PETSPELLS" then
									return CDTL2.db.profile.global["petSpells"]["defaultReady"]
								end
								
								return nil
							end,
						set = function(info, val)
								local default = CDTL2.currentFilter["default"]
								if default == "SPELLS" then
									CDTL2.db.profile.global["spells"]["defaultReady"] = val
								elseif default == "ITEMS" then
									CDTL2.db.profile.global["items"]["defaultReady"] = val
								elseif default == "BUFFS" then
									CDTL2.db.profile.global["buffs"]["defaultReady"] = val
								elseif default == "DEBUFFS" then
									CDTL2.db.profile.global["debuffs"]["defaultReady"] = val
								elseif default == "OFFENSIVES" then
									CDTL2.db.profile.global["offensives"]["defaultReady"] = val
								elseif default == "PETSPELLS" then
									CDTL2.db.profile.global["petSpells"]["defaultReady"] = val
								end
							end,
					},
					setDefaultReady = {
						name = "Set All",
						desc = function(info)
								return "This will override current settings, and set all "..string.lower(CDTL2.currentFilter["default"]).." cooldowns to the default value"
							end,
						confirm = true,
						order = 322,
						type = "execute",hidden = function(info)
								return CDTL2.currentFilterHidden["default"]
							end,
						width = "half",
						func = function(info)
								local default = CDTL2.currentFilter["default"]
								
								for _, spell in pairs(CDTL2.cooldowns) do
									if spell.data["type"] == string.lower(default) then
										local default = CDTL2.currentFilter["default"]
										if default == "SPELLS" then
											spell.data["type"] = CDTL2.db.profile.global["spells"]["defaultReady"]
										elseif default == "ITEMS" then
											spell.data["type"] = CDTL2.db.profile.global["items"]["defaultReady"]
										elseif default == "BUFFS" then
											spell.data["type"] = CDTL2.db.profile.global["buffs"]["defaultReady"]
										elseif default == "DEBUFFS" then
											spell.data["type"] = CDTL2.db.profile.global["debuffs"]["defaultReady"]
										elseif default == "OFFENSIVES" then
											spell.data["type"] = CDTL2.db.profile.global["offensives"]["defaultReady"]
										elseif default == "PETSPELLS" then
											spell.data["type"] = CDTL2.db.profile.global["petSpells"]["defaultReady"]
										end
									end
								end
								
								for _, spell in pairs(CDTL2.db.profile.tables[string.lower(default)]) do
									spell["readyFrame"] = CDTL2.db.profile.global[string.lower(default)]["defaultReady"]
								end
							end
					},
				},
			},
			spells = {
				name = "Spells",
				type = "group",
				order = 200,
				args = {
					spacer100 = {
						name = "\n\nSelect a spell to edit its settings\n",
						type = "description",
						order = 100,
					},
					list = {
						name = "",
						desc = "",
						order = 101,
						type = "select",
						values = function(info)
								local list = CDTL2:LoadFilterList("spells")
								return list
							end,
						get = function(info)
								return CDTL2.currentFilter["spells"]
							end,
						set = function(info, val)
								local list = CDTL2:LoadFilterList("spells")
								CDTL2.currentFilter["spells"] = list[val]
								CDTL2.currentFilterHidden["spells"] = false
							end,
					},
					spacer200 = {
						name = "*Spells will show up here once they have been detected at least once*\n\n\n",
						type = "description",
						order = 200,
					},
					spacer201 = {
						name = function(info)
								local s = ""
								
								if CDTL2.currentFilter["spells"] then
									s = CDTL2.currentFilter["spells"]
								end
								
								return s
							end,
						type = "description",
						order = 201,
						fontSize = "large",
						hidden = function(info)
								return CDTL2.currentFilterHidden["spells"]
							end,
						image = function(info)
							local s = CDTL2:GetSpellSettings(CDTL2.currentFilter["spells"], "spells")
							
							if s then
								return s["icon"]
							else
								return 134400
							end
						end,
						imageWidth = 24,
						imageHeight = 24,
						imageCoords = {0.1,0.9,0.1,0.9},
					},
					spacer202 = {
						name = function(info)
								local n = "\n\n"
								local s = CDTL2:GetSpellSettings(CDTL2.currentFilter["spells"], "spells")
								
								if s then
									n =	n.."Cooldown: "..(s["bCD"] / 1000).." sec\n"
								end
								
								return n
							end,
						type = "description",
						order = 202,
						hidden = function(info)
								return CDTL2.currentFilterHidden["spells"]
							end,
						fontSize = "medium",
					},	
					spacer300 = {
						name = "\n\n",
						type = "description",
						order = 300,
						hidden = function(info)
								return CDTL2.currentFilterHidden["spells"]
							end,
					},
					enabled = {
						name = "Enabled - Untick this to never show this as an icon/bar",
						desc = "",
						order = 301,
						type = "toggle",
						width = "full",
						hidden = function(info)
								return CDTL2.currentFilterHidden["spells"]
							end,
						get = function(info, index)
								local s = CDTL2:GetSpellSettings(CDTL2.currentFilter["spells"], "spells")
								
								if s then
									return s["enabled"]
								else
									return nil
								end
							end,
						set = function(info, val)
								CDTL2:SetSpellData(CDTL2.currentFilter["spells"], "spells", "enabled", val)
								
								local s = CDTL2:GetExistingCooldown(CDTL2.currentFilter["spells"], "spells")
								if s then
									s.data["enabled"] = val
									
									CDTL2:RefreshIcon(s)
									CDTL2:RefreshBar(s)
								end
							end,
					},					
					spacer400 = {
						name = "\n\n",
						type = "description",
						order = 400,
						hidden = function(info)
								return CDTL2.currentFilterHidden["spells"]
							end,
					},
					highlight = {
						name = "Highlight - Tick to apply highlighting to the icon",
						desc = "",
						order = 401,
						type = "toggle",
						width = "full",
						hidden = function(info)
								return CDTL2.currentFilterHidden["spells"]
							end,
						get = function(info, index)
								local s = CDTL2:GetSpellSettings(CDTL2.currentFilter["spells"], "spells")
								
								if s then
									return s["highlight"]
								else
									return nil
								end
							end,
						set = function(info, val)
								CDTL2:SetSpellData(CDTL2.currentFilter["spells"], "spells", "highlight", val)
								--CDTL2:RefreshAllIcons()
								
								local s = CDTL2:GetExistingCooldown(CDTL2.currentFilter["spells"], "spells")
								if s then
									s.data["highlight"] = val
									
									CDTL2:RefreshIcon(s)
									--CDTL2:RefreshBar(s)
								end
							end,
					},
					pinned = {
						name = "Pinned - Tick to keep an icon in the ready frame until the spell is used again",
						desc = "",
						order = 402,
						type = "toggle",
						width = "full",
						hidden = function(info)
								return CDTL2.currentFilterHidden["spells"]
							end,
						get = function(info, index)
								local s = CDTL2:GetSpellSettings(CDTL2.currentFilter["spells"], "spells")
								
								if s then
									return s["pinned"]
								else
									return nil
								end
							end,
						set = function(info, val)
								CDTL2:SetSpellData(CDTL2.currentFilter["spells"], "spells", "pinned", val)
								
								local s = CDTL2:GetExistingCooldown(CDTL2.currentFilter["spells"], "spells")
								if s then
									s.data["pinned"] = val
									
									CDTL2:RefreshIcon(s)
									--CDTL2:RefreshBar(s)
								end
							end,
					},
					spacer500 = {
						name = "\n\nSelect in which frame the icon/bar will be shown\nIf a the selected frame is not enabled, the icon/bar will default to 1\n\n",
						type = "description",
						order = 500,
						hidden = function(info)
								return CDTL2.currentFilterHidden["spells"]
							end,
					},
					lane = {
						name = "Lane",
						desc = "",
						order = 501,
						type = "select",
						hidden = function(info)
								return CDTL2.currentFilterHidden["spells"]
							end,
						width = "half",
						values = { [0] = "Hide", [1] = "1", [2] = "2", [3] = "3" },
						get = function(info)
								local s = CDTL2:GetSpellSettings(CDTL2.currentFilter["spells"], "spells")
								
								if s then
									return s["lane"]
								else
									return nil
								end
							end,
						set = function(info, val)
								CDTL2:SetSpellData(CDTL2.currentFilter["spells"], "spells", "lane", tonumber(val))
								
								local s = CDTL2:GetExistingCooldown(CDTL2.currentFilter["spells"], "spells")
								if s then
									s.data["lane"] = val
									
									CDTL2:SendToLane(s)
								end
							end,
					},
					barFrame = {
						name = "Bar Frame",
						desc = "",
						order = 502,
						type = "select",
						hidden = function(info)
								return CDTL2.currentFilterHidden["spells"]
							end,
						width = "half",
						values = { [0] = "Hide", [1] = "1", [2] = "2", [3] = "3" },
						get = function(info)
								local s = CDTL2:GetSpellSettings(CDTL2.currentFilter["spells"], "spells")
								
								if s then
									return s["barFrame"]
								else
									return nil
								end
							end,
						set = function(info, val)
								CDTL2:SetSpellData(CDTL2.currentFilter["spells"], "spells", "barFrame", tonumber(val))
								
								local s = CDTL2:GetExistingCooldown(CDTL2.currentFilter["spells"], "spells")
								if s then
									s.data["barFrame"] = val
									
									CDTL2:SendToBarFrame(s)
								end
							end,
					},
					readyFrame = {
						name = "Ready Frame",
						desc = "",
						order = 503,
						hidden = function(info)
								return CDTL2.currentFilterHidden["spells"]
							end,
						type = "select",
						width = "half",
						values = { [0] = "Hide", [1] = "1", [2] = "2", [3] = "3" },
						get = function(info)
								local s = CDTL2:GetSpellSettings(CDTL2.currentFilter["spells"], "spells")
								
								if s then
									return s["readyFrame"]
								else
									return nil
								end
							end,
						set = function(info, val)
								CDTL2:SetSpellData(CDTL2.currentFilter["spells"], "spells", "readyFrame", tonumber(val))
								
								local s = CDTL2:GetExistingCooldown(CDTL2.currentFilter["spells"], "spells")
								if s then
									s.data["readyFrame"] = val
									
									CDTL2:SendToReady(s)
								end
							end,
					},
				},
			},
			items = {
				name = "Items",
				type = "group",
				order = 300,
				args = {
					spacer100 = {
						name = "\n\nSelect an item spell to edit its settings\n",
						type = "description",
						order = 100,
					},
					list = {
						name = "",
						desc = "",
						order = 101,
						type = "select",
						values = function(info)
								local list = CDTL2:LoadFilterList("items")
								return list
							end,
						get = function(info)
								return CDTL2.currentFilter["items"]
							end,
						set = function(info, val)
								local list = CDTL2:LoadFilterList("items")
								CDTL2.currentFilter["items"] = list[val]
								CDTL2.currentFilterHidden["items"] = false
							end,
					},
					spacer200 = {
						name = "*Item spells will show up here once they have been detected at least once*\n\n\n",
						type = "description",
						order = 200,
					},
					spacer201 = {
						name = function(info)
								local s = ""
								
								if CDTL2.currentFilter["items"] then
									s = CDTL2.currentFilter["items"]
								end
								
								return s
							end,
						type = "description",
						order = 201,
						fontSize = "large",
						hidden = function(info)
								return CDTL2.currentFilterHidden["items"]
							end,
						image = function(info)
							local s = CDTL2:GetSpellSettings(CDTL2.currentFilter["items"], "items")
							
							if s then
								return s["icon"]
							else
								return 134400
							end
						end,
						imageWidth = 24,
						imageHeight = 24,
						imageCoords = {0.1,0.9,0.1,0.9},
					},
					spacer202 = {
						name = function(info)
								local n = "\n\n"
								local s = CDTL2:GetSpellSettings(CDTL2.currentFilter["items"], "items")
								
								if s then
									--n =	n.."From Item: "..s["itemName"].."\n"
									local itemLink = "Unknown"
									if s["link"] then
										itemLink = (s["link"])
									end
									
									n =	n.."From Item: "..itemLink.."\n"
									n =	n.."Cooldown: "..(s["bCD"] / 1000).." sec\n"
								end
								
								return n
							end,
						type = "description",
						order = 202,
						hidden = function(info)
								return CDTL2.currentFilterHidden["items"]
							end,
						fontSize = "medium",
					},	
					spacer300 = {
						name = "\n\n",
						type = "description",
						order = 300,
						hidden = function(info)
								return CDTL2.currentFilterHidden["items"]
							end,
					},
					enabled = {
						name = "Enabled - Untick this to never show this as an icon/bar",
						desc = "",
						order = 301,
						type = "toggle",
						width = "full",
						hidden = function(info)
								return CDTL2.currentFilterHidden["items"]
							end,
						get = function(info, index)
								local s = CDTL2:GetSpellSettings(CDTL2.currentFilter["items"], "items")
								
								if s then
									return s["enabled"]
								else
									return nil
								end
							end,
						set = function(info, val)
								CDTL2:SetSpellData(CDTL2.currentFilter["items"], "items", "enabled", val)
								
								local s = CDTL2:GetExistingCooldown(CDTL2.currentFilter["items"], "items")
								s.data["enabled"] = val
								
								CDTL2:RefreshIcon(s)
								CDTL2:RefreshBar(s)
							end,
					},					
					spacer400 = {
						name = "\n\n",
						type = "description",
						order = 400,
						hidden = function(info)
								return CDTL2.currentFilterHidden["items"]
							end,
					},
					highlight = {
						name = "Highlight - Tick to apply highlighting to the icon",
						desc = "",
						order = 401,
						type = "toggle",
						width = "full",
						hidden = function(info)
								return CDTL2.currentFilterHidden["items"]
							end,
						get = function(info, index)
								local s = CDTL2:GetSpellSettings(CDTL2.currentFilter["items"], "items")
								
								if s then
									return s["highlight"]
								else
									return nil
								end
							end,
						set = function(info, val)
								CDTL2:SetSpellData(CDTL2.currentFilter["items"], "items", "highlight", val)
								
								local s = CDTL2:GetExistingCooldown(CDTL2.currentFilter["items"], "items")
								if s then
									s.data["highlight"] = val
									
									CDTL2:RefreshIcon(s)
									--CDTL2:RefreshBar(s)
								end
							end,
					},
					pinned = {
						name = "Pinned - Tick to keep an icon in the ready frame until the item spell is used again",
						desc = "",
						order = 402,
						type = "toggle",
						width = "full",
						hidden = function(info)
								return CDTL2.currentFilterHidden["items"]
							end,
						get = function(info, index)
								local s = CDTL2:GetSpellSettings(CDTL2.currentFilter["items"], "items")
								
								if s then
									return s["pinned"]
								else
									return nil
								end
							end,
						set = function(info, val)
								CDTL2:SetSpellData(CDTL2.currentFilter["items"], "items", "pinned", val)
								
								local s = CDTL2:GetExistingCooldown(CDTL2.currentFilter["items"], "items")
								if s then
									s.data["pinned"] = val
									
									CDTL2:RefreshIcon(s)
									--CDTL2:RefreshBar(s)
								end
							end,
					},
					spacer500 = {
						name = "\n\nSelect in which frame the icon/bar will be shown\nIf a the selected frame is not enabled, the icon/bar will default to 1\n\n",
						type = "description",
						order = 500,
						hidden = function(info)
								return CDTL2.currentFilterHidden["items"]
							end,
					},
					lane = {
						name = "Lane",
						desc = "",
						order = 501,
						type = "select",
						hidden = function(info)
								return CDTL2.currentFilterHidden["items"]
							end,
						width = "half",
						values = { [0] = "Hide", [1] = "1", [2] = "2", [3] = "3" },
						get = function(info)
								local s = CDTL2:GetSpellSettings(CDTL2.currentFilter["items"], "items")
								
								if s then
									return s["lane"]
								else
									return nil
								end
							end,
						set = function(info, val)
								CDTL2:SetSpellData(CDTL2.currentFilter["items"], "items", "lane", tonumber(val))
								
								local s = CDTL2:GetExistingCooldown(CDTL2.currentFilter["items"], "items")
								if s then
									s.data["lane"] = val
									
									CDTL2:SendToLane(s)
								end
							end,
					},
					barFrame = {
						name = "Bar Frame",
						desc = "",
						order = 502,
						type = "select",
						hidden = function(info)
								return CDTL2.currentFilterHidden["items"]
							end,
						width = "half",
						values = { [0] = "Hide", [1] = "1", [2] = "2", [3] = "3" },
						get = function(info)
								local s = CDTL2:GetSpellSettings(CDTL2.currentFilter["items"], "items")
								
								if s then
									return s["barFrame"]
								else
									return nil
								end
							end,
						set = function(info, val)
								CDTL2:SetSpellData(CDTL2.currentFilter["items"], "items", "barFrame", tonumber(val))
								
								local s = CDTL2:GetExistingCooldown(CDTL2.currentFilter["items"], "items")
								if s then
									s.data["barFrame"] = val
									
									CDTL2:SendToBarFrame(s)
								end
							end,
					},
					readyFrame = {
						name = "Ready Frame",
						desc = "",
						order = 503,
						hidden = function(info)
								return CDTL2.currentFilterHidden["items"]
							end,
						type = "select",
						width = "half",
						values = { [0] = "Hide", [1] = "1", [2] = "2", [3] = "3" },
						get = function(info)
								local s = CDTL2:GetSpellSettings(CDTL2.currentFilter["items"], "items")
								
								if s then
									return s["readyFrame"]
								else
									return nil
								end
							end,
						set = function(info, val)
								CDTL2:SetSpellData(CDTL2.currentFilter["items"], "items", "readyFrame", tonumber(val))
								
								local s = CDTL2:GetExistingCooldown(CDTL2.currentFilter["items"], "items")
								if s then
									s.data["readyFrame"] = val
									
									CDTL2:SendToReady(s)
								end
							end,
					},
				},
			},
			buffs = {
				name = "Buffs",
				type = "group",
				order = 400,
				args = {
					spacer100 = {
						name = "\n\nSelect a buff to edit its settings\n",
						type = "description",
						order = 100,
					},
					list = {
						name = "",
						desc = "",
						order = 101,
						type = "select",
						values = function(info)
								local list = CDTL2:LoadFilterList("buffs")
								return list
							end,
						get = function(info)
								return CDTL2.currentFilter["buffs"]
							end,
						set = function(info, val)
								local list = CDTL2:LoadFilterList("buffs")
								CDTL2.currentFilter["buffs"] = list[val]
								CDTL2.currentFilterHidden["buffs"] = false
							end,
					},
					spacer200 = {
						name = "*Buffs will show up here once they have been detected at least once*\n\n\n",
						type = "description",
						order = 200,
					},
					spacer201 = {
						name = function(info)
								local s = ""
								
								if CDTL2.currentFilter["buffs"] then
									s = CDTL2.currentFilter["buffs"]
								end
								
								return s
							end,
						type = "description",
						order = 201,
						fontSize = "large",
						hidden = function(info)
								return CDTL2.currentFilterHidden["buffs"]
							end,
						image = function(info)
							local s = CDTL2:GetSpellSettings(CDTL2.currentFilter["buffs"], "buffs")
							
							if s then
								return s["icon"]
							else
								return 134400
							end
						end,
						imageWidth = 24,
						imageHeight = 24,
						imageCoords = {0.1,0.9,0.1,0.9},
					},
					spacer202 = {
						name = function(info)
								local n = "\n\n"
								local s = CDTL2:GetSpellSettings(CDTL2.currentFilter["buffs"], "buffs")
								
								if s then
									n =	n.."Duration: "..(s["bCD"] / 1000).." sec\n"
								end
								
								return n
							end,
						type = "description",
						order = 202,
						hidden = function(info)
								return CDTL2.currentFilterHidden["buffs"]
							end,
						fontSize = "medium",
					},	
					spacer300 = {
						name = "\n\n",
						type = "description",
						order = 300,
						hidden = function(info)
								return CDTL2.currentFilterHidden["buffs"]
							end,
					},
					enabled = {
						name = "Enabled - Untick this to never show this as an icon/bar",
						desc = "",
						order = 301,
						type = "toggle",
						width = "full",
						hidden = function(info)
								return CDTL2.currentFilterHidden["buffs"]
							end,
						get = function(info, index)
								local s = CDTL2:GetSpellSettings(CDTL2.currentFilter["buffs"], "buffs")
								
								if s then
									return s["enabled"]
								else
									return nil
								end
							end,
						set = function(info, val)
								CDTL2:SetSpellData(CDTL2.currentFilter["buffs"], "buffs", "enabled", val)
								
								local s = CDTL2:GetExistingCooldown(CDTL2.currentFilter["buffs"], "buffs")
								s.data["enabled"] = val
								
								CDTL2:RefreshIcon(s)
								CDTL2:RefreshBar(s)
							end,
					},					
					spacer400 = {
						name = "\n\n",
						type = "description",
						order = 400,
						hidden = function(info)
								return CDTL2.currentFilterHidden["buffs"]
							end,
					},
					highlight = {
						name = "Highlight - Tick to apply highlighting to the icon",
						desc = "",
						order = 401,
						type = "toggle",
						width = "full",
						hidden = function(info)
								return CDTL2.currentFilterHidden["buffs"]
							end,
						get = function(info, index)
								local s = CDTL2:GetSpellSettings(CDTL2.currentFilter["buffs"], "buffs")
								
								if s then
									return s["highlight"]
								else
									return nil
								end
							end,
						set = function(info, val)
								CDTL2:SetSpellData(CDTL2.currentFilter["buffs"], "buffs", "highlight", val)
								
								local s = CDTL2:GetExistingCooldown(CDTL2.currentFilter["buffs"], "buffs")
								if s then
									s.data["highlight"] = val
									
									CDTL2:RefreshIcon(s)
									--CDTL2:RefreshBar(s)
								end
							end,
					},
					pinned = {
						name = "Pinned - Tick to keep an icon in the ready frame until the buff is applied again",
						desc = "",
						order = 402,
						type = "toggle",
						width = "full",
						hidden = function(info)
								return CDTL2.currentFilterHidden["buffs"]
							end,
						get = function(info, index)
								local s = CDTL2:GetSpellSettings(CDTL2.currentFilter["buffs"], "buffs")
								
								if s then
									return s["pinned"]
								else
									return nil
								end
							end,
						set = function(info, val)
								CDTL2:SetSpellData(CDTL2.currentFilter["buffs"], "buffs", "pinned", val)
								
								local s = CDTL2:GetExistingCooldown(CDTL2.currentFilter["buffs"], "buffs")
								if s then
									s.data["pinned"] = val
									
									CDTL2:RefreshIcon(s)
									--CDTL2:RefreshBar(s)
								end
							end,
					},
					spacer500 = {
						name = "\n\nSelect in which frame the icon/bar will be shown\nIf a the selected frame is not enabled, the icon/bar will default to 1\n\n",
						type = "description",
						order = 500,
						hidden = function(info)
								return CDTL2.currentFilterHidden["buffs"]
							end,
					},
					lane = {
						name = "Lane",
						desc = "",
						order = 501,
						type = "select",
						hidden = function(info)
								return CDTL2.currentFilterHidden["buffs"]
							end,
						width = "half",
						values = { [0] = "Hide", [1] = "1", [2] = "2", [3] = "3" },
						get = function(info)
								local s = CDTL2:GetSpellSettings(CDTL2.currentFilter["buffs"], "buffs")
								
								if s then
									return s["lane"]
								else
									return nil
								end
							end,
						set = function(info, val)
								CDTL2:SetSpellData(CDTL2.currentFilter["buffs"], "buffs", "lane", tonumber(val))
								
								local s = CDTL2:GetExistingCooldown(CDTL2.currentFilter["buffs"], "buffs")
								if s then
									s.data["lane"] = val
									
									CDTL2:SendToLane(s)
								end
							end,
					},
					barFrame = {
						name = "Bar Frame",
						desc = "",
						order = 502,
						type = "select",
						hidden = function(info)
								return CDTL2.currentFilterHidden["buffs"]
							end,
						width = "half",
						values = { [0] = "Hide", [1] = "1", [2] = "2", [3] = "3" },
						get = function(info)
								local s = CDTL2:GetSpellSettings(CDTL2.currentFilter["buffs"], "buffs")
								
								if s then
									return s["barFrame"]
								else
									return nil
								end
							end,
						set = function(info, val)
								CDTL2:SetSpellData(CDTL2.currentFilter["buffs"], "buffs", "barFrame", tonumber(val))
								
								local s = CDTL2:GetExistingCooldown(CDTL2.currentFilter["buffs"], "buffs")
								if s then
									s.data["barFrame"] = val
									
									CDTL2:SendToBarFrame(s)
								end
							end,
					},
					readyFrame = {
						name = "Ready Frame",
						desc = "",
						order = 503,
						hidden = function(info)
								return CDTL2.currentFilterHidden["buffs"]
							end,
						type = "select",
						width = "half",
						values = { [0] = "Hide", [1] = "1", [2] = "2", [3] = "3" },
						get = function(info)
								local s = CDTL2:GetSpellSettings(CDTL2.currentFilter["buffs"], "buffs")
								
								if s then
									return s["readyFrame"]
								else
									return nil
								end
							end,
						set = function(info, val)
								CDTL2:SetSpellData(CDTL2.currentFilter["buffs"], "buffs", "readyFrame", tonumber(val))
								
								local s = CDTL2:GetExistingCooldown(CDTL2.currentFilter["buffs"], "buffs")
								if s then
									s.data["readyFrame"] = val
									
									CDTL2:SendToReady(s)
								end
							end,
					},
				},
			},
			debuffs = {
				name = "Debuffs",
				type = "group",
				order = 500,
				args = {
					spacer100 = {
						name = "\n\nSelect a debuff to edit its settings\n",
						type = "description",
						order = 100,
					},
					list = {
						name = "",
						desc = "",
						order = 101,
						type = "select",
						values = function(info)
								local list = CDTL2:LoadFilterList("debuffs")
								return list
							end,
						get = function(info)
								return CDTL2.currentFilter["debuffs"]
							end,
						set = function(info, val)
								local list = CDTL2:LoadFilterList("debuffs")
								CDTL2.currentFilter["debuffs"] = list[val]
								CDTL2.currentFilterHidden["debuffs"] = false
							end,
					},
					spacer200 = {
						name = "*Debuffs will show up here once they have been detected at least once*\n\n\n",
						type = "description",
						order = 200,
					},
					spacer201 = {
						name = function(info)
								local s = ""
								
								if CDTL2.currentFilter["debuffs"] then
									s = CDTL2.currentFilter["debuffs"]
								end
								
								return s
							end,
						type = "description",
						order = 201,
						fontSize = "large",
						hidden = function(info)
								return CDTL2.currentFilterHidden["debuffs"]
							end,
						image = function(info)
							local s = CDTL2:GetSpellSettings(CDTL2.currentFilter["debuffs"], "debuffs")
							
							if s then
								return s["icon"]
							else
								return 134400
							end
						end,
						imageWidth = 24,
						imageHeight = 24,
						imageCoords = {0.1,0.9,0.1,0.9},
					},
					spacer202 = {
						name = function(info)
								local n = "\n\n"
								local s = CDTL2:GetSpellSettings(CDTL2.currentFilter["debuffs"], "debuffs")
								
								if s then
									n =	n.."Duration: "..(s["bCD"] / 1000).." sec\n"
								end
								
								return n
							end,
						type = "description",
						order = 202,
						hidden = function(info)
								return CDTL2.currentFilterHidden["debuffs"]
							end,
						fontSize = "medium",
					},	
					spacer300 = {
						name = "\n\n",
						type = "description",
						order = 300,
						hidden = function(info)
								return CDTL2.currentFilterHidden["debuffs"]
							end,
					},
					enabled = {
						name = "Enabled - Untick this to never show this as an icon/bar",
						desc = "",
						order = 301,
						type = "toggle",
						width = "full",
						hidden = function(info)
								return CDTL2.currentFilterHidden["debuffs"]
							end,
						get = function(info, index)
								local s = CDTL2:GetSpellSettings(CDTL2.currentFilter["debuffs"], "debuffs")
								
								if s then
									return s["enabled"]
								else
									return nil
								end
							end,
						set = function(info, val)
								CDTL2:SetSpellData(CDTL2.currentFilter["debuffs"], "debuffs", "enabled", val)
								
								local s = CDTL2:GetExistingCooldown(CDTL2.currentFilter["debuffs"], "debuffs")
								s.data["enabled"] = val
								
								CDTL2:RefreshIcon(s)
								CDTL2:RefreshBar(s)
							end,
					},					
					spacer400 = {
						name = "\n\n",
						type = "description",
						order = 400,
						hidden = function(info)
								return CDTL2.currentFilterHidden["debuffs"]
							end,
					},
					highlight = {
						name = "Highlight - Tick to apply highlighting to the icon",
						desc = "",
						order = 401,
						type = "toggle",
						width = "full",
						hidden = function(info)
								return CDTL2.currentFilterHidden["debuffs"]
							end,
						get = function(info, index)
								local s = CDTL2:GetSpellSettings(CDTL2.currentFilter["debuffs"], "debuffs")
								
								if s then
									return s["highlight"]
								else
									return nil
								end
							end,
						set = function(info, val)
								CDTL2:SetSpellData(CDTL2.currentFilter["debuffs"], "debuffs", "highlight", val)
								
								local s = CDTL2:GetExistingCooldown(CDTL2.currentFilter["debuffs"], "debuffs")
								if s then
									s.data["highlight"] = val
									
									CDTL2:RefreshIcon(s)
									--CDTL2:RefreshBar(s)
								end
							end,
					},
					pinned = {
						name = "Pinned - Tick to keep an icon in the ready frame until the debuff is applied again",
						desc = "",
						order = 402,
						type = "toggle",
						width = "full",
						hidden = function(info)
								return CDTL2.currentFilterHidden["debuffs"]
							end,
						get = function(info, index)
								local s = CDTL2:GetSpellSettings(CDTL2.currentFilter["debuffs"], "debuffs")
								
								if s then
									return s["pinned"]
								else
									return nil
								end
							end,
						set = function(info, val)
								CDTL2:SetSpellData(CDTL2.currentFilter["debuffs"], "debuffs", "pinned", val)
								
								local s = CDTL2:GetExistingCooldown(CDTL2.currentFilter["debuffs"], "debuffs")
								if s then
									s.data["pinned"] = val
									
									CDTL2:RefreshIcon(s)
									--CDTL2:RefreshBar(s)
								end
							end,
					},
					spacer500 = {
						name = "\n\nSelect in which frame the icon/bar will be shown\nIf a the selected frame is not enabled, the icon/bar will default to 1\n\n",
						type = "description",
						order = 500,
						hidden = function(info)
								return CDTL2.currentFilterHidden["debuffs"]
							end,
					},
					lane = {
						name = "Lane",
						desc = "",
						order = 501,
						type = "select",
						hidden = function(info)
								return CDTL2.currentFilterHidden["debuffs"]
							end,
						width = "half",
						values = { [0] = "Hide", [1] = "1", [2] = "2", [3] = "3" },
						get = function(info)
								local s = CDTL2:GetSpellSettings(CDTL2.currentFilter["debuffs"], "debuffs")
								
								if s then
									return s["lane"]
								else
									return nil
								end
							end,
						set = function(info, val)
								CDTL2:SetSpellData(CDTL2.currentFilter["debuffs"], "debuffs", "lane", tonumber(val))
								
								local s = CDTL2:GetExistingCooldown(CDTL2.currentFilter["debuffs"], "debuffs")
								if s then
									s.data["lane"] = val
									
									CDTL2:SendToLane(s)
								end
							end,
					},
					barFrame = {
						name = "Bar Frame",
						desc = "",
						order = 502,
						type = "select",
						hidden = function(info)
								return CDTL2.currentFilterHidden["debuffs"]
							end,
						width = "half",
						values = { [0] = "Hide", [1] = "1", [2] = "2", [3] = "3" },
						get = function(info)
								local s = CDTL2:GetSpellSettings(CDTL2.currentFilter["debuffs"], "debuffs")
								
								if s then
									return s["barFrame"]
								else
									return nil
								end
							end,
						set = function(info, val)
								CDTL2:SetSpellData(CDTL2.currentFilter["debuffs"], "debuffs", "barFrame", tonumber(val))
								
								local s = CDTL2:GetExistingCooldown(CDTL2.currentFilter["debuffs"], "debuffs")
								if s then
									s.data["barFrame"] = val
									
									CDTL2:SendToBarFrame(s)
								end
							end,
					},
					readyFrame = {
						name = "Ready Frame",
						desc = "",
						order = 503,
						hidden = function(info)
								return CDTL2.currentFilterHidden["debuffs"]
							end,
						type = "select",
						width = "half",
						values = { [0] = "Hide", [1] = "1", [2] = "2", [3] = "3" },
						get = function(info)
								local s = CDTL2:GetSpellSettings(CDTL2.currentFilter["debuffs"], "debuffs")
								
								if s then
									return s["readyFrame"]
								else
									return nil
								end
							end,
						set = function(info, val)
								CDTL2:SetSpellData(CDTL2.currentFilter["debuffs"], "debuffs", "readyFrame", tonumber(val))
								
								local s = CDTL2:GetExistingCooldown(CDTL2.currentFilter["debuffs"], "debuffs")
								if s then
									s.data["readyFrame"] = val
									
									CDTL2:SendToReady(s)
								end
							end,
					},
				},
			},
			offensives = {
				name = "Offensives",
				type = "group",
				order = 600,
				args = {
					spacer100 = {
						name = "\n\nSelect an offensive to edit its settings\n",
						type = "description",
						order = 100,
					},
					list = {
						name = "",
						desc = "",
						order = 101,
						type = "select",
						values = function(info)
								local list = CDTL2:LoadFilterList("offensives")
								return list
							end,
						get = function(info)
								return CDTL2.currentFilter["offensives"]
							end,
						set = function(info, val)
								local list = CDTL2:LoadFilterList("offensives")
								CDTL2.currentFilter["offensives"] = list[val]
								CDTL2.currentFilterHidden["offensives"] = false
							end,
					},
					spacer200 = {
						name = "*Offensives will show up here once they have been detected at least once*\n\n\n",
						type = "description",
						order = 200,
					},
					spacer201 = {
						name = function(info)
								local s = ""
								
								if CDTL2.currentFilter["offensives"] then
									s = CDTL2.currentFilter["offensives"]
								end
								
								return s
							end,
						type = "description",
						order = 201,
						fontSize = "large",
						hidden = function(info)
								return CDTL2.currentFilterHidden["offensives"]
							end,
						image = function(info)
							local s = CDTL2:GetSpellSettings(CDTL2.currentFilter["offensives"], "offensives")
							
							if s then
								return s["icon"]
							else
								return 134400
							end
						end,
						imageWidth = 24,
						imageHeight = 24,
						imageCoords = {0.1,0.9,0.1,0.9},
					},
					spacer202 = {
						name = function(info)
								local n = "\n\n"
								local s = CDTL2:GetSpellSettings(CDTL2.currentFilter["offensives"], "offensives")
								
								if s then
									n =	n.."Duration: "..(s["bCD"] / 1000).." sec\n"
								end
								
								return n
							end,
						type = "description",
						order = 202,
						hidden = function(info)
								return CDTL2.currentFilterHidden["offensives"]
							end,
						fontSize = "medium",
					},	
					spacer300 = {
						name = "\n\n",
						type = "description",
						order = 300,
						hidden = function(info)
								return CDTL2.currentFilterHidden["offensives"]
							end,
					},
					enabled = {
						name = "Enabled - Untick this to never show this as an icon/bar",
						desc = "",
						order = 301,
						type = "toggle",
						width = "full",
						hidden = function(info)
								return CDTL2.currentFilterHidden["offensives"]
							end,
						get = function(info, index)
								local s = CDTL2:GetSpellSettings(CDTL2.currentFilter["offensives"], "offensives")
								
								if s then
									return s["enabled"]
								else
									return nil
								end
							end,
						set = function(info, val)
								CDTL2:SetSpellData(CDTL2.currentFilter["offensives"], "offensives", "enabled", val)
								
								local s = CDTL2:GetExistingCooldown(CDTL2.currentFilter["offensives"], "offensives")
								s.data["enabled"] = val
								
								CDTL2:RefreshIcon(s)
								CDTL2:RefreshBar(s)
							end,
					},					
					spacer400 = {
						name = "\n\n",
						type = "description",
						order = 400,
						hidden = function(info)
								return CDTL2.currentFilterHidden["offensives"]
							end,
					},
					highlight = {
						name = "Highlight - Tick to apply highlighting to the icon",
						desc = "",
						order = 401,
						type = "toggle",
						width = "full",
						hidden = function(info)
								return CDTL2.currentFilterHidden["offensives"]
							end,
						get = function(info, index)
								local s = CDTL2:GetSpellSettings(CDTL2.currentFilter["offensives"], "offensives")
								
								if s then
									return s["highlight"]
								else
									return nil
								end
							end,
						set = function(info, val)
								CDTL2:SetSpellData(CDTL2.currentFilter["offensives"], "offensives", "highlight", val)
								
								local s = CDTL2:GetExistingCooldown(CDTL2.currentFilter["offensives"], "offensives")
								if s then
									s.data["highlight"] = val
									
									CDTL2:RefreshIcon(s)
									--CDTL2:RefreshBar(s)
								end
							end,
					},
					pinned = {
						name = "Pinned - Tick to keep an icon in the ready frame until the offensive is applied again",
						desc = "",
						order = 402,
						type = "toggle",
						width = "full",
						hidden = function(info)
								return CDTL2.currentFilterHidden["offensives"]
							end,
						get = function(info, index)
								local s = CDTL2:GetSpellSettings(CDTL2.currentFilter["offensives"], "offensives")
								
								if s then
									return s["pinned"]
								else
									return nil
								end
							end,
						set = function(info, val)
								CDTL2:SetSpellData(CDTL2.currentFilter["offensives"], "offensives", "pinned", val)
								
								local s = CDTL2:GetExistingCooldown(CDTL2.currentFilter["offensives"], "offensives")
								if s then
									s.data["pinned"] = val
									
									CDTL2:RefreshIcon(s)
									--CDTL2:RefreshBar(s)
								end
							end,
					},
					spacer500 = {
						name = "\n\nSelect in which frame the icon/bar will be shown\nIf a the selected frame is not enabled, the icon/bar will default to 1\n\n",
						type = "description",
						order = 500,
						hidden = function(info)
								return CDTL2.currentFilterHidden["offensives"]
							end,
					},
					lane = {
						name = "Lane",
						desc = "",
						order = 501,
						type = "select",
						hidden = function(info)
								return CDTL2.currentFilterHidden["offensives"]
							end,
						width = "half",
						values = { [0] = "Hide", [1] = "1", [2] = "2", [3] = "3" },
						get = function(info)
								local s = CDTL2:GetSpellSettings(CDTL2.currentFilter["offensives"], "offensives")
								
								if s then
									return s["lane"]
								else
									return nil
								end
							end,
						set = function(info, val)
								CDTL2:SetSpellData(CDTL2.currentFilter["offensives"], "offensives", "lane", tonumber(val))
								
								local s = CDTL2:GetExistingCooldown(CDTL2.currentFilter["offensives"], "offensives")
								if s then
									s.data["lane"] = val
									
									CDTL2:SendToLane(s)
								end
							end,
					},
					barFrame = {
						name = "Bar Frame",
						desc = "",
						order = 502,
						type = "select",
						hidden = function(info)
								return CDTL2.currentFilterHidden["offensives"]
							end,
						width = "half",
						values = { [0] = "Hide", [1] = "1", [2] = "2", [3] = "3" },
						get = function(info)
								local s = CDTL2:GetSpellSettings(CDTL2.currentFilter["offensives"], "offensives")
								
								if s then
									return s["barFrame"]
								else
									return nil
								end
							end,
						set = function(info, val)
								CDTL2:SetSpellData(CDTL2.currentFilter["offensives"], "offensives", "barFrame", tonumber(val))
								
								local s = CDTL2:GetExistingCooldown(CDTL2.currentFilter["offensives"], "offensives")
								if s then
									s.data["barFrame"] = val
									
									CDTL2:SendToBarFrame(s)
								end
							end,
					},
					readyFrame = {
						name = "Ready Frame",
						desc = "",
						order = 503,
						hidden = function(info)
								return CDTL2.currentFilterHidden["offensives"]
							end,
						type = "select",
						width = "half",
						values = { [0] = "Hide", [1] = "1", [2] = "2", [3] = "3" },
						get = function(info)
								local s = CDTL2:GetSpellSettings(CDTL2.currentFilter["offensives"], "offensives")
								
								if s then
									return s["readyFrame"]
								else
									return nil
								end
							end,
						set = function(info, val)
								CDTL2:SetSpellData(CDTL2.currentFilter["offensives"], "offensives", "readyFrame", tonumber(val))
								
								local s = CDTL2:GetExistingCooldown(CDTL2.currentFilter["offensives"], "offensives")
								if s then
									s.data["readyFrame"] = val
									
									CDTL2:SendToReady(s)
								end
							end,
					},
				},
			},
			petSpells = {
				name = "Pet Spells",
				type = "group",
				order = 700,
				args = {
					spacer100 = {
						name = "\n\nSelect a pet spell to edit its settings\n",
						type = "description",
						order = 100,
					},
					list = {
						name = "",
						desc = "",
						order = 101,
						type = "select",
						values = function(info)
								local list = CDTL2:LoadFilterList("petSpells")
								return list
							end,
						get = function(info)
								return CDTL2.currentFilter["petSpells"]
							end,
						set = function(info, val)
								local list = CDTL2:LoadFilterList("petSpells")
								CDTL2.currentFilter["petSpells"] = list[val]
								CDTL2.currentFilterHidden["petSpells"] = false
							end,
					},
					spacer200 = {
						name = "*Pet spells will show up here once they have been detected at least once*\n\n\n",
						type = "description",
						order = 200,
					},
					spacer201 = {
						name = function(info)
								local s = ""
								
								if CDTL2.currentFilter["petSpells"] then
									s = CDTL2.currentFilter["petSpells"]
								end
								
								return s
							end,
						type = "description",
						order = 201,
						fontSize = "large",
						hidden = function(info)
								return CDTL2.currentFilterHidden["petSpells"]
							end,
						image = function(info)
							local s = CDTL2:GetSpellSettings(CDTL2.currentFilter["petSpells"], "petSpells")
							
							if s then
								return s["icon"]
							else
								return 134400
							end
						end,
						imageWidth = 24,
						imageHeight = 24,
						imageCoords = {0.1,0.9,0.1,0.9},
					},
					spacer202 = {
						name = function(info)
								local n = "\n\n"
								local s = CDTL2:GetSpellSettings(CDTL2.currentFilter["petSpells"], "petSpells")
								
								if s then
									n =	n.."Duration: "..(s["bCD"] / 1000).." sec\n"
								end
								
								return n
							end,
						type = "description",
						order = 202,
						hidden = function(info)
								return CDTL2.currentFilterHidden["petSpells"]
							end,
						fontSize = "medium",
					},	
					spacer300 = {
						name = "\n\n",
						type = "description",
						order = 300,
						hidden = function(info)
								return CDTL2.currentFilterHidden["petSpells"]
							end,
					},
					enabled = {
						name = "Enabled - Untick this to never show this as an icon/bar",
						desc = "",
						order = 301,
						type = "toggle",
						width = "full",
						hidden = function(info)
								return CDTL2.currentFilterHidden["petSpells"]
							end,
						get = function(info, index)
								local s = CDTL2:GetSpellSettings(CDTL2.currentFilter["petSpells"], "petSpells")
								
								if s then
									return s["enabled"]
								else
									return nil
								end
							end,
						set = function(info, val)
								CDTL2:SetSpellData(CDTL2.currentFilter["petSpells"], "petSpells", "enabled", val)
								
								local s = CDTL2:GetExistingCooldown(CDTL2.currentFilter["petSpells"], "petSpells")
								s.data["enabled"] = val
								
								CDTL2:RefreshIcon(s)
								CDTL2:RefreshBar(s)
							end,
					},					
					spacer400 = {
						name = "\n\n",
						type = "description",
						order = 400,
						hidden = function(info)
								return CDTL2.currentFilterHidden["petSpells"]
							end,
					},
					highlight = {
						name = "Highlight - Tick to apply highlighting to the icon",
						desc = "",
						order = 401,
						type = "toggle",
						width = "full",
						hidden = function(info)
								return CDTL2.currentFilterHidden["petSpells"]
							end,
						get = function(info, index)
								local s = CDTL2:GetSpellSettings(CDTL2.currentFilter["petSpells"], "petSpells")
								
								if s then
									return s["highlight"]
								else
									return nil
								end
							end,
						set = function(info, val)
								CDTL2:SetSpellData(CDTL2.currentFilter["petSpells"], "petSpells", "highlight", val)
								
								local s = CDTL2:GetExistingCooldown(CDTL2.currentFilter["petSpells"], "petSpells")
								if s then
									s.data["highlight"] = val
									
									CDTL2:RefreshIcon(s)
									--CDTL2:RefreshBar(s)
								end
							end,
					},
					pinned = {
						name = "Pinned - Tick to keep an icon in the ready frame until the pet spell is cast again",
						desc = "",
						order = 402,
						type = "toggle",
						width = "full",
						hidden = function(info)
								return CDTL2.currentFilterHidden["petSpells"]
							end,
						get = function(info, index)
								local s = CDTL2:GetSpellSettings(CDTL2.currentFilter["petSpells"], "petSpells")
								
								if s then
									return s["pinned"]
								else
									return nil
								end
							end,
						set = function(info, val)
								CDTL2:SetSpellData(CDTL2.currentFilter["petSpells"], "petSpells", "pinned", val)
								
								local s = CDTL2:GetExistingCooldown(CDTL2.currentFilter["petSpells"], "petSpells")
								if s then
									s.data["pinned"] = val
									
									CDTL2:RefreshIcon(s)
									--CDTL2:RefreshBar(s)
								end
							end,
					},
					spacer500 = {
						name = "\n\nSelect in which frame the icon/bar will be shown\nIf a the selected frame is not enabled, the icon/bar will default to 1\n\n",
						type = "description",
						order = 500,
						hidden = function(info)
								return CDTL2.currentFilterHidden["petSpells"]
							end,
					},
					lane = {
						name = "Lane",
						desc = "",
						order = 501,
						type = "select",
						hidden = function(info)
								return CDTL2.currentFilterHidden["petSpells"]
							end,
						width = "half",
						values = { [0] = "Hide", [1] = "1", [2] = "2", [3] = "3" },
						get = function(info)
								local s = CDTL2:GetSpellSettings(CDTL2.currentFilter["petSpells"], "petSpells")
								
								if s then
									return s["lane"]
								else
									return nil
								end
							end,
						set = function(info, val)
								CDTL2:SetSpellData(CDTL2.currentFilter["petSpells"], "petSpells", "lane", tonumber(val))
								
								local s = CDTL2:GetExistingCooldown(CDTL2.currentFilter["petSpells"], "petSpells")
								if s then
									s.data["lane"] = val
									
									CDTL2:SendToLane(s)
								end
							end,
					},
					barFrame = {
						name = "Bar Frame",
						desc = "",
						order = 502,
						type = "select",
						hidden = function(info)
								return CDTL2.currentFilterHidden["petSpells"]
							end,
						width = "half",
						values = { [0] = "Hide", [1] = "1", [2] = "2", [3] = "3" },
						get = function(info)
								local s = CDTL2:GetSpellSettings(CDTL2.currentFilter["petSpells"], "petSpells")
								
								if s then
									return s["barFrame"]
								else
									return nil
								end
							end,
						set = function(info, val)
								CDTL2:SetSpellData(CDTL2.currentFilter["petSpells"], "petSpells", "barFrame", tonumber(val))
								
								local s = CDTL2:GetExistingCooldown(CDTL2.currentFilter["petSpells"], "petSpells")
								if s then
									s.data["barFrame"] = val
									
									CDTL2:SendToBarFrame(s)
								end
							end,
					},
					readyFrame = {
						name = "Ready Frame",
						desc = "",
						order = 503,
						hidden = function(info)
								return CDTL2.currentFilterHidden["petSpells"]
							end,
						type = "select",
						width = "half",
						values = { [0] = "Hide", [1] = "1", [2] = "2", [3] = "3" },
						get = function(info)
								local s = CDTL2:GetSpellSettings(CDTL2.currentFilter["petSpells"], "petSpells")
								
								if s then
									return s["readyFrame"]
								else
									return nil
								end
							end,
						set = function(info, val)
								CDTL2:SetSpellData(CDTL2.currentFilter["petSpells"], "petSpells", "readyFrame", tonumber(val))
								
								local s = CDTL2:GetExistingCooldown(CDTL2.currentFilter["petSpells"], "petSpells")
								if s then
									s.data["readyFrame"] = val
									
									CDTL2:SendToReady(s)
								end
							end,
					},
				},
			},
		}
	}
	
	return options
end

function CDTL2:GetLaneOptions()
	local options = {
		name = "Lanes",
		type = "group",
		childGroups  = "tab",
		args = {
			lane1 = private.GetLaneSet(1),
			lane2 = private.GetLaneSet(2),
			lane3 = private.GetLaneSet(3),
		}
	}
	
	return options
end

function CDTL2:GetReadyOptions()
	local options = {
		name = "Ready",
		type = "group",
		childGroups  = "tab",
		args = {
			ready1 = private.GetReadySet(1),
			ready2 = private.GetReadySet(2),
			ready3 = private.GetReadySet(3),
		}
	}
	
	return options	
end

function CDTL2:GetBarFrameOptions()
	local options = {
		name = "Bar Frames",
		type = "group",
		childGroups  = "tab",
		args = {
			barFrame1 = private.GetBarFrameSet(1),
			barFrame2 = private.GetBarFrameSet(2),
			barFrame3 = private.GetBarFrameSet(3),
		}
	}
	
	return options	
end

private.GetBarFrameSet = function(i)
	local frame = nil
	
	if i == 1 then
		-- = "frame1"
		frame = CDTL2.db.profile.barFrames["frame1"]
	elseif i == 2 then
		--frame = "frame2"
		frame = CDTL2.db.profile.barFrames["frame2"]
	elseif i == 3 then
		--frame = "frame3"
		frame = CDTL2.db.profile.barFrames["frame3"]
	end
	
	local options = {
		--name = "Bar Frame "..i,
		name = function(info)
				return frame["name"]
			end,
		type = "group",
		order = 100,
		--childGroups  = "tab",
		args = {
			general = {
				name = "General",
				type = "group",
				order = 100,
				args = {
					barFrameName = {
						name = "Frame Name",
						type = "input",
						order = 101,
						get = function(info)
								return frame["name"]
							end,
						set = function(info, val)
								frame["name"] = val
								CDTL2:RefreshBarFrame(i)
							end,
					},
					barFrameEnabled = {
						name = "Enabled",
						desc = "",
						order = 102,
						type = "toggle",
						--width = "full",
						get = function(info, index)
								return frame["enabled"]
							end,
						set = function(info, val)
								frame["enabled"] = val
								CDTL2:CreateBarFrames()
								--CDTL2:RefreshBarFrame(i)
							end,
					},
					spacer200 = {
						name = "\n\n",
						type = "description",
						order = 200,
					},
					grow = {
						name = "Grow Direction",
						desc = "Select which direction new bars will grow in",
						order = 201,
						type = "select",
						values = {
								["UP"] = "UP",
								["DOWN"] = "DOWN",
								["LEFT"] = "LEFT",
								["RIGHT"] = "RIGHT",
								["CENTER_H"] = "CENTER (H)",
								["CENTER_V"] = "CENTER (V)",
							},
						get = function(info, index)
								return frame["grow"]
							end,
						set = function(info, val)
								frame["grow"] = val
							end,
					},
					--[[sorting = {
						name = "Sorting",
						desc = "How to sort",
						order = 202,
						type = "select",
						values = {
								["ASCENDING"] = "ASCENDING",
								["DESCENDING"] = "DESCENDING",
							},
						get = function(info, index)
								return frame["sorting"]
							end,
						set = function(info, val)
								frame["sorting"] = val
							end,
					},]]--
					spacer300 = {
						name = "\n\n",
						type = "description",
						order = 300,
					},
					hideTransitioned = {
						name = "Transition bars",
						desc = "If enabled, bars will be hidden once its cooldown drops below the lane max time, and its icon has appeared",
						order = 301,
						type = "toggle",
						--width = "full",
						get = function(info, index)
								return frame["transition"]["hideTransitioned"]
							end,
						set = function(info, val)
								frame["transition"]["hideTransitioned"] = val
							end,
					},
					spacer310 = {
						name = "",
						type = "description",
						order = 310,
						hidden = function(info)
								return not frame["transition"]["hideTransitioned"]
							end,
					},
					showTI = {
						name = "Show Indicator",
						desc = "If enabled, the bar will have an indicator showing when it will transition",
						order = 311,
						type = "toggle",
						--width = "full",
						hidden = function(info)
								return not frame["transition"]["hideTransitioned"]
							end,
						get = function(info, index)
								return frame["transition"]["showTI"]
							end,
						set = function(info, val)
								frame["transition"]["showTI"] = val
								CDTL2:RefreshBarFrame(i)
							end,
					},
					style = {
						name = "Indicator Style",
						desc = "Select a style of indicator",
						order = 312,
						type = "select",
						hidden = function(info)
								if frame["transition"]["hideTransitioned"] then
									if frame["transition"]["showTI"] then
										return false
									end
								end
								
								return true
							end,
						values = {
								["SHORTEN"] = "Shorten",
								["LINE"] = "Line",
								["REGION"] = "Region",
								--["NONE"] = "None",
							},
						get = function(info, index)
								return frame["transition"]["style"]
							end,
						set = function(info, val)
								frame["transition"]["style"] = val
							end,
					},
					spacer320 = {
						name = "",
						type = "description",
						hidden = function(info)
								if frame["transition"]["hideTransitioned"] then
									if frame["transition"]["showTI"] then
										local style = frame["transition"]["style"]
										if style == "LINE" or style == "REGION" then
											return false
										end
									end
								end
								
								return true
							end,
						order = 320,
					},
					fgTexture = {
						name = "Indicator Texture",
						desc = "Selects the texture",
						order = 321,
						type = "select",
						hidden = function(info)
								if frame["transition"]["hideTransitioned"] then
									if frame["transition"]["showTI"] then
										local style = frame["transition"]["style"]
										if style == "LINE" or style == "REGION" then
											return false
										end
									end
								end
								
								return true
							end,
						dialogControl = 'LSM30_Statusbar',
						values = AceGUIWidgetLSMlists.statusbar,
						get = function(info, index)
								return frame["transition"]["texture"]
							end,
						set = function(info, val)
								frame["transition"]["texture"] = val
								CDTL2:RefreshBarFrame(i)
							end,
					},
					fgTextureColor = {
						name = "Indicator Color",
						desc = "Sets the texture color",
						order = 322,
						type = "color",
						hidden = function(info)
								if frame["transition"]["hideTransitioned"] then
									if frame["transition"]["showTI"] then
										local style = frame["transition"]["style"]
										if style == "LINE" or style == "REGION" then
											return false
										end
									end
								end
								
								return true
							end,
						hasAlpha = true,
						get = function(info)
								local r = frame["transition"]["textureColor"]["r"]
								local g = frame["transition"]["textureColor"]["g"]
								local b = frame["transition"]["textureColor"]["b"]
								local a = frame["transition"]["textureColor"]["a"]
								return r, g, b, a
							end,
						set = function(info, red, green, blue, alpha)
								frame["transition"]["textureColor"] = { r = red, g = green, b = blue, a = alpha }
								CDTL2:RefreshBarFrame(i)
							end,
					},
					spacer330 = {
						name = "",
						type = "description",
						order = 330,
						hidden = function(info)
								if frame["transition"]["hideTransitioned"] then
									if frame["transition"]["showTI"] then
										local style = frame["transition"]["style"]
										if style == "LINE" then
											return false
										end
									end
								end
								
								return true
							end,
					},
					widthTI = {
						name = "Indicator Line Width",
						desc = "Sets the width of the transition indicator line",
						order = 331,
						type = "range",
						hidden = function(info)
								if frame["transition"]["hideTransitioned"] then
									if frame["transition"]["showTI"] then
										local style = frame["transition"]["style"]
										if style == "LINE" then
											return false
										end
									end
								end
								
								return true
							end,
						softMin = 0,
						softMax = 40,
						get = function(info, index)
								return frame["transition"]["width"]
							end,
						set = function(info, val)
								frame["transition"]["width"] = val
						end,
					},
					
					spacer500 = {
						name = "\n\n",
						type = "description",
						order = 500,
					},
					xPadding = {
						name = "x Padding",
						desc = "Sets the x offset between growing bars",
						order = 501,
						type = "range",
						softMin = 0,
						softMax = 40,
						get = function(info, index)
								return frame["bar"]["xPadding"]
							end,
						set = function(info, val)
								frame["bar"]["xPadding"] = val
						end,
					},
					yPadding = {
						name = "y Padding",
						desc = "Sets the y offset between growing bars",
						order = 502,
						type = "range",
						softMin = 0,
						softMax = 40,
						get = function(info, index)
								return frame["bar"]["yPadding"]
							end,
						set = function(info, val)
								frame["bar"]["yPadding"] = val
						end,
					},
				}
			},
			appearance = {
				name = "Appearance",
				type = "group",
				order = 200,
				args = {
					--[[spacer100 = {
						name = "\n\n",
						type = "description",
						order = 100,
					},]]--
					posX = {
						name = "x Position",
						desc = "Sets the x position of the frame",
						order = 111,
						type = "range",
						softMin = -500,
						softMax = 500,
						get = function(info, index)
								return frame["posX"]
							end,
						set = function(info, val)
								frame["posX"] = val
								CDTL2:RefreshBarFrame(i)
							end,
					},
					posY = {
						name = "y Position",
						desc = "Sets the y position of the frame",
						order = 112,
						type = "range",
						softMin = -500,
						softMax = 500,
						get = function(info, index)
								return frame["posY"]
							end,
						set = function(info, val)
								frame["posY"] = val
								CDTL2:RefreshBarFrame(i)
							end,
					},
					spacer113 = {
						name = "",
						type = "description",
						order = 113,
					},
					relativeTo = {
						name = "Anchor Point",
						desc = "The x/y position is relative to this point of the screen",
						order = 114,
						type = "select",
						values = {
								["TOPLEFT"] = "TOPLEFT",
								["TOP"] = "TOP",
								["TOPRIGHT"] = "TOPRIGHT",
								["LEFT"] = "LEFT",
								["CENTER"] = "CENTER",
								["RIGHT"] = "RIGHT",
								["BOTTOMLEFT"] = "BOTTOMLEFT",
								["BOTTOM"] = "BOTTOM",
								["BOTTOMRIGHT"] = "BOTTOMRIGHT",
							},
						get = function(info, index)
								return frame["relativeTo"]
							end,
						set = function(info, val)
								frame["relativeTo"] = val
								CDTL2:RefreshBarFrame(i)
							end,
					},
					spacer200 = {
						name = "\n\n",
						type = "description",
						order = 200,
					},
					padding = {
						name = "Padding",
						desc = "Sets the amount of padding in the frame",
						order = 201,
						type = "range",
						softMin = 0,
						softMax = 40,
						get = function(info, index)
								return frame["padding"]
							end,
						set = function(info, val)
								frame["padding"] = val
								CDTL2:RefreshAllIcons()
								CDTL2:RefreshBarFrame(i)
						end,
					},
					spacer300 = {
						name = "\n\n",
						type = "description",
						order = 300,
					},
					bgTexture = {
						name = "Background Texture",
						desc = "Sets the background texture",
						order = 301,
						type = "select",
						dialogControl = 'LSM30_Statusbar',
						values = AceGUIWidgetLSMlists.statusbar,
						get = function(info, index)
								return frame["bgTexture"]
							end,
						set = function(info, val)
								frame["bgTexture"] = val
								CDTL2:RefreshBarFrame(i)
							end,
					},
					bgTextureColor = {
						name = "Color",
						desc = "Sets the background texture color",
						order = 302,
						type = "color",
						hasAlpha = true,
						get = function(info)
								local r = frame["bgTextureColor"]["r"]
								local g = frame["bgTextureColor"]["g"]
								local b = frame["bgTextureColor"]["b"]
								local a = frame["bgTextureColor"]["a"]
								return r, g, b, a
							end,
						set = function(info, red, green, blue, alpha)
								frame["bgTextureColor"] = { r = red, g = green, b = blue, a = alpha }
								CDTL2:RefreshBarFrame(i)
							end,
					},
					spacer400 = {
						name = "\n\n",
						type = "description",
						order = 400,
					},
					border = {
						name = "                    Border Texture",
						desc = "Selects the border texture",
						order = 401,
						type = "select",
						dialogControl = 'LSM30_Border',
						values = AceGUIWidgetLSMlists.border,
						get = function(info) return frame["border"]["style"] end,
						set = function(info, val)
								frame["border"]["style"] = val
								CDTL2:RefreshBarFrame(i)
							end,
					},
					borderColor = {
						name = "Color",
						desc = "Selects the border color",
						order = 402,
						type = "color",
						hasAlpha = true,
						get = function(info)
								local r = frame["border"]["color"]["r"]
								local g = frame["border"]["color"]["g"]
								local b = frame["border"]["color"]["b"]
								local a = frame["border"]["color"]["a"]
								
								return r, g, b, a
							end,
						set = function(info, red, green, blue, alpha)
								frame["border"]["color"] = { r = red, g = green, b = blue, a = alpha }							
								CDTL2:RefreshBarFrame(i)
							end,
					},
					spacer410 = {
						name = "",
						type = "description",
						order = 410,
					},
					borderSize = {
						name = "Size",
						desc = "Sets the size of the border",
						order = 411,
						type = "range",
						softMin = 1,
						softMax = 40,
						get = function(info) return frame["border"]["size"] end,
						set = function(info, val)
								frame["border"]["size"] = val
								CDTL2:RefreshBarFrame(i)
							end,
					},
					borderPadding = {
						name = "Padding",
						desc = "Sets the size of the border",
						order = 412,
						type = "range",
						softMin = 0,
						softMax = 40,
						get = function(info) return frame["border"]["padding"] end,
						set = function(info, val)
								frame["border"]["padding"] = val
								CDTL2:RefreshBarFrame(i)
							end,
					},
				}
			},
			bars = {
				name = "Bars",
				type = "group",
				order = 300,
				args = {
					--[[spacer100 = {
						name = "\n\n",
						type = "description",
						order = 100,
					},]]--
					width = {
						name = "Width",
						desc = "Sets the width of the bars",
						order = 101,
						type = "range",
						softMin = 1,
						softMax = 128,
						get = function(info, index)
								return frame["width"]
							end,
						set = function(info, val)
								frame["width"] = val
								CDTL2:RefreshAllBars()
								CDTL2:RefreshBarFrame(i)
						end,
					},
					height = {
						name = "Height",
						desc = "Sets the height of the bars",
						order = 102,
						type = "range",
						softMin = 1,
						softMax = 128,
						get = function(info, index)
								return frame["height"]
							end,
						set = function(info, val)
								frame["height"] = val
								CDTL2:RefreshAllBars()
								CDTL2:RefreshBarFrame(i)
						end,
					},
					spacer102 = {
						name = "\n\n",
						type = "description",
						order = 103,
					},
					fgTexture = {
						name = "Foreground Texture",
						desc = "Selects the texture",
						order = 104,
						type = "select",
						dialogControl = 'LSM30_Statusbar',
						values = AceGUIWidgetLSMlists.statusbar,
						get = function(info, index)
								return frame["bar"]["fgTexture"]
							end,
						set = function(info, val)
								frame["bar"]["fgTexture"] = val
								CDTL2:RefreshAllBars()
							end,
					},
					fgTextureColor = {
						name = "Color",
						desc = "Sets the texture color",
						order = 105,
						type = "color",
						hasAlpha = true,
						get = function(info)
								local r = frame["bar"]["fgTextureColor"]["r"]
								local g = frame["bar"]["fgTextureColor"]["g"]
								local b = frame["bar"]["fgTextureColor"]["b"]
								local a = frame["bar"]["fgTextureColor"]["a"]
								return r, g, b, a
							end,
						set = function(info, red, green, blue, alpha)
								frame["bar"]["fgTextureColor"] = { r = red, g = green, b = blue, a = alpha }
								CDTL2:RefreshAllBars(i)
							end,
					},
					spacer106 = {
						name = "",
						type = "description",
						order = 106,
					},
					bgTexture = {
						name = "Background Texture",
						desc = "Sets the background texture",
						order = 107,
						type = "select",
						dialogControl = 'LSM30_Statusbar',
						values = AceGUIWidgetLSMlists.statusbar,
						get = function(info, index)
								return frame["bar"]["bgTexture"]
							end,
						set = function(info, val)
								frame["bar"]["bgTexture"] = val
								CDTL2:RefreshAllBars(i)
							end,
					},
					bgTextureColor = {
						name = "Color",
						desc = "Sets the background texture color",
						order = 108,
						type = "color",
						hasAlpha = true,
						get = function(info)
								local r = frame["bar"]["bgTextureColor"]["r"]
								local g = frame["bar"]["bgTextureColor"]["g"]
								local b = frame["bar"]["bgTextureColor"]["b"]
								local a = frame["bar"]["bgTextureColor"]["a"]
								return r, g, b, a
							end,
						set = function(info, red, green, blue, alpha)
								frame["bar"]["bgTextureColor"] = { r = red, g = green, b = blue, a = alpha }
								CDTL2:RefreshAllBars(i)
							end,
					},
					spacer109 = {
						name = "\n\n",
						type = "description",
						order = 109,
					},
					textHeader = {
						name = "Bar Text",
						type = "header",
						order = 200.1,
					},
					-- BAR TEXT 1
					--bText1S0 = { name = "\n", type = "description", order = 210, },
					bText1Text = private.GetTextText(frame["bar"]["text1"], 210, i, "BAR", false),
					bText1Enabled = private.GetTextEnabled(frame["bar"]["text1"], 210, i, "BAR"),
					bText1Edit = private.GetTextEdit(frame["bar"]["text1"], 210, i, "BAR"),
					bText1Font = private.GetTextFont(frame["bar"]["text1"], 210, i, "BAR"),
					bText1Outline = private.GetTextOutline(frame["bar"]["text1"], 210, i, "BAR"),
					bText1S1 = { name = "", type = "description", order = 210 + 4.3, hidden = function(info) return not frame["bar"]["text1"]["edit"] end, },
					bText1Size = private.GetTextSize(frame["bar"]["text1"], 210, i, "BAR"),
					bText1Color = private.GetTextColor(frame["bar"]["text1"], 210, i, "BAR"),
					bText1ShadowColor = private.GetTextShadowColor(frame["bar"]["text1"], 210, i, "BAR"),
					bText1S2 = { name = "", type = "description", order = 210 + 4.7, hidden = function(info) return not frame["bar"]["text1"]["edit"] end, },
					bText1ShadowXOffset = private.GetTextShadowXOffset(frame["bar"]["text1"], 210, i, "BAR"),
					bText1ShadowYOffset = private.GetTextShadowYOffset(frame["bar"]["text1"], 210, i, "BAR"),
					bText1S3 = { name = "", type = "description", order = 210 + 5.1, hidden = function(info) return not frame["bar"]["text1"]["edit"] end, },
					bText1Anchor = private.GetTextAnchor(frame["bar"]["text1"], 210, i, "BAR"),
					bText1Align = private.GetTextAlign(frame["bar"]["text1"], 210, i, "BAR"),
					bText1S4 = { name = "", type = "description", order = 210 + 5.4, hidden = function(info) return not frame["bar"]["text1"]["edit"] end, },
					bText1XOffset = private.GetTextXOffset(frame["bar"]["text1"], 210, i, "BAR"),
					bText1YOffset = private.GetTextYOffset(frame["bar"]["text1"], 210, i, "BAR"),
					bText1Footer = { name = "", type = "header", order = 210 + 9.8, hidden = function(info) return not frame["bar"]["text1"]["edit"] end, },
					-- BAR TEXT 2
					bText2S0 = { name = "\n", type = "description", order = 220, hidden = function(info) return not frame["bar"]["text2"]["used"] end, },
					bText2Header = { name = "", type = "header", order = 220 + 0.1, hidden = function(info) return not frame["bar"]["text2"]["edit"] end, },
					bText2Text = private.GetTextText(frame["bar"]["text2"], 220, i, "BAR", false),
					bText2Enabled = private.GetTextEnabled(frame["bar"]["text2"], 220, i, "BAR"),
					bText2Edit = private.GetTextEdit(frame["bar"]["text2"], 220, i, "BAR"),
					bText2Font = private.GetTextFont(frame["bar"]["text2"], 220, i, "BAR"),
					bText2Outline = private.GetTextOutline(frame["bar"]["text2"], 220, i, "BAR"),
					bText2S1 = { name = "", type = "description", order = 220 + 4.3, hidden = function(info) return not frame["bar"]["text2"]["edit"] end, },
					bText2Size = private.GetTextSize(frame["bar"]["text2"], 220, i, "BAR"),
					bText2Color = private.GetTextColor(frame["bar"]["text2"], 220, i, "BAR"),
					bText2ShadowColor = private.GetTextShadowColor(frame["bar"]["text2"], 220, i, "BAR"),
					bText2S2 = { name = "", type = "description", order = 220 + 4.7, hidden = function(info) return not frame["bar"]["text2"]["edit"] end, },
					bText2ShadowXOffset = private.GetTextShadowXOffset(frame["bar"]["text2"], 220, i, "BAR"),
					bText2ShadowYOffset = private.GetTextShadowYOffset(frame["bar"]["text2"], 220, i, "BAR"),
					bText2S3 = { name = "", type = "description", order = 220 + 5.1, hidden = function(info) return not frame["bar"]["text2"]["edit"] end, },
					bText2Anchor = private.GetTextAnchor(frame["bar"]["text2"], 220, i, "BAR"),
					bText2Align = private.GetTextAlign(frame["bar"]["text2"], 220, i, "BAR"),
					bText2S4 = { name = "", type = "description", order = 220 + 5.4, hidden = function(info) return not frame["bar"]["text2"]["edit"] end, },
					bText2XOffset = private.GetTextXOffset(frame["bar"]["text2"], 220, i, "BAR"),
					bText2YOffset = private.GetTextYOffset(frame["bar"]["text2"], 220, i, "BAR"),
					bText2Footer = { name = "", type = "header", order = 220 + 9.8, hidden = function(info) return not frame["bar"]["text2"]["edit"] end, },
					-- BAR TEXT 3
					bText3S0 = { name = "\n", type = "description", order = 230, hidden = function(info) return not frame["bar"]["text3"]["used"] end, },
					bText3Header = { name = "", type = "header", order = 230 + 0.1, hidden = function(info) return not frame["bar"]["text3"]["edit"] end, },
					bText3Text = private.GetTextText(frame["bar"]["text3"], 230, i, "BAR", false),
					bText3Enabled = private.GetTextEnabled(frame["bar"]["text3"], 230, i, "BAR"),
					bText3Edit = private.GetTextEdit(frame["bar"]["text3"], 230, i, "BAR"),
					bText3Font = private.GetTextFont(frame["bar"]["text3"], 230, i, "BAR"),
					bText3Outline = private.GetTextOutline(frame["bar"]["text3"], 230, i, "BAR"),
					bText3S1 = { name = "", type = "description", order = 230 + 4.3, hidden = function(info) return not frame["bar"]["text3"]["edit"] end, },
					bText3Size = private.GetTextSize(frame["bar"]["text3"], 230, i, "BAR"),
					bText3Color = private.GetTextColor(frame["bar"]["text3"], 230, i, "BAR"),
					bText3ShadowColor = private.GetTextShadowColor(frame["bar"]["text3"], 230, i, "BAR"),
					bText3S2 = { name = "", type = "description", order = 230 + 4.7, hidden = function(info) return not frame["bar"]["text3"]["edit"] end, },
					bText3ShadowXOffset = private.GetTextShadowXOffset(frame["bar"]["text3"], 230, i, "BAR"),
					bText3ShadowYOffset = private.GetTextShadowYOffset(frame["bar"]["text3"], 230, i, "BAR"),
					bText3S3 = { name = "", type = "description", order = 230 + 5.1, hidden = function(info) return not frame["bar"]["text3"]["edit"] end, },
					bText3Anchor = private.GetTextAnchor(frame["bar"]["text3"], 230, i, "BAR"),
					bText3Align = private.GetTextAlign(frame["bar"]["text3"], 230, i, "BAR"),
					bText3S4 = { name = "", type = "description", order = 230 + 5.4, hidden = function(info) return not frame["bar"]["text3"]["edit"] end, },
					bText3XOffset = private.GetTextXOffset(frame["bar"]["text3"], 230, i, "BAR"),
					bText3YOffset = private.GetTextYOffset(frame["bar"]["text3"], 230, i, "BAR"),
					bText3Footer = { name = "", type = "header", order = 230 + 9.8, hidden = function(info) return not frame["bar"]["text3"]["edit"] end, },
					spacer298 = {
						name = "\n",
						type = "description",
						order = 298,
					},
					textFooter = {
						name = "",
						type = "header",
						order = 299,
					},
					spacer300 = {
						name = "\n\n",
						type = "description",
						order = 300,
					},
					iconEnabled = {
						name = "Show icon",
						desc = "",
						order = 301,
						type = "toggle",
						--width = "full",
						get = function(info, index)
								return frame["bar"]["iconEnabled"]
							end,
						set = function(info, val)
								frame["bar"]["iconEnabled"] = val
								CDTL2:RefreshAllBars()
							end,
					},
					iconPosition = {
						name = "Icon position",
						desc = "Which side of the bar should the icon appear",
						order = 302,
						type = "select",
						values = {
								["LEFT"] = "Left",
								["RIGHT"] = "Right",
							},
						get = function(info, index)
								return frame["bar"]["iconPosition"]
							end,
						set = function(info, val)
								frame["bar"]["iconPosition"] = val
								CDTL2:RefreshAllBars()
							end,
					},
					spacer400 = {
						name = "\n\n",
						type = "description",
						order = 400,
					},
					border = {
						name = "                    Border Texture",
						desc = "Selects the border texture",
						order = 401,
						type = "select",
						dialogControl = 'LSM30_Border',
						values = AceGUIWidgetLSMlists.border,
						get = function(info) return frame["bar"]["border"]["style"] end,
						set = function(info, val)
								frame["bar"]["border"]["style"] = val
								CDTL2:RefreshAllBars()
							end,
					},
					borderColor = {
						name = "Color",
						desc = "Selects the border color",
						order = 402,
						type = "color",
						hasAlpha = true,
						get = function(info)
								local r = frame["bar"]["border"]["color"]["r"]
								local g = frame["bar"]["border"]["color"]["g"]
								local b = frame["bar"]["border"]["color"]["b"]
								local a = frame["bar"]["border"]["color"]["a"]
								
								return r, g, b, a
							end,
						set = function(info, red, green, blue, alpha)
								frame["bar"]["border"]["color"] = { r = red, g = green, b = blue, a = alpha }							
								CDTL2:RefreshAllBars()
							end,
					},
					spacer410 = {
						name = "",
						type = "description",
						order = 410,
					},
					borderSize = {
						name = "Size",
						desc = "Sets the size of the border",
						order = 411,
						type = "range",
						softMin = 1,
						softMax = 40,
						get = function(info) return frame["bar"]["border"]["size"] end,
						set = function(info, val)
								frame["bar"]["border"]["size"] = val
								CDTL2:RefreshAllBars()
							end,
					},
					borderPadding = {
						name = "Padding",
						desc = "Sets the size of the border",
						order = 411,
						type = "range",
						softMin = 0,
						softMax = 40,
						get = function(info) return frame["bar"]["border"]["padding"] end,
						set = function(info, val)
								frame["bar"]["border"]["padding"] = val
								CDTL2:RefreshAllBars()
							end,
					},
				}
			},
		}
	}
	
	return options
end

private.GetLaneSet = function(i)
	local frame = nil
	local lane = nil
	
	if i == 1 then
		--frame = "CDTL2_Lane_1"
		lane = CDTL2.db.profile.lanes["lane1"]
	elseif i == 2 then
		--frame = "CDTL2_Lane_2"
		lane = CDTL2.db.profile.lanes["lane2"]
	elseif i == 3 then
		--frame = "CDTL2_Lane_3"
		lane = CDTL2.db.profile.lanes["lane3"]
	end
	
	local options = {
		--name = "Lane "..i,
		name = function(info)
				return lane["name"]
			end,
		type = "group",
		order = 100,
		--childGroups  = "tab",
		args = {
			general = {
				name = "General",
				type = "group",
				order = 100,
				args = {
					name = {
						name = "Frame Name",
						type = "input",
						order = 101,
						get = function(info)
								return lane["name"]
							end,
						set = function(info, val)
								lane["name"] = val
								CDTL2:RefreshLane(i)
							end,
					},
					enabled = {
						name = "Enabled",
						desc = "",
						order = 102,
						type = "toggle",
						--width = "full",
						get = function(info, index)
								return lane["enabled"]
							end,
						set = function(info, val)
								lane["enabled"] = val
								CDTL2:CreateLanes()
								--CDTL2:RefreshLane(i)
							end,
					},
					spacer200 = {
						name = "",
						type = "description",
						order = 200,
					},
					mode = {
						name = "Mode",
						desc = "How icons will progress along the lane",
						order = 201,
						type = "select",
						values = {
								["LINEAR"] = "Linear (%)",
								["LINEAR_ABS"] = "Linear (Time)",
								["SPLIT"] = "Split (%)",
								["SPLIT_ABS"] = "Split (Time)",
							},
						get = function(info, index)
								return lane["mode"]["type"]
							end,
						set = function(info, val)
								lane["mode"]["type"] = val
								CDTL2:RefreshLane(i)
							end,
					},
					
					spacer300 = {
						name = "",
						type = "description",
						order = 300,
					},
					maxTime = {
						name = "Max Time",
						desc = "How many seconds the lane represents",
						order = 301,
						type = "range",
						softMin = 20,
						softMax = 180,
						bigStep = 1,
						get = function(info, index)
								local mt = 0
								local mode = lane["mode"]["type"]
								
								if mode == "LINEAR" then
									mt = lane["mode"]["linear"]["max"]
								elseif mode == "LINEAR_ABS" then
									mt = lane["mode"]["linearAbs"]["max"]
								elseif mode == "SPLIT" then
									mt = lane["mode"]["split"]["max"]
								elseif mode == "SPLIT_ABS" then
									mt = lane["mode"]["splitAbs"]["max"]
								end
								
								return mt
							end,
						set = function(info, val)
								local mode = lane["mode"]["type"]
								
								if mode == "LINEAR" then
									lane["mode"]["linear"]["max"] = val
								elseif mode == "LINEAR_ABS" then
									lane["mode"]["linearAbs"]["max"] = val
								elseif mode == "SPLIT" then
									lane["mode"]["split"]["max"] = val
								elseif mode == "SPLIT_ABS" then
									lane["mode"]["splitAbs"]["max"] = val
								end
								
								CDTL2:RefreshLane(i)
						end,
					},
					hideLongTimers = {
						name = "Hide Long Timers",
						desc = "Should icons with remaining cooldowns longer than the max time be hidden",
						order = 302,
						type = "toggle",
						get = function(info, index)
								--local hlt = nil
								local mode = lane["mode"]["type"]
								
								if mode == "LINEAR" then
									return lane["mode"]["linear"]["hideTimeSurplus"]
								elseif mode == "LINEAR_ABS" then
									return lane["mode"]["linearAbs"]["hideTimeSurplus"]
								elseif mode == "SPLIT" then
									return lane["mode"]["split"]["hideTimeSurplus"]
								elseif mode == "SPLIT_ABS" then
									return lane["mode"]["splitAbs"]["hideTimeSurplus"]
								end
								
								return nil
							end,
						set = function(info, val)
								local mode = lane["mode"]["type"]
								
								if mode == "LINEAR" then
									lane["mode"]["linear"]["hideTimeSurplus"] = val
								elseif mode == "LINEAR_ABS" then
									lane["mode"]["linearAbs"]["hideTimeSurplus"] = val
								elseif mode == "SPLIT" then
									lane["mode"]["split"]["hideTimeSurplus"] = val
								elseif mode == "SPLIT_ABS" then
									lane["mode"]["splitAbs"]["hideTimeSurplus"] = val
								end
							end,
					},
					spacer400 = {
						name = "\n\n",
						type = "description",
						order = 400,
						hidden = function(info)
							if lane["mode"]["type"] == "SPLIT" then
								return false
							end
							return true
						end,
					},
					sSplitCount = {
						name = "Split Count",
						desc = "How many % splits appear on the lane",
						order = 401,
						type = "range",
						hidden = function(info)
							if lane["mode"]["type"] == "SPLIT" then
								return false
							end
							return true
						end,
						min = 1,
						max = 3,
						bigStep = 1,
						get = function(info, index)
								return lane["mode"]["split"]["count"]
							end,
						set = function(info, val)
								lane["mode"]["split"]["count"] = val
								CDTL2:RefreshLane(i)
						end,
					},
					spacer410 = {
						name = "\n\n",
						type = "description",
						order = 410,
						hidden = function(info)
							if lane["mode"]["type"] == "SPLIT" then
								return false
							end
							return true
						end,
					},
					sSplit1Position = {
						name = "Split 1 Position",
						desc = "What position should split 1 appear",
						order = 411,
						type = "range",
						hidden = function(info)
							if lane["mode"]["type"] == "SPLIT" then
								return false
							end
							return true
						end,
						min = 0,
						max = 1,
						get = function(info, index)
								return lane["mode"]["split"]["s1p"]
							end,
						set = function(info, val)
								lane["mode"]["split"]["s1p"] = val
								CDTL2:RefreshLane(i)
						end,
					},
					sSplit1Value = {
						name = "Split 1 Value",
						desc = "What percentage should split 1 occur",
						order = 412,
						type = "range",
						hidden = function(info)
							if lane["mode"]["type"] == "SPLIT" then
								return false
							end
							return true
						end,
						min = 0,
						max = 100,
						bigStep = 1,
						get = function(info, index)
								return lane["mode"]["split"]["s1v"]
							end,
						set = function(info, val)
								lane["mode"]["split"]["s1v"] = val
								CDTL2:RefreshLane(i)
						end,
					},
					spacer420 = {
						name = "",
						type = "description",
						order = 420,
						hidden = function(info)
							if lane["mode"]["type"] == "SPLIT" then
								if lane["mode"]["split"]["count"] >= 2 then
									return false
								end
							end
							return true
						end,
					},
					sSplit2Position = {
						name = "Split 2 Position",
						desc = "What position should split 2 appear",
						order = 421,
						type = "range",
						hidden = function(info)
							if lane["mode"]["type"] == "SPLIT" then
								if lane["mode"]["split"]["count"] >= 2 then
									return false
								end
							end
							return true
						end,
						min = 0,
						max = 1,
						get = function(info, index)
								return lane["mode"]["split"]["s2p"]
							end,
						set = function(info, val)
								lane["mode"]["split"]["s2p"] = val
								CDTL2:RefreshLane(i)
						end,
					},
					sSplit2Value = {
						name = "Split 2 Value",
						desc = "What percentage should split 2 occur",
						order = 422,
						type = "range",
						hidden = function(info)
							if lane["mode"]["type"] == "SPLIT" then
								if lane["mode"]["split"]["count"] >= 2 then
									return false
								end
							end
							return true
						end,
						min = 0,
						max = 100,
						bigStep = 1,
						get = function(info, index)
								return lane["mode"]["split"]["s2v"]
							end,
						set = function(info, val)
								lane["mode"]["split"]["s2v"] = val
								CDTL2:RefreshLane(i)
						end,
					},
					spacer430 = {
						name = "",
						type = "description",
						order = 430,
						hidden = function(info)
							if lane["mode"]["type"] == "SPLIT" then
								if lane["mode"]["split"]["count"] >= 2 then
									return false
								end
							end
							return true
						end,
					},
					sSplit3Position = {
						name = "Split 3 Position",
						desc = "What position should split 3 appear",
						order = 431,
						type = "range",
						hidden = function(info)
							if lane["mode"]["type"] == "SPLIT" then
								if lane["mode"]["split"]["count"] >= 3 then
									return false
								end
							end
							return true
						end,
						min = 0,
						max = 1,
						get = function(info, index)
								return lane["mode"]["split"]["s3p"]
							end,
						set = function(info, val)
								lane["mode"]["split"]["s3p"] = val
								CDTL2:RefreshLane(i)
						end,
					},
					sSplit3Value = {
						name = "Split 3 Value",
						desc = "What percentage should split 3 occur",
						order = 432,
						type = "range",
						hidden = function(info)
							if lane["mode"]["type"] == "SPLIT" then
								if lane["mode"]["split"]["count"] >= 3 then
									return false
								end
							end
							return true
						end,
						min = 0,
						max = 100,
						bigStep = 1,
						get = function(info, index)
								return lane["mode"]["split"]["s3v"]
							end,
						set = function(info, val)
								lane["mode"]["split"]["s3v"] = val
								CDTL2:RefreshLane(i)
						end,
					},
					spacer500 = {
						name = "\n\n",
						type = "description",
						order = 500,
						hidden = function(info)
							if lane["mode"]["type"] == "SPLIT_ABS" then
								return false
							end
							return true
						end,
					},
					saSplitCount = {
						name = "Split Count",
						desc = "How many % splits appear on the lane",
						order = 502,
						type = "range",
						hidden = function(info)
							if lane["mode"]["type"] == "SPLIT_ABS" then
								return false
							end
							return true
						end,
						min = 1,
						max = 3,
						bigStep = 1,
						get = function(info, index)
								return lane["mode"]["splitAbs"]["count"]
							end,
						set = function(info, val)
								lane["mode"]["splitAbs"]["count"] = val
								CDTL2:RefreshLane(i)
						end,
					},
					spacer510 = {
						name = "\n\n",
						type = "description",
						order = 510,
						hidden = function(info)
							if lane["mode"]["type"] == "SPLIT_ABS" then
								return false
							end
							return true
						end,
					},
					saSplit1Position = {
						name = "Split 1 Position",
						desc = "What position should splitAbs 1 appear",
						order = 511,
						type = "range",
						hidden = function(info)
							if lane["mode"]["type"] == "SPLIT_ABS" then
								return false
							end
							return true
						end,
						min = 0,
						max = 1,
						get = function(info, index)
								return lane["mode"]["splitAbs"]["s1p"]
							end,
						set = function(info, val)
								lane["mode"]["splitAbs"]["s1p"] = val
								CDTL2:RefreshLane(i)
						end,
					},
					saSplit1Value = {
						name = "Split 1 Value",
						desc = "What percentage should splitAbs 1 occur",
						order = 512,
						type = "range",
						hidden = function(info)
							if lane["mode"]["type"] == "SPLIT_ABS" then
								return false
							end
							return true
						end,
						min = 1,
						max = 360,
						bigStep = 1,
						get = function(info, index)
								return lane["mode"]["splitAbs"]["s1v"]
							end,
						set = function(info, val)
								lane["mode"]["splitAbs"]["s1v"] = val
								CDTL2:RefreshLane(i)
						end,
					},
					spacer520 = {
						name = "",
						type = "description",
						order = 520,
						hidden = function(info)
							if lane["mode"]["type"] == "SPLIT_ABS" then
								if lane["mode"]["splitAbs"]["count"] >= 2 then
									return false
								end
							end
							return true
						end,
					},
					saSplit2Position = {
						name = "Split 2 Position",
						desc = "What position should splitAbs 2 appear",
						order = 521,
						type = "range",
						hidden = function(info)
							if lane["mode"]["type"] == "SPLIT_ABS" then
								if lane["mode"]["splitAbs"]["count"] >= 2 then
									return false
								end
							end
							return true
						end,
						min = 0,
						max = 1,
						get = function(info, index)
								return lane["mode"]["splitAbs"]["s2p"]
							end,
						set = function(info, val)
								lane["mode"]["splitAbs"]["s2p"] = val
								CDTL2:RefreshLane(i)
						end,
					},
					saSplit2Value = {
						name = "Split 2 Value",
						desc = "What percentage should splitAbs 2 occur",
						order = 522,
						type = "range",
						hidden = function(info)
							if lane["mode"]["type"] == "SPLIT_ABS" then
								if lane["mode"]["splitAbs"]["count"] >= 2 then
									return false
								end
							end
							return true
						end,
						min = 1,
						max = 360,
						bigStep = 1,
						get = function(info, index)
								return lane["mode"]["splitAbs"]["s2v"]
							end,
						set = function(info, val)
								lane["mode"]["splitAbs"]["s2v"] = val
								CDTL2:RefreshLane(i)
						end,
					},
					spacer530 = {
						name = "",
						type = "description",
						order = 530,
						hidden = function(info)
							if lane["mode"]["type"] == "SPLIT_ABS" then
								if lane["mode"]["splitAbs"]["count"] >= 2 then
									return false
								end
							end
							return true
						end,
					},
					saSplit3Position = {
						name = "Split 3 Position",
						desc = "What position should splitAbs 3 appear",
						order = 531,
						type = "range",
						hidden = function(info)
							if lane["mode"]["type"] == "SPLIT_ABS" then
								if lane["mode"]["splitAbs"]["count"] >= 3 then
									return false
								end
							end
							return true
						end,
						min = 0,
						max = 1,
						get = function(info, index)
								return lane["mode"]["splitAbs"]["s3p"]
							end,
						set = function(info, val)
								lane["mode"]["splitAbs"]["s3p"] = val
								CDTL2:RefreshLane(i)
						end,
					},
					saSplit3Value = {
						name = "Split 3 Value",
						desc = "What percentage should splitAbs 3 occur",
						order = 532,
						type = "range",
						hidden = function(info)
							if lane["mode"]["type"] == "SPLIT_ABS" then
								if lane["mode"]["splitAbs"]["count"] >= 3 then
									return false
								end
							end
							return true
						end,
						max = 360,
						min = 1,
						bigStep = 1,
						get = function(info, index)
								return lane["mode"]["splitAbs"]["s3v"]
							end,
						set = function(info, val)
								lane["mode"]["splitAbs"]["s3v"] = val
								CDTL2:RefreshLane(i)
						end,
					},					
					spacer600 = {
						name = "\n\n",
						type = "description",
						order = 600,
					},
					primaryTracking = {
						name = "Primary Tracking",
						desc = "",
						order = 601,
						type = "select",
						values = {
								["NONE"] = "None",
								["GCD"] = "GCD",
								["HEALTH"] = "Health",
								["CLASS_POWER"] = "Class Power",
							},
						get = function(info, index)
								return lane["tracking"]["primaryTracking"]
							end,
						set = function(info, val)
								lane["tracking"]["primaryTracking"] = val
							end,
					},
					secondaryTracking = {
						name = "Secondary Tracking",
						desc = "",
						order = 602,
						type = "select",
						values = {
								["NONE"] = "None",
								["GCD"] = "GCD",
								["HEALTH"] = "Health",
								["CLASS_POWER"] = "Class Power",
							},
						get = function(info, index)
								return lane["tracking"]["secondaryTracking"]
							end,
						set = function(info, val)
								lane["tracking"]["secondaryTracking"] = val
							end,
					},
				},
			},
			appearance = {
				name = "Appearance",
				type = "group",
				order = 200,
				args = {
					--[[spacer200 = {
						name = "",
						type = "description",
						order = 200,
					},]]--
					width = {
						name = "Width",
						desc = "Sets the width for the lane",
						order = 201,
						type = "range",
						softMin = 0,
						softMax = 600,
						get = function(info, index)
								return lane["width"]
							end,
						set = function(info, val)
								lane["width"] = val
								CDTL2:RefreshLane(i)
						end,
					},
					height = {
						name = "Height",
						desc = "Sets the height for the lane",
						order = 202,
						type = "range",
						softMin = 0,
						softMax = 600,
						get = function(info, index)
								return lane["height"]
							end,
						set = function(info, val)
								lane["height"] = val
								CDTL2:RefreshLane(i)
							end,
					},
					spacer300 = {
						name = "",
						type = "description",
						order = 300,
					},
					posX = {
						name = "x Position",
						desc = "Sets the x position of the lane",
						order = 301,
						type = "range",
						softMin = -500,
						softMax = 500,
						get = function(info, index)
								return lane["posX"]
							end,
						set = function(info, val)
								lane["posX"] = val
								CDTL2:RefreshLane(i)
							end,
					},
					posY = {
						name = "y Position",
						desc = "Sets the y position of the lane",
						order = 302,
						type = "range",
						softMin = -500,
						softMax = 500,
						get = function(info, index)
								return lane["posY"]
							end,
						set = function(info, val)
								lane["posY"] = val
								CDTL2:RefreshLane(i)
							end,
					},
					spacer400 = {
						name = "",
						type = "description",
						order = 400,
					},
					relativeTo = {
						name = "Anchor Point",
						desc = "The x/y position is relative to this point of the screen",
						order = 401,
						type = "select",
						values = {
								["TOPLEFT"] = "TOPLEFT",
								["TOP"] = "TOP",
								["TOPRIGHT"] = "TOPRIGHT",
								["LEFT"] = "LEFT",
								["CENTER"] = "CENTER",
								["RIGHT"] = "RIGHT",
								["BOTTOMLEFT"] = "BOTTOMLEFT",
								["BOTTOM"] = "BOTTOM",
								["BOTTOMRIGHT"] = "BOTTOMRIGHT",
							},
						get = function(info, index)
								return lane["relativeTo"]
							end,
						set = function(info, val)
								lane["relativeTo"] = val
								CDTL2:RefreshLane(i)
							end,
					},
					spacer500 = {
						name = "\n\n",
						type = "description",
						order = 500,
					},
					fgTexture = {
						name = "Foreground Texture",
						desc = "Selects the texture",
						order = 501,
						type = "select",
						dialogControl = 'LSM30_Statusbar',
						values = AceGUIWidgetLSMlists.statusbar,
						get = function(info, index)
								return lane["fgTexture"]
							end,
						set = function(info, val)
								lane["fgTexture"] = val
								CDTL2:RefreshLane(i)
							end,
					},
					fgTextureColor = {
						name = "Color",
						desc = "Sets the texture color",
						order = 502,
						type = "color",
						hasAlpha = true,
						get = function(info)
								local r = lane["fgTextureColor"]["r"]
								local g = lane["fgTextureColor"]["g"]
								local b = lane["fgTextureColor"]["b"]
								local a = lane["fgTextureColor"]["a"]
								return r, g, b, a
							end,
						set = function(info, red, green, blue, alpha)
								lane["fgTextureColor"] = { r = red, g = green, b = blue, a = alpha }
								CDTL2:RefreshLane(i)
							end,
					},
					spacer600 = {
						name = "",
						type = "description",
						order = 600,
					},
					bgTexture = {
						name = "Background Texture",
						desc = "Sets the background texture",
						order = 601,
						type = "select",
						dialogControl = 'LSM30_Statusbar',
						values = AceGUIWidgetLSMlists.statusbar,
						get = function(info, index)
								return lane["bgTexture"]
							end,
						set = function(info, val)
								lane["bgTexture"] = val
								CDTL2:RefreshLane(i)
							end,
					},
					bgTextureColor = {
						name = "Color",
						desc = "Sets the background texture color",
						order = 602,
						type = "color",
						hasAlpha = true,
						get = function(info)
								local r = lane["bgTextureColor"]["r"]
								local g = lane["bgTextureColor"]["g"]
								local b = lane["bgTextureColor"]["b"]
								local a = lane["bgTextureColor"]["a"]
								return r, g, b, a
							end,
						set = function(info, red, green, blue, alpha)
								lane["bgTextureColor"] = { r = red, g = green, b = blue, a = alpha }
								CDTL2:RefreshLane(i)
							end,
					},
					spacer700 = {
						name = "\n\n",
						type = "description",
						order = 700,
					},
					border = {
						name = "                    Border Texture",
						desc = "Selects the border texture",
						order = 701,
						type = "select",
						dialogControl = 'LSM30_Border',
						values = AceGUIWidgetLSMlists.border,
						get = function(info) return lane["border"]["style"] end,
						set = function(info, val)
								lane["border"]["style"] = val
								CDTL2:RefreshLane(i)
							end,
					},
					borderColor = {
						name = "Color",
						desc = "Selects the border color",
						order = 702,
						type = "color",
						hasAlpha = true,
						get = function(info)
								local r = lane["border"]["color"]["r"]
								local g = lane["border"]["color"]["g"]
								local b = lane["border"]["color"]["b"]
								local a = lane["border"]["color"]["a"]
								
								return r, g, b, a
							end,
						set = function(info, red, green, blue, alpha)
								lane["border"]["color"] = { r = red, g = green, b = blue, a = alpha }							
								CDTL2:RefreshLane(i)
							end,
					},
					spacer710 = {
						name = "",
						type = "description",
						order = 710,
					},
					borderSize = {
						name = "Size",
						desc = "Sets the size of the border",
						order = 711,
						type = "range",
						softMin = 1,
						softMax = 40,
						get = function(info) return lane["border"]["size"] end,
						set = function(info, val)
								lane["border"]["size"] = val
								CDTL2:RefreshLane(i)
							end,
					},
					borderPadding = {
						name = "Padding",
						desc = "Sets the size of the border",
						order = 711,
						type = "range",
						softMin = 0,
						softMax = 40,
						get = function(info) return lane["border"]["padding"] end,
						set = function(info, val)
								lane["border"]["padding"] = val
								CDTL2:RefreshLane(i)
							end,
					},
				},
			},
			icons = {
				name = "Icons",
				type = "group",
				order = 300,
				args = {
					--[[spacer100 = {
						name = "\n\n",
						type = "description",
						order = 100,
					},]]--
					size = {
						name = "Size",
						desc = "Sets the size of icons in lane 1",
						order = 101.1,
						type = "range",
						softMin = 1,
						softMax = 128,
						get = function(info, index)
								return lane["icons"]["size"]
							end,
						set = function(info, val)
								lane["icons"]["size"] = val
								CDTL2:RefreshAllIcons()
						end,
					},
					spacer102 = {
						name = "\n\n",
						type = "description",
						order = 102,
					},
					textHeader = {
						name = "Icon Text",
						type = "header",
						order = 200.1,
					},
					spacer200 = {
						name = "\n",
						type = "description",
						order = 200.2,
					},
					-- ICON TEXT 1
					--iText1S0 = { name = "\n", type = "description", order = 210, },
					iText1Text = private.GetTextText(lane["icons"]["text1"], 210, i, "ICON", false),
					iText1Enabled = private.GetTextEnabled(lane["icons"]["text1"], 210, i, "ICON"),
					iText1Edit = private.GetTextEdit(lane["icons"]["text1"], 210, i, "ICON"),
					iText1Font = private.GetTextFont(lane["icons"]["text1"], 210, i, "ICON"),
					iText1Outline = private.GetTextOutline(lane["icons"]["text1"], 210, i, "ICON"),
					iText1S1 = { name = "", type = "description", order = 210 + 4.3, hidden = function(info) return not lane["icons"]["text1"]["edit"] end, },
					iText1Size = private.GetTextSize(lane["icons"]["text1"], 210, i, "ICON"),
					iText1Color = private.GetTextColor(lane["icons"]["text1"], 210, i, "ICON"),
					iText1ShadowColor = private.GetTextShadowColor(lane["icons"]["text1"], 210, i, "ICON"),
					iText1S2 = { name = "", type = "description", order = 210 + 4.7, hidden = function(info) return not lane["icons"]["text1"]["edit"] end, },
					iText1ShadowXOffset = private.GetTextShadowXOffset(lane["icons"]["text1"], 210, i, "ICON"),
					iText1ShadowYOffset = private.GetTextShadowYOffset(lane["icons"]["text1"], 210, i, "ICON"),
					iText1S3 = { name = "", type = "description", order = 210 + 5.1, hidden = function(info) return not lane["icons"]["text1"]["edit"] end, },
					iText1Anchor = private.GetTextAnchor(lane["icons"]["text1"], 210, i, "ICON"),
					iText1Align = private.GetTextAlign(lane["icons"]["text1"], 210, i, "ICON"),
					iText1S4 = { name = "", type = "description", order = 210 + 5.4, hidden = function(info) return not lane["icons"]["text1"]["edit"] end, },
					iText1XOffset = private.GetTextXOffset(lane["icons"]["text1"], 210, i, "ICON"),
					iText1YOffset = private.GetTextYOffset(lane["icons"]["text1"], 210, i, "ICON"),
					iText1Footer = { name = "", type = "header", order = 210 + 9.8, hidden = function(info) return not lane["icons"]["text1"]["edit"] end, },
					-- ICON TEXT 2
					iText2S0 = { name = "\n", type = "description", order = 220, hidden = function(info) return not lane["icons"]["text2"]["used"] end, },
					iText2Header = { name = "", type = "header", order = 220 + 0.1, hidden = function(info) return not lane["icons"]["text2"]["edit"] end, },
					iText2Text = private.GetTextText(lane["icons"]["text2"], 220, i, "ICON", false),
					iText2Enabled = private.GetTextEnabled(lane["icons"]["text2"], 220, i, "ICON"),
					iText2Edit = private.GetTextEdit(lane["icons"]["text2"], 220, i, "ICON"),
					iText2Font = private.GetTextFont(lane["icons"]["text2"], 220, i, "ICON"),
					iText2Outline = private.GetTextOutline(lane["icons"]["text2"], 220, i, "ICON"),
					iText2S1 = { name = "", type = "description", order = 220 + 4.3, hidden = function(info) return not lane["icons"]["text2"]["edit"] end, },
					iText2Size = private.GetTextSize(lane["icons"]["text2"], 220, i, "ICON"),
					iText2Color = private.GetTextColor(lane["icons"]["text2"], 220, i, "ICON"),
					iText2ShadowColor = private.GetTextShadowColor(lane["icons"]["text2"], 220, i, "ICON"),
					iText2S2 = { name = "", type = "description", order = 220 + 4.7, hidden = function(info) return not lane["icons"]["text2"]["edit"] end, },
					iText2ShadowXOffset = private.GetTextShadowXOffset(lane["icons"]["text2"], 220, i, "ICON"),
					iText2ShadowYOffset = private.GetTextShadowYOffset(lane["icons"]["text2"], 220, i, "ICON"),
					iText2S3 = { name = "", type = "description", order = 220 + 5.1, hidden = function(info) return not lane["icons"]["text2"]["edit"] end, },
					iText2Anchor = private.GetTextAnchor(lane["icons"]["text2"], 220, i, "ICON"),
					iText2Align = private.GetTextAlign(lane["icons"]["text2"], 220, i, "ICON"),
					iText2S4 = { name = "", type = "description", order = 220 + 5.4, hidden = function(info) return not lane["icons"]["text2"]["edit"] end, },
					iText2XOffset = private.GetTextXOffset(lane["icons"]["text2"], 220, i, "ICON"),
					iText2YOffset = private.GetTextYOffset(lane["icons"]["text2"], 220, i, "ICON"),
					iText2Footer = { name = "", type = "header", order = 220 + 9.8, hidden = function(info) return not lane["icons"]["text2"]["edit"] end, },
					-- ICON TEXT 3
					iText3S0 = { name = "\n", type = "description", order = 230, hidden = function(info) return not lane["icons"]["text3"]["used"] end, },
					iText3Header = { name = "", type = "header", order = 230 + 0.1, hidden = function(info) return not lane["icons"]["text3"]["edit"] end, },
					iText3Text = private.GetTextText(lane["icons"]["text3"], 230, i, "ICON", false),
					iText3Enabled = private.GetTextEnabled(lane["icons"]["text3"], 230, i, "ICON"),
					iText3Edit = private.GetTextEdit(lane["icons"]["text3"], 230, i, "ICON"),
					iText3Font = private.GetTextFont(lane["icons"]["text3"], 230, i, "ICON"),
					iText3Outline = private.GetTextOutline(lane["icons"]["text3"], 230, i, "ICON"),
					iText3S1 = { name = "", type = "description", order = 230 + 4.3, hidden = function(info) return not lane["icons"]["text3"]["edit"] end, },
					iText3Size = private.GetTextSize(lane["icons"]["text3"], 230, i, "ICON"),
					iText3Color = private.GetTextColor(lane["icons"]["text3"], 230, i, "ICON"),
					iText3ShadowColor = private.GetTextShadowColor(lane["icons"]["text3"], 230, i, "ICON"),
					iText3S2 = { name = "", type = "description", order = 230 + 4.7, hidden = function(info) return not lane["icons"]["text3"]["edit"] end, },
					iText3ShadowXOffset = private.GetTextShadowXOffset(lane["icons"]["text3"], 230, i, "ICON"),
					iText3ShadowYOffset = private.GetTextShadowYOffset(lane["icons"]["text3"], 230, i, "ICON"),
					iText3S3 = { name = "", type = "description", order = 230 + 5.1, hidden = function(info) return not lane["icons"]["text3"]["edit"] end, },
					iText3Anchor = private.GetTextAnchor(lane["icons"]["text3"], 230, i, "ICON"),
					iText3Align = private.GetTextAlign(lane["icons"]["text3"], 230, i, "ICON"),
					iText3S4 = { name = "", type = "description", order = 230 + 5.4, hidden = function(info) return not lane["icons"]["text3"]["edit"] end, },
					iText3XOffset = private.GetTextXOffset(lane["icons"]["text3"], 230, i, "ICON"),
					iText3YOffset = private.GetTextYOffset(lane["icons"]["text3"], 230, i, "ICON"),
					iText3Footer = { name = "", type = "header", order = 230 + 9.8, hidden = function(info) return not lane["icons"]["text3"]["edit"] end, },
					spacer298 = {
						name = "\n",
						type = "description",
						order = 298,
					},
					textFooter = {
						name = "",
						type = "header",
						order = 299,
					},
					spacer300 = {
						name = "\n\n",
						type = "description",
						order = 300,
					},
					border = {
						name = "                    Border Texture",
						desc = "Selects the border texture",
						order = 301,
						type = "select",
						dialogControl = 'LSM30_Border',
						values = AceGUIWidgetLSMlists.border,
						get = function(info) return lane["icons"]["border"]["style"] end,
						set = function(info, val)
								lane["icons"]["border"]["style"] = val
								CDTL2:RefreshAllIcons()
							end,
					},
					borderColor = {
						name = "Color",
						desc = "Selects the border color",
						order = 302,
						type = "color",
						hasAlpha = true,
						get = function(info)
								local r = lane["icons"]["border"]["color"]["r"]
								local g = lane["icons"]["border"]["color"]["g"]
								local b = lane["icons"]["border"]["color"]["b"]
								local a = lane["icons"]["border"]["color"]["a"]
								
								return r, g, b, a
							end,
						set = function(info, red, green, blue, alpha)
								lane["icons"]["border"]["color"] = { r = red, g = green, b = blue, a = alpha }							
								CDTL2:RefreshAllIcons()
							end,
					},
					spacer310 = {
						name = "",
						type = "description",
						order = 310,
					},
					borderSize = {
						name = "Size",
						desc = "Sets the size of the border",
						order = 311,
						type = "range",
						softMin = 1,
						softMax = 40,
						get = function(info) return lane["icons"]["border"]["size"] end,
						set = function(info, val)
								lane["icons"]["border"]["size"] = val
								CDTL2:RefreshAllIcons()
							end,
					},
					borderPadding = {
						name = "Padding",
						desc = "Sets the size of the border",
						order = 311,
						type = "range",
						softMin = 0,
						softMax = 40,
						get = function(info) return lane["icons"]["border"]["padding"] end,
						set = function(info, val)
								lane["icons"]["border"]["padding"] = val
								CDTL2:RefreshAllIcons()
							end,
					},
				}
			},
			stacking = {
				name = "Stacking",
				type = "group",
				order = 400,
				args = {
					--[[spacer100 = {
						name = "\n\n",
						type = "description",
						order = 100,
					},]]--
					enabled = {
						name = "Enabled",
						desc = "Enable icon stacking",
						order = 101,
						type = "toggle",
						width = 0.5,
						get = function(info)
								return lane["stacking"]["enabled"]
							end,
						set = function(info, val)
								lane["stacking"]["enabled"] = val
							end,
					},
					raise = {
						name = "Raise On Mouseover",
						desc = "Mousing over an icon will raise it to the top of its stack and make it easier to see",
						order = 102,
						type = "toggle",
						--width = 0.5,
						get = function(info)
								return lane["stacking"]["raiseOnMouseOver"]
							end,
						set = function(info, val)
								lane["stacking"]["raiseOnMouseOver"] = val
							end,
					},
					spacer200 = {
						name = "\n\n",
						type = "description",
						order = 200,
					},
					style = {
						name = "Stack Style",
						desc = "Select which direction new icons will grow in",
						order = 201,
						type = "select",
						values = {
								["GROUPED"] = "Grouped",
								["OFFSET"] = "Offset",
							},
						get = function(info, index)
								return lane["stacking"]["style"]
							end,
						set = function(info, val)
								lane["stacking"]["style"] = val
							end,
					},
					grow = {
						name = "Grow Direction",
						desc = "What direction should icon stacks grow",
						order = 202,
						type = "select",
						values = {
								["UP"] = "Up",
								["DOWN"] = "Down",
								["CENTER"] = "Center",
							},
						get = function(info, index)
								return lane["stacking"]["grow"]
							end,
						set = function(info, val)
								lane["stacking"]["grow"] = val
							end,
					},
					spacer300 = {
						name = "\n\n",
						type = "description",
						order = 300,
					},
					height = {
						name = "Height",
						desc = "Sets the max height of the stack",
						order = 301,
						type = "range",
						softMin = 0,
						softMax = 100,
						get = function(info, index)
								return lane["stacking"]["height"]
							end,
						set = function(info, val)
								lane["stacking"]["height"] = val
						end,
					},
				}
			},
			text = {
				name = "Text",
				type = "group",
				order = 500,
				args = {
					defaultHeader = {
						name = "Default Text",
						type = "header",
						order = 100,
					},
					--[[spacer200 = {
						name = "\n",
						type = "description",
						order = 200,
					},]]--
					-- DEFAULT TEXT 1
					--dText1S0 = { name = "\n", type = "description", order = 210, },
					dText1Text = private.GetTextText(lane["modeText"]["text1"], 210, i, "LANE", true),
					dText1Enabled = private.GetTextEnabled(lane["modeText"]["text1"], 210, i, "LANE"),
					dText1Edit = private.GetTextEdit(lane["modeText"]["text1"], 210, i, "LANE"),
					dText1Font = private.GetTextFont(lane["modeText"]["text1"], 210, i, "LANE"),
					dText1Outline = private.GetTextOutline(lane["modeText"]["text1"], 210, i, "LANE"),
					dText1S1 = { name = "", type = "description", order = 210 + 4.3, hidden = function(info) return not lane["modeText"]["text1"]["edit"] end, },
					dText1Size = private.GetTextSize(lane["modeText"]["text1"], 210, i, "LANE"),
					dText1Color = private.GetTextColor(lane["modeText"]["text1"], 210, i, "LANE"),
					dText1ShadowColor = private.GetTextShadowColor(lane["modeText"]["text1"], 210, i, "LANE"),
					dText1S2 = { name = "", type = "description", order = 210 + 4.7, hidden = function(info) return not lane["modeText"]["text1"]["edit"] end, },
					dText1ShadowXOffset = private.GetTextShadowXOffset(lane["modeText"]["text1"], 210, i, "LANE"),
					dText1ShadowYOffset = private.GetTextShadowYOffset(lane["modeText"]["text1"], 210, i, "LANE"),
					dText1S3 = { name = "", type = "description", order = 210 + 5.1, hidden = function(info) return not lane["modeText"]["text1"]["edit"] end, },
					dText1Anchor = private.GetTextAnchor(lane["modeText"]["text1"], 210, i, "LANE"),
					dText1Align = private.GetTextAlign(lane["modeText"]["text1"], 210, i, "LANE"),
					dText1S4 = { name = "", type = "description", order = 210 + 5.4, hidden = function(info) return not lane["modeText"]["text1"]["edit"] end, },
					dText1XOffset = private.GetTextXOffset(lane["modeText"]["text1"], 210, i, "LANE"),
					dText1YOffset = private.GetTextYOffset(lane["modeText"]["text1"], 210, i, "LANE"),
					dText1Footer = { name = "", type = "header", order = 210 + 9.8, hidden = function(info) return not lane["modeText"]["text1"]["edit"] end, },
					-- DEFAULT TEXT 2
					dText2S0 = { name = "\n", type = "description", order = 220, hidden = function(info) return not lane["modeText"]["text2"]["used"] end, },
					dText2Header = { name = "", type = "header", order = 220 + 0.1, hidden = function(info) return not lane["modeText"]["text2"]["edit"] end, },
					dText2Text = private.GetTextText(lane["modeText"]["text2"], 220, i, "LANE", true),
					dText2Enabled = private.GetTextEnabled(lane["modeText"]["text2"], 220, i, "LANE"),
					dText2Edit = private.GetTextEdit(lane["modeText"]["text2"], 220, i, "LANE"),
					dText2Font = private.GetTextFont(lane["modeText"]["text2"], 220, i, "LANE"),
					dText2Outline = private.GetTextOutline(lane["modeText"]["text2"], 220, i, "LANE"),
					dText2S1 = { name = "", type = "description", order = 220 + 4.3, hidden = function(info) return not lane["modeText"]["text2"]["edit"] end, },
					dText2Size = private.GetTextSize(lane["modeText"]["text2"], 220, i, "LANE"),
					dText2Color = private.GetTextColor(lane["modeText"]["text2"], 220, i, "LANE"),
					dText2ShadowColor = private.GetTextShadowColor(lane["modeText"]["text2"], 220, i, "LANE"),
					dText2S2 = { name = "", type = "description", order = 220 + 4.7, hidden = function(info) return not lane["modeText"]["text2"]["edit"] end, },
					dText2ShadowXOffset = private.GetTextShadowXOffset(lane["modeText"]["text2"], 220, i, "LANE"),
					dText2ShadowYOffset = private.GetTextShadowYOffset(lane["modeText"]["text2"], 220, i, "LANE"),
					dText2S3 = { name = "", type = "description", order = 220 + 5.1, hidden = function(info) return not lane["modeText"]["text2"]["edit"] end, },
					dText2Anchor = private.GetTextAnchor(lane["modeText"]["text2"], 220, i, "LANE"),
					dText2Align = private.GetTextAlign(lane["modeText"]["text2"], 220, i, "LANE"),
					dText2S4 = { name = "", type = "description", order = 220 + 5.4, hidden = function(info) return not lane["modeText"]["text2"]["edit"] end, },
					dText2XOffset = private.GetTextXOffset(lane["modeText"]["text2"], 220, i, "LANE"),
					dText2YOffset = private.GetTextYOffset(lane["modeText"]["text2"], 220, i, "LANE"),
					dText2Footer = { name = "", type = "header", order = 220 + 9.8, hidden = function(info) return not lane["modeText"]["text2"]["edit"] end, },
					-- DEFAULT TEXT 3
					dText3S0 = { name = "\n", type = "description", order = 230, hidden = function(info) return not lane["modeText"]["text3"]["used"] end, },
					dText3Header = { name = "", type = "header", order = 230 + 0.1, hidden = function(info) return not lane["modeText"]["text3"]["edit"] end, },
					dText3Text = private.GetTextText(lane["modeText"]["text3"], 230, i, "LANE", true),
					dText3Enabled = private.GetTextEnabled(lane["modeText"]["text3"], 230, i, "LANE"),
					dText3Edit = private.GetTextEdit(lane["modeText"]["text3"], 230, i, "LANE"),
					dText3Font = private.GetTextFont(lane["modeText"]["text3"], 230, i, "LANE"),
					dText3Outline = private.GetTextOutline(lane["modeText"]["text3"], 230, i, "LANE"),
					dText3S1 = { name = "", type = "description", order = 230 + 4.3, hidden = function(info) return not lane["modeText"]["text3"]["edit"] end, },
					dText3Size = private.GetTextSize(lane["modeText"]["text3"], 230, i, "LANE"),
					dText3Color = private.GetTextColor(lane["modeText"]["text3"], 230, i, "LANE"),
					dText3ShadowColor = private.GetTextShadowColor(lane["modeText"]["text3"], 230, i, "LANE"),
					dText3S2 = { name = "", type = "description", order = 230 + 4.7, hidden = function(info) return not lane["modeText"]["text3"]["edit"] end, },
					dText3ShadowXOffset = private.GetTextShadowXOffset(lane["modeText"]["text3"], 230, i, "LANE"),
					dText3ShadowYOffset = private.GetTextShadowYOffset(lane["modeText"]["text3"], 230, i, "LANE"),
					dText3S3 = { name = "", type = "description", order = 230 + 5.1, hidden = function(info) return not lane["modeText"]["text3"]["edit"] end, },
					dText3Anchor = private.GetTextAnchor(lane["modeText"]["text3"], 230, i, "LANE"),
					dText3Align = private.GetTextAlign(lane["modeText"]["text3"], 230, i, "LANE"),
					dText3S4 = { name = "", type = "description", order = 230 + 5.4, hidden = function(info) return not lane["modeText"]["text3"]["edit"] end, },
					dText3XOffset = private.GetTextXOffset(lane["modeText"]["text3"], 230, i, "LANE"),
					dText3YOffset = private.GetTextYOffset(lane["modeText"]["text3"], 230, i, "LANE"),
					dText3Footer = { name = "", type = "header", order = 230 + 9.8, hidden = function(info) return not lane["modeText"]["text3"]["edit"] end, },
					-- DEFAULT TEXT 4
					dText4S0 = { name = "\n", type = "description", order = 240, hidden = function(info) return not lane["modeText"]["text4"]["used"] end, },
					dText4Header = { name = "", type = "header", order = 240 + 0.1, hidden = function(info) return not lane["modeText"]["text4"]["edit"] end, },
					dText4Text = private.GetTextText(lane["modeText"]["text4"], 240, i, "LANE", true),
					dText4Enabled = private.GetTextEnabled(lane["modeText"]["text4"], 240, i, "LANE"),
					dText4Edit = private.GetTextEdit(lane["modeText"]["text4"], 240, i, "LANE"),
					dText4Font = private.GetTextFont(lane["modeText"]["text4"], 240, i, "LANE"),
					dText4Outline = private.GetTextOutline(lane["modeText"]["text4"], 240, i, "LANE"),
					dText4S1 = { name = "", type = "description", order = 240 + 4.3, hidden = function(info) return not lane["modeText"]["text4"]["edit"] end, },
					dText4Size = private.GetTextSize(lane["modeText"]["text4"], 240, i, "LANE"),
					dText4Color = private.GetTextColor(lane["modeText"]["text4"], 240, i, "LANE"),
					dText4ShadowColor = private.GetTextShadowColor(lane["modeText"]["text4"], 240, i, "LANE"),
					dText4S2 = { name = "", type = "description", order = 240 + 4.7, hidden = function(info) return not lane["modeText"]["text4"]["edit"] end, },
					dText4ShadowXOffset = private.GetTextShadowXOffset(lane["modeText"]["text4"], 240, i, "LANE"),
					dText4ShadowYOffset = private.GetTextShadowYOffset(lane["modeText"]["text4"], 240, i, "LANE"),
					dText4S3 = { name = "", type = "description", order = 240 + 5.1, hidden = function(info) return not lane["modeText"]["text4"]["edit"] end, },
					dText4Anchor = private.GetTextAnchor(lane["modeText"]["text4"], 240, i, "LANE"),
					dText4Align = private.GetTextAlign(lane["modeText"]["text4"], 240, i, "LANE"),
					dText4S4 = { name = "", type = "description", order = 240 + 5.4, hidden = function(info) return not lane["modeText"]["text4"]["edit"] end, },
					dText4XOffset = private.GetTextXOffset(lane["modeText"]["text4"], 240, i, "LANE"),
					dText4YOffset = private.GetTextYOffset(lane["modeText"]["text4"], 240, i, "LANE"),
					dText4Footer = { name = "", type = "header", order = 240 + 9.8, hidden = function(info) return not lane["modeText"]["text4"]["edit"] end, },
					-- DEFAULT TEXT 5
					dText5S0 = { name = "\n", type = "description", order = 250, hidden = function(info) return not lane["modeText"]["text5"]["used"] end, },
					dText5Header = { name = "", type = "header", order = 250 + 0.1, hidden = function(info) return not lane["modeText"]["text5"]["edit"] end, },
					dText5Text = private.GetTextText(lane["modeText"]["text5"], 250, i, "LANE", true),
					dText5Enabled = private.GetTextEnabled(lane["modeText"]["text5"], 250, i, "LANE"),
					dText5Edit = private.GetTextEdit(lane["modeText"]["text5"], 250, i, "LANE"),
					dText5Font = private.GetTextFont(lane["modeText"]["text5"], 250, i, "LANE"),
					dText5Outline = private.GetTextOutline(lane["modeText"]["text5"], 250, i, "LANE"),
					dText5S1 = { name = "", type = "description", order = 250 + 4.3, hidden = function(info) return not lane["modeText"]["text5"]["edit"] end, },
					dText5Size = private.GetTextSize(lane["modeText"]["text5"], 250, i, "LANE"),
					dText5Color = private.GetTextColor(lane["modeText"]["text5"], 250, i, "LANE"),
					dText5ShadowColor = private.GetTextShadowColor(lane["modeText"]["text5"], 250, i, "LANE"),
					dText5S2 = { name = "", type = "description", order = 250 + 4.7, hidden = function(info) return not lane["modeText"]["text5"]["edit"] end, },
					dText5ShadowXOffset = private.GetTextShadowXOffset(lane["modeText"]["text5"], 250, i, "LANE"),
					dText5ShadowYOffset = private.GetTextShadowYOffset(lane["modeText"]["text5"], 250, i, "LANE"),
					dText5S3 = { name = "", type = "description", order = 250 + 5.1, hidden = function(info) return not lane["modeText"]["text5"]["edit"] end, },
					dText5Anchor = private.GetTextAnchor(lane["modeText"]["text5"], 250, i, "LANE"),
					dText5Align = private.GetTextAlign(lane["modeText"]["text5"], 250, i, "LANE"),
					dText5S4 = { name = "", type = "description", order = 250 + 5.4, hidden = function(info) return not lane["modeText"]["text5"]["edit"] end, },
					dText5XOffset = private.GetTextXOffset(lane["modeText"]["text5"], 250, i, "LANE"),
					dText5YOffset = private.GetTextYOffset(lane["modeText"]["text5"], 250, i, "LANE"),
					--dText5Footer = { name = "", type = "header", order = 250 + 9.8, hidden = function(info) return not lane["modeText"]["text5"]["edit"] end, },
					customHeader = {
						name = "Custom Text",
						type = "header",
						order = 300,
					},
					-- CUSTOM TEXT 1
					cText1S0 = { name = "\n", type = "description", order = 310, },
					cText1Text = private.GetTextText(lane["customText"]["text1"], 310, i, "LANE"),
					cText1Enabled = private.GetTextEnabled(lane["customText"]["text1"], 310, i, "LANE"),
					cText1Edit = private.GetTextEdit(lane["customText"]["text1"], 310, i, "LANE"),
					cText1Font = private.GetTextFont(lane["customText"]["text1"], 310, i, "LANE"),
					cText1Outline = private.GetTextOutline(lane["customText"]["text1"], 310, i, "LANE"),
					cText1S1 = { name = "", type = "description", order = 310 + 4.3, hidden = function(info) return not lane["customText"]["text1"]["edit"] end, },
					cText1Size = private.GetTextSize(lane["customText"]["text1"], 310, i, "LANE"),
					cText1Color = private.GetTextColor(lane["customText"]["text1"], 310, i, "LANE"),
					cText1ShadowColor = private.GetTextShadowColor(lane["customText"]["text1"], 310, i, "LANE"),
					cText1S2 = { name = "", type = "description", order = 310 + 4.7, hidden = function(info) return not lane["customText"]["text1"]["edit"] end, },
					cText1ShadowXOffset = private.GetTextShadowXOffset(lane["customText"]["text1"], 310, i, "LANE"),
					cText1ShadowYOffset = private.GetTextShadowYOffset(lane["customText"]["text1"], 310, i, "LANE"),
					cText1S3 = { name = "", type = "description", order = 310 + 5.1, hidden = function(info) return not lane["customText"]["text1"]["edit"] end, },
					cText1Anchor = private.GetTextAnchor(lane["customText"]["text1"], 310, i, "LANE"),
					cText1Align = private.GetTextAlign(lane["customText"]["text1"], 310, i, "LANE"),
					cText1S4 = { name = "", type = "description", order = 310 + 5.4, hidden = function(info) return not lane["customText"]["text1"]["edit"] end, },
					cText1XOffset = private.GetTextXOffset(lane["customText"]["text1"], 310, i, "LANE"),
					cText1YOffset = private.GetTextYOffset(lane["customText"]["text1"], 310, i, "LANE"),
					cText1Footer = { name = "", type = "header", order = 310 + 9.8, hidden = function(info) return not lane["customText"]["text1"]["edit"] end, },
					-- CUSTOM TEXT 2
					cText2S0 = { name = "\n", type = "description", order = 320, hidden = function(info) return not lane["customText"]["text2"]["used"] end, },
					cText2Header = { name = "", type = "header", order = 320 + 0.1, hidden = function(info) return not lane["customText"]["text2"]["edit"] end, },
					cText2Text = private.GetTextText(lane["customText"]["text2"], 320, i, "LANE"),
					cText2Enabled = private.GetTextEnabled(lane["customText"]["text2"], 320, i, "LANE"),
					cText2Edit = private.GetTextEdit(lane["customText"]["text2"], 320, i, "LANE"),
					cText2Font = private.GetTextFont(lane["customText"]["text2"], 320, i, "LANE"),
					cText2Outline = private.GetTextOutline(lane["customText"]["text2"], 320, i, "LANE"),
					cText2S1 = { name = "", type = "description", order = 320 + 4.3, hidden = function(info) return not lane["customText"]["text2"]["edit"] end, },
					cText2Size = private.GetTextSize(lane["customText"]["text2"], 320, i, "LANE"),
					cText2Color = private.GetTextColor(lane["customText"]["text2"], 320, i, "LANE"),
					cText2ShadowColor = private.GetTextShadowColor(lane["customText"]["text2"], 320, i, "LANE"),
					cText2S2 = { name = "", type = "description", order = 320 + 4.7, hidden = function(info) return not lane["customText"]["text2"]["edit"] end, },
					cText2ShadowXOffset = private.GetTextShadowXOffset(lane["customText"]["text2"], 320, i, "LANE"),
					cText2ShadowYOffset = private.GetTextShadowYOffset(lane["customText"]["text2"], 320, i, "LANE"),
					cText2S3 = { name = "", type = "description", order = 320 + 5.1, hidden = function(info) return not lane["customText"]["text2"]["edit"] end, },
					cText2Anchor = private.GetTextAnchor(lane["customText"]["text2"], 320, i, "LANE"),
					cText2Align = private.GetTextAlign(lane["customText"]["text2"], 320, i, "LANE"),
					cText2S4 = { name = "", type = "description", order = 320 + 5.4, hidden = function(info) return not lane["customText"]["text2"]["edit"] end, },
					cText2XOffset = private.GetTextXOffset(lane["customText"]["text2"], 320, i, "LANE"),
					cText2YOffset = private.GetTextYOffset(lane["customText"]["text2"], 320, i, "LANE"),
					cText2Footer = { name = "", type = "header", order = 320 + 9.8, hidden = function(info) return not lane["customText"]["text2"]["edit"] end, },
					-- CUSTOM TEXT 3
					cText3S0 = { name = "\n", type = "description", order = 330, hidden = function(info) return not lane["customText"]["text3"]["used"] end, },
					cText3Header = { name = "", type = "header", order = 330 + 0.1, hidden = function(info) return not lane["customText"]["text3"]["edit"] end, },
					cText3Text = private.GetTextText(lane["customText"]["text3"], 330, i, "LANE"),
					cText3Enabled = private.GetTextEnabled(lane["customText"]["text3"], 330, i, "LANE"),
					cText3Edit = private.GetTextEdit(lane["customText"]["text3"], 330, i, "LANE"),
					cText3Font = private.GetTextFont(lane["customText"]["text3"], 330, i, "LANE"),
					cText3Outline = private.GetTextOutline(lane["customText"]["text3"], 330, i, "LANE"),
					cText3S1 = { name = "", type = "description", order = 330 + 4.3, hidden = function(info) return not lane["customText"]["text3"]["edit"] end, },
					cText3Size = private.GetTextSize(lane["customText"]["text3"], 330, i, "LANE"),
					cText3Color = private.GetTextColor(lane["customText"]["text3"], 330, i, "LANE"),
					cText3ShadowColor = private.GetTextShadowColor(lane["customText"]["text3"], 330, i, "LANE"),
					cText3S2 = { name = "", type = "description", order = 330 + 4.7, hidden = function(info) return not lane["customText"]["text3"]["edit"] end, },
					cText3ShadowXOffset = private.GetTextShadowXOffset(lane["customText"]["text3"], 330, i, "LANE"),
					cText3ShadowYOffset = private.GetTextShadowYOffset(lane["customText"]["text3"], 330, i, "LANE"),
					cText3S3 = { name = "", type = "description", order = 330 + 5.1, hidden = function(info) return not lane["customText"]["text3"]["edit"] end, },
					cText3Anchor = private.GetTextAnchor(lane["customText"]["text3"], 330, i, "LANE"),
					cText3Align = private.GetTextAlign(lane["customText"]["text3"], 330, i, "LANE"),
					cText3S4 = { name = "", type = "description", order = 330 + 5.4, hidden = function(info) return not lane["customText"]["text3"]["edit"] end, },
					cText3XOffset = private.GetTextXOffset(lane["customText"]["text3"], 330, i, "LANE"),
					cText3YOffset = private.GetTextYOffset(lane["customText"]["text3"], 330, i, "LANE"),
					cText3Footer = { name = "", type = "header", order = 330 + 9.8, hidden = function(info) return not lane["customText"]["text3"]["edit"] end, },
					-- CUSTOM TEXT 4
					cText4S0 = { name = "\n", type = "description", order = 340, hidden = function(info) return not lane["customText"]["text4"]["used"] end, },
					cText4Header = { name = "", type = "header", order = 340 + 0.1, hidden = function(info) return not lane["customText"]["text4"]["edit"] end, },
					cText4Text = private.GetTextText(lane["customText"]["text4"], 340, i, "LANE"),
					cText4Enabled = private.GetTextEnabled(lane["customText"]["text4"], 340, i, "LANE"),
					cText4Edit = private.GetTextEdit(lane["customText"]["text4"], 340, i, "LANE"),
					cText4Font = private.GetTextFont(lane["customText"]["text4"], 340, i, "LANE"),
					cText4Outline = private.GetTextOutline(lane["customText"]["text4"], 340, i, "LANE"),
					cText4S1 = { name = "", type = "description", order = 340 + 4.3, hidden = function(info) return not lane["customText"]["text4"]["edit"] end, },
					cText4Size = private.GetTextSize(lane["customText"]["text4"], 340, i, "LANE"),
					cText4Color = private.GetTextColor(lane["customText"]["text4"], 340, i, "LANE"),
					cText4ShadowColor = private.GetTextShadowColor(lane["customText"]["text4"], 340, i, "LANE"),
					cText4S2 = { name = "", type = "description", order = 340 + 4.7, hidden = function(info) return not lane["customText"]["text4"]["edit"] end, },
					cText4ShadowXOffset = private.GetTextShadowXOffset(lane["customText"]["text4"], 340, i, "LANE"),
					cText4ShadowYOffset = private.GetTextShadowYOffset(lane["customText"]["text4"], 340, i, "LANE"),
					cText4S3 = { name = "", type = "description", order = 340 + 5.1, hidden = function(info) return not lane["customText"]["text4"]["edit"] end, },
					cText4Anchor = private.GetTextAnchor(lane["customText"]["text4"], 340, i, "LANE"),
					cText4Align = private.GetTextAlign(lane["customText"]["text4"], 340, i, "LANE"),
					cText4S4 = { name = "", type = "description", order = 340 + 5.4, hidden = function(info) return not lane["customText"]["text4"]["edit"] end, },
					cText4XOffset = private.GetTextXOffset(lane["customText"]["text4"], 340, i, "LANE"),
					cText4YOffset = private.GetTextYOffset(lane["customText"]["text4"], 340, i, "LANE"),
					cText4Footer = { name = "", type = "header", order = 340 + 9.8, hidden = function(info) return not lane["customText"]["text4"]["edit"] end, },
					-- CUSTOM TEXT 5
					cText5S0 = { name = "\n", type = "description", order = 350, hidden = function(info) return not lane["customText"]["text5"]["used"] end, },
					cText5Header = { name = "", type = "header", order = 350 + 0.1, hidden = function(info) return not lane["customText"]["text5"]["edit"] end, },
					cText5Text = private.GetTextText(lane["customText"]["text5"], 350, i, "LANE"),
					cText5Enabled = private.GetTextEnabled(lane["customText"]["text5"], 350, i, "LANE"),
					cText5Edit = private.GetTextEdit(lane["customText"]["text5"], 350, i, "LANE"),
					cText5Font = private.GetTextFont(lane["customText"]["text5"], 350, i, "LANE"),
					cText5Outline = private.GetTextOutline(lane["customText"]["text5"], 350, i, "LANE"),
					cText5S1 = { name = "", type = "description", order = 350 + 4.3, hidden = function(info) return not lane["customText"]["text5"]["edit"] end, },
					cText5Size = private.GetTextSize(lane["customText"]["text5"], 350, i, "LANE"),
					cText5Color = private.GetTextColor(lane["customText"]["text5"], 350, i, "LANE"),
					cText5ShadowColor = private.GetTextShadowColor(lane["customText"]["text5"], 350, i, "LANE"),
					cText5S2 = { name = "", type = "description", order = 350 + 4.7, hidden = function(info) return not lane["customText"]["text5"]["edit"] end, },
					cText5ShadowXOffset = private.GetTextShadowXOffset(lane["customText"]["text5"], 350, i, "LANE"),
					cText5ShadowYOffset = private.GetTextShadowYOffset(lane["customText"]["text5"], 350, i, "LANE"),
					cText5S3 = { name = "", type = "description", order = 350 + 5.1, hidden = function(info) return not lane["customText"]["text5"]["edit"] end, },
					cText5Anchor = private.GetTextAnchor(lane["customText"]["text5"], 350, i, "LANE"),
					cText5Align = private.GetTextAlign(lane["customText"]["text5"], 350, i, "LANE"),
					cText5S4 = { name = "", type = "description", order = 350 + 5.4, hidden = function(info) return not lane["customText"]["text5"]["edit"] end, },
					cText5XOffset = private.GetTextXOffset(lane["customText"]["text5"], 350, i, "LANE"),
					cText5YOffset = private.GetTextYOffset(lane["customText"]["text5"], 350, i, "LANE"),
					cText5Footer = { name = "", type = "header", order = 350 + 9.8, hidden = function(info) return not lane["customText"]["text5"]["edit"] end, },
				},
			},
		}
	}
	
	return options
end

private.GetReadySet = function(i)
	local ready = nil
	
	if i == 1 then
		ready = CDTL2.db.profile.ready["ready1"]
	elseif i == 2 then
		ready = CDTL2.db.profile.ready["ready2"]
	elseif i == 3 then
		ready = CDTL2.db.profile.ready["ready3"]
	end
	
	local options = {
		--name = "Ready "..i,
		name = function(info)
				return ready["name"]
			end,
		type = "group",
		order = 100,
		--childGroups  = "tab",
		args = {
			general = {
				name = "General",
				type = "group",
				order = 100,
				args = {
					name = {
						name = "Frame Name",
						type = "input",
						order = 101,
						get = function(info)
								return ready["name"]
							end,
						set = function(info, val)
								ready["name"] = val
								CDTL2:RefreshReady(i)
							end,
					},
					enabled = {
						name = "Enabled",
						desc = "",
						order = 102,
						type = "toggle",
						--width = "full",
						get = function(info, index)
								return ready["enabled"]
							end,
						set = function(info, val)
								ready["enabled"] = val
								CDTL2:CreateReadyFrames()
								--CDTL2:RefreshReady(i)
							end,
					},
					spacer200 = {
						name = "\n\n",
						type = "description",
						order = 200,
					},
					grow = {
						name = "Grow Direction",
						desc = "Select which direction new icons will grow in",
						order = 201,
						type = "select",
						values = {
								["UP"] = "UP",
								["DOWN"] = "DOWN",
								["LEFT"] = "LEFT",
								["RIGHT"] = "RIGHT",
								["CENTER_H"] = "CENTER (H)",
								["CENTER_V"] = "CENTER (V)",
							},
						get = function(info, index)
								return ready["grow"]
							end,
						set = function(info, val)
								ready["grow"] = val
							end,
					},
					--[[sorting = {
						name = "Sorting",
						desc = "How to sort",
						order = 202,
						type = "select",
						values = {
								["ASCENDING"] = "ASCENDING",
								["DESCENDING"] = "DESCENDING",
							},
						get = function(info, index)
								return ready["sorting"]
							end,
						set = function(info, val)
								ready["sorting"] = val
							end,
					},]]--
					spacer300 = {
						name = "\n\n",
						type = "description",
						order = 300,
					},
					nTime = {
						name = "Normal Duration",
						desc = "How long an icon should stay in the ready frame",
						order = 301,
						type = "range",
						softMin = 1,
						softMax = 20,
						get = function(info, index)
								return ready["nTime"]
							end,
						set = function(info, val)
								ready["nTime"] = val
						end,
					},
					nSound = {
						name = "Normal Sound",
						desc = "Select which sound to play when an icon enters the frame",
						order = 302,
						type = "select",
						dialogControl = 'LSM30_Sound',
						values = AceGUIWidgetLSMlists.sound,
						get = function(info, index)
								return ready["nSound"]
							end,
						set = function(info, val)
								ready["nSound"] = val
							end,
					},
					spacer400 = {
						name = "",
						type = "description",
						order = 400,
					},
					hTime = {
						name = "Highlight Duration",
						desc = "How long a highlighted icon should stay in the ready frame",
						order = 401,
						type = "range",
						softMin = 1,
						softMax = 20,
						get = function(info, index)
								return ready["hTime"]
							end,
						set = function(info, val)
								ready["hTime"] = val
						end,
					},
					hSound = {
						name = "Highlight Sound",
						desc = "Select which sound to play when a highlighted icon enters the frame",
						order = 402,
						type = "select",
						dialogControl = 'LSM30_Sound',
						values = AceGUIWidgetLSMlists.sound,
						get = function(info, index)
								return ready["hSound"]
							end,
						set = function(info, val)
								ready["hSound"] = val
							end,
					},
					spacer500 = {
						name = "\n\n",
						type = "description",
						order = 500,
					},
					xPadding = {
						name = "x Padding",
						desc = "Sets the x offset between growing icons",
						order = 501,
						type = "range",
						softMin = 0,
						softMax = 40,
						get = function(info, index)
								return ready["icons"]["xPadding"]
							end,
						set = function(info, val)
								ready["icons"]["xPadding"] = val
						end,
					},
					yPadding = {
						name = "y Padding",
						desc = "Sets the y offset between growing icons",
						order = 502,
						type = "range",
						softMin = 0,
						softMax = 40,
						get = function(info, index)
								return ready["icons"]["yPadding"]
							end,
						set = function(info, val)
								ready["icons"]["yPadding"] = val
						end,
					},
				}
			},
			appearance = {
				name = "Appearance",
				type = "group",
				order = 200,
				args = {
					--[[spacer100 = {
						name = "\n\n",
						type = "description",
						order = 100,
					},]]--
					posX = {
						name = "x Position",
						desc = "Sets the x position of the frame",
						order = 111,
						type = "range",
						softMin = -500,
						softMax = 500,
						get = function(info, index)
								return ready["posX"]
							end,
						set = function(info, val)
								ready["posX"] = val
								CDTL2:RefreshReady(i)
							end,
					},
					posY = {
						name = "y Position",
						desc = "Sets the y position of the frame",
						order = 112,
						type = "range",
						softMin = -500,
						softMax = 500,
						get = function(info, index)
								return ready["posY"]
							end,
						set = function(info, val)
								ready["posY"] = val
								CDTL2:RefreshReady(i)
							end,
					},
					spacer113 = {
						name = "",
						type = "description",
						order = 113,
					},
					relativeTo = {
						name = "Anchor Point",
						desc = "The x/y position is relative to this point of the screen",
						order = 114,
						type = "select",
						values = {
								["TOPLEFT"] = "TOPLEFT",
								["TOP"] = "TOP",
								["TOPRIGHT"] = "TOPRIGHT",
								["LEFT"] = "LEFT",
								["CENTER"] = "CENTER",
								["RIGHT"] = "RIGHT",
								["BOTTOMLEFT"] = "BOTTOMLEFT",
								["BOTTOM"] = "BOTTOM",
								["BOTTOMRIGHT"] = "BOTTOMRIGHT",
							},
						get = function(info, index)
								return ready["relativeTo"]
							end,
						set = function(info, val)
								ready["relativeTo"] = val
								CDTL2:RefreshReady(i)
							end,
					},
					spacer200 = {
						name = "\n\n",
						type = "description",
						order = 200,
					},
					padding = {
						name = "Padding",
						desc = "Sets the amount of padding in the frame",
						order = 201,
						type = "range",
						softMin = 0,
						softMax = 40,
						get = function(info, index)
								return ready["padding"]
							end,
						set = function(info, val)
								ready["padding"] = val
								CDTL2:RefreshAllIcons()
								CDTL2:RefreshReady(i)
						end,
					},
					spacer300 = {
						name = "\n\n",
						type = "description",
						order = 300,
					},
					bgTexture = {
						name = "Background Texture",
						desc = "Sets the background texture",
						order = 301,
						type = "select",
						dialogControl = 'LSM30_Statusbar',
						values = AceGUIWidgetLSMlists.statusbar,
						get = function(info, index)
								return ready["bgTexture"]
							end,
						set = function(info, val)
								ready["bgTexture"] = val
								CDTL2:RefreshReady(i)
							end,
					},
					bgTextureColor = {
						name = "Color",
						desc = "Sets the background texture color",
						order = 302,
						type = "color",
						hasAlpha = true,
						get = function(info)
								local r = ready["bgTextureColor"]["r"]
								local g = ready["bgTextureColor"]["g"]
								local b = ready["bgTextureColor"]["b"]
								local a = ready["bgTextureColor"]["a"]
								return r, g, b, a
							end,
						set = function(info, red, green, blue, alpha)
								ready["bgTextureColor"] = { r = red, g = green, b = blue, a = alpha }
								CDTL2:RefreshReady(i)
							end,
					},
					spacer400 = {
						name = "\n\n",
						type = "description",
						order = 400,
					},
					border = {
						name = "                    Border Texture",
						desc = "Selects the border texture",
						order = 401,
						type = "select",
						dialogControl = 'LSM30_Border',
						values = AceGUIWidgetLSMlists.border,
						get = function(info) return ready["border"]["style"] end,
						set = function(info, val)
								ready["border"]["style"] = val
								CDTL2:RefreshReady(i)
							end,
					},
					borderColor = {
						name = "Color",
						desc = "Selects the border color",
						order = 402,
						type = "color",
						hasAlpha = true,
						get = function(info)
								local r = ready["border"]["color"]["r"]
								local g = ready["border"]["color"]["g"]
								local b = ready["border"]["color"]["b"]
								local a = ready["border"]["color"]["a"]
								
								return r, g, b, a
							end,
						set = function(info, red, green, blue, alpha)
								ready["border"]["color"] = { r = red, g = green, b = blue, a = alpha }							
								CDTL2:RefreshReady(i)
							end,
					},
					spacer410 = {
						name = "",
						type = "description",
						order = 410,
					},
					borderSize = {
						name = "Size",
						desc = "Sets the size of the border",
						order = 411,
						type = "range",
						softMin = 1,
						softMax = 40,
						get = function(info) return ready["border"]["size"] end,
						set = function(info, val)
								ready["border"]["size"] = val
								CDTL2:RefreshReady(i)
							end,
					},
					borderPadding = {
						name = "Padding",
						desc = "Sets the size of the border",
						order = 412,
						type = "range",
						softMin = 0,
						softMax = 40,
						get = function(info) return ready["border"]["padding"] end,
						set = function(info, val)
								ready["border"]["padding"] = val
								CDTL2:RefreshReady(i)
							end,
					},
				}
			},
			icons = {
				name = "Icons",
				type = "group",
				order = 300,
				args = {
					--[[spacer100 = {
						name = "\n\n",
						type = "description",
						order = 100,
					},]]--
					size = {
						name = "Size",
						desc = "Sets the size of the frame, and icons in the ready frame",
						order = 101,
						type = "range",
						softMin = 1,
						softMax = 128,
						get = function(info, index)
								return ready["icons"]["size"]
							end,
						set = function(info, val)
								ready["icons"]["size"] = val
								CDTL2:RefreshAllIcons()
								CDTL2:RefreshReady(i)
						end,
					},
					spacer102 = {
						name = "\n\n",
						type = "description",
						order = 102,
					},
					textHeader = {
						name = "Icon Text",
						type = "header",
						order = 200.1,
					},
					spacer200 = {
						name = "\n",
						type = "description",
						order = 200.2,
					},
					-- ICON TEXT 1
					--iText1S0 = { name = "\n", type = "description", order = 210, },
					iText1Text = private.GetTextText(ready["icons"]["text1"], 210, i, "ICON", false),
					iText1Enabled = private.GetTextEnabled(ready["icons"]["text1"], 210, i, "ICON"),
					iText1Edit = private.GetTextEdit(ready["icons"]["text1"], 210, i, "ICON"),
					iText1Font = private.GetTextFont(ready["icons"]["text1"], 210, i, "ICON"),
					iText1Outline = private.GetTextOutline(ready["icons"]["text1"], 210, i, "ICON"),
					iText1S1 = { name = "", type = "description", order = 210 + 4.3, hidden = function(info) return not ready["icons"]["text1"]["edit"] end, },
					iText1Size = private.GetTextSize(ready["icons"]["text1"], 210, i, "ICON"),
					iText1Color = private.GetTextColor(ready["icons"]["text1"], 210, i, "ICON"),
					iText1ShadowColor = private.GetTextShadowColor(ready["icons"]["text1"], 210, i, "ICON"),
					iText1S2 = { name = "", type = "description", order = 210 + 4.7, hidden = function(info) return not ready["icons"]["text1"]["edit"] end, },
					iText1ShadowXOffset = private.GetTextShadowXOffset(ready["icons"]["text1"], 210, i, "ICON"),
					iText1ShadowYOffset = private.GetTextShadowYOffset(ready["icons"]["text1"], 210, i, "ICON"),
					iText1S3 = { name = "", type = "description", order = 210 + 5.1, hidden = function(info) return not ready["icons"]["text1"]["edit"] end, },
					iText1Anchor = private.GetTextAnchor(ready["icons"]["text1"], 210, i, "ICON"),
					iText1Align = private.GetTextAlign(ready["icons"]["text1"], 210, i, "ICON"),
					iText1S4 = { name = "", type = "description", order = 210 + 5.4, hidden = function(info) return not ready["icons"]["text1"]["edit"] end, },
					iText1XOffset = private.GetTextXOffset(ready["icons"]["text1"], 210, i, "ICON"),
					iText1YOffset = private.GetTextYOffset(ready["icons"]["text1"], 210, i, "ICON"),
					iText1Footer = { name = "", type = "header", order = 210 + 9.8, hidden = function(info) return not ready["icons"]["text1"]["edit"] end, },
					-- ICON TEXT 2
					iText2S0 = { name = "\n", type = "description", order = 220, hidden = function(info) return not ready["icons"]["text2"]["used"] end, },
					iText2Header = { name = "", type = "header", order = 220 + 0.1, hidden = function(info) return not ready["icons"]["text2"]["edit"] end, },
					iText2Text = private.GetTextText(ready["icons"]["text2"], 220, i, "ICON", false),
					iText2Enabled = private.GetTextEnabled(ready["icons"]["text2"], 220, i, "ICON"),
					iText2Edit = private.GetTextEdit(ready["icons"]["text2"], 220, i, "ICON"),
					iText2Font = private.GetTextFont(ready["icons"]["text2"], 220, i, "ICON"),
					iText2Outline = private.GetTextOutline(ready["icons"]["text2"], 220, i, "ICON"),
					iText2S1 = { name = "", type = "description", order = 220 + 4.3, hidden = function(info) return not ready["icons"]["text2"]["edit"] end, },
					iText2Size = private.GetTextSize(ready["icons"]["text2"], 220, i, "ICON"),
					iText2Color = private.GetTextColor(ready["icons"]["text2"], 220, i, "ICON"),
					iText2ShadowColor = private.GetTextShadowColor(ready["icons"]["text2"], 220, i, "ICON"),
					iText2S2 = { name = "", type = "description", order = 220 + 4.7, hidden = function(info) return not ready["icons"]["text2"]["edit"] end, },
					iText2ShadowXOffset = private.GetTextShadowXOffset(ready["icons"]["text2"], 220, i, "ICON"),
					iText2ShadowYOffset = private.GetTextShadowYOffset(ready["icons"]["text2"], 220, i, "ICON"),
					iText2S3 = { name = "", type = "description", order = 220 + 5.1, hidden = function(info) return not ready["icons"]["text2"]["edit"] end, },
					iText2Anchor = private.GetTextAnchor(ready["icons"]["text2"], 220, i, "ICON"),
					iText2Align = private.GetTextAlign(ready["icons"]["text2"], 220, i, "ICON"),
					iText2S4 = { name = "", type = "description", order = 220 + 5.4, hidden = function(info) return not ready["icons"]["text2"]["edit"] end, },
					iText2XOffset = private.GetTextXOffset(ready["icons"]["text2"], 220, i, "ICON"),
					iText2YOffset = private.GetTextYOffset(ready["icons"]["text2"], 220, i, "ICON"),
					iText2Footer = { name = "", type = "header", order = 220 + 9.8, hidden = function(info) return not ready["icons"]["text2"]["edit"] end, },
					-- ICON TEXT 3
					iText3S0 = { name = "\n", type = "description", order = 230, hidden = function(info) return not ready["icons"]["text3"]["used"] end, },
					iText3Header = { name = "", type = "header", order = 230 + 0.1, hidden = function(info) return not ready["icons"]["text3"]["edit"] end, },
					iText3Text = private.GetTextText(ready["icons"]["text3"], 230, i, "ICON", false),
					iText3Enabled = private.GetTextEnabled(ready["icons"]["text3"], 230, i, "ICON"),
					iText3Edit = private.GetTextEdit(ready["icons"]["text3"], 230, i, "ICON"),
					iText3Font = private.GetTextFont(ready["icons"]["text3"], 230, i, "ICON"),
					iText3Outline = private.GetTextOutline(ready["icons"]["text3"], 230, i, "ICON"),
					iText3S1 = { name = "", type = "description", order = 230 + 4.3, hidden = function(info) return not ready["icons"]["text3"]["edit"] end, },
					iText3Size = private.GetTextSize(ready["icons"]["text3"], 230, i, "ICON"),
					iText3Color = private.GetTextColor(ready["icons"]["text3"], 230, i, "ICON"),
					iText3ShadowColor = private.GetTextShadowColor(ready["icons"]["text3"], 230, i, "ICON"),
					iText3S2 = { name = "", type = "description", order = 230 + 4.7, hidden = function(info) return not ready["icons"]["text3"]["edit"] end, },
					iText3ShadowXOffset = private.GetTextShadowXOffset(ready["icons"]["text3"], 230, i, "ICON"),
					iText3ShadowYOffset = private.GetTextShadowYOffset(ready["icons"]["text3"], 230, i, "ICON"),
					iText3S3 = { name = "", type = "description", order = 230 + 5.1, hidden = function(info) return not ready["icons"]["text3"]["edit"] end, },
					iText3Anchor = private.GetTextAnchor(ready["icons"]["text3"], 230, i, "ICON"),
					iText3Align = private.GetTextAlign(ready["icons"]["text3"], 230, i, "ICON"),
					iText3S4 = { name = "", type = "description", order = 230 + 5.4, hidden = function(info) return not ready["icons"]["text3"]["edit"] end, },
					iText3XOffset = private.GetTextXOffset(ready["icons"]["text3"], 230, i, "ICON"),
					iText3YOffset = private.GetTextYOffset(ready["icons"]["text3"], 230, i, "ICON"),
					iText3Footer = { name = "", type = "header", order = 230 + 9.8, hidden = function(info) return not ready["icons"]["text3"]["edit"] end, },
					spacer298 = {
						name = "\n",
						type = "description",
						order = 298,
					},
					textFooter = {
						name = "",
						type = "header",
						order = 299,
					},
					spacer300 = {
						name = "\n\n",
						type = "description",
						order = 300,
					},
					border = {
						name = "                    Border Texture",
						desc = "Selects the border texture",
						order = 301,
						type = "select",
						dialogControl = 'LSM30_Border',
						values = AceGUIWidgetLSMlists.border,
						get = function(info) return ready["icons"]["border"]["style"] end,
						set = function(info, val)
								ready["icons"]["border"]["style"] = val
								CDTL2:RefreshAllIcons()
							end,
					},
					borderColor = {
						name = "Color",
						desc = "Selects the border color",
						order = 302,
						type = "color",
						hasAlpha = true,
						get = function(info)
								local r = ready["icons"]["border"]["color"]["r"]
								local g = ready["icons"]["border"]["color"]["g"]
								local b = ready["icons"]["border"]["color"]["b"]
								local a = ready["icons"]["border"]["color"]["a"]
								
								return r, g, b, a
							end,
						set = function(info, red, green, blue, alpha)
								ready["icons"]["border"]["color"] = { r = red, g = green, b = blue, a = alpha }							
								CDTL2:RefreshAllIcons()
							end,
					},
					spacer310 = {
						name = "",
						type = "description",
						order = 310,
					},
					borderSize = {
						name = "Size",
						desc = "Sets the size of the border",
						order = 311,
						type = "range",
						softMin = 1,
						softMax = 40,
						get = function(info) return ready["icons"]["border"]["size"] end,
						set = function(info, val)
								ready["icons"]["border"]["size"] = val
								CDTL2:RefreshAllIcons()
							end,
					},
					borderPadding = {
						name = "Padding",
						desc = "Sets the size of the border",
						order = 311,
						type = "range",
						softMin = 0,
						softMax = 40,
						get = function(info) return ready["icons"]["border"]["padding"] end,
						set = function(info, val)
								ready["icons"]["border"]["padding"] = val
								CDTL2:RefreshAllIcons()
							end,
					},
				}
			},
		}
	}
	
	return options
end

private.GetTextText = function(s, o, i, r, d)
	local options = {
		name = function(info)
				return " "
			end,
		desc = function(info)
				if r == "LANE" then
					return CDTL2:GetCustomTextTagDescription()
				else
					return CDTL2:GetCustomIconTagDescription()
				end
			end,
		type = "input",
		order = o + 1,
		disabled = d,
		hidden = function(info)
				return not s["used"]
			end,
		get = function(info)
				return s["text"]
			end,
		set = function(info, val)
				s["text"] = val
				if r == "LANE" then
					CDTL2:RefreshLane(i)
				elseif r == "BAR" then
					CDTL2:RefreshAllBars()
				elseif r == "ICON" then
					CDTL2:RefreshAllIcons()
				end
			end,
	}
	
	return options
end

private.GetTextEnabled = function(s, o, i, r)
	local options = {
		name = "Enabled",
		desc = "Show this text",
		order = o + 2,
		type = "toggle",
		width = 0.5,
		hidden = function(info)
				return not s["used"]
			end,
		get = function(info)
				return s["enabled"]
			end,
		set = function(info, val)
				s["enabled"] = val
				if r == "LANE" then
					CDTL2:RefreshLane(i)
				elseif r == "BAR" then
					CDTL2:RefreshAllBars()
				elseif r == "ICON" then
					CDTL2:RefreshAllIcons()
				end
			end,
	}
	
	return options
end

private.GetTextEdit = function(s, o, i, r)
	local options = {
		name = "Edit",
		desc = "Show this text",
		order = o + 3,
		type = "toggle",
		width = 0.5,
		hidden = function(info)
				return not s["used"]
			end,
		get = function(info)
				return s["edit"]
			end,
		set = function(info, val)
				s["edit"] = val
				if r == "LANE" then
					CDTL2:RefreshLane(i)
				elseif r == "BAR" then
					CDTL2:RefreshAllBars()
				elseif r == "ICON" then
					CDTL2:RefreshAllIcons()
				end
			end,
	}
	
	return options
end

private.GetTextFont = function(s, o, i, r)
	local options = {
		name = "Font",
		desc = "Selects the font for text on the bars",
		order = o + 4.1,
		hidden = function(info)
				return not s["edit"]
			end,
		type = "select",
		dialogControl = 'LSM30_Font',
		values = AceGUIWidgetLSMlists.font,
		get = function(info) return s["font"] end,
		set = function(info, val)
				s["font"] = val
				if r == "LANE" then
					CDTL2:RefreshLane(i)
				elseif r == "BAR" then
					CDTL2:RefreshAllBars()
				elseif r == "ICON" then
					CDTL2:RefreshAllIcons()
				end
			end,
	}
	
	return options
end

private.GetTextOutline = function(s, o, i, r)
	local options = {
		name = "Outline",
		desc = "Sets the text outline",
		order = o + 4.2,
		hidden = function(info)
				return not s["edit"]
			end,
		type = "select",
		values = {
				["NONE"] = "None",
				["OUTLINE"] = "Outline",
				["THICKOUTLINE"] = "Thick Outline",
				["MONOCHROME"] = "Monochrome"
			},
		get = function(info) return s["outline"] end,
		set = function(info, val)
				s["outline"] = val
				if r == "LANE" then
					CDTL2:RefreshLane(i)
				elseif r == "BAR" then
					CDTL2:RefreshAllBars()
				elseif r == "ICON" then
					CDTL2:RefreshAllIcons()
				end
			end,
	}
	
	return options
end

private.GetTextSize = function(s, o, i, r)
	local options = {
		name = "Font Size",
		desc = "Sets the size of the font",
		order = o + 4.4,
		hidden = function(info)
				return not s["edit"]
			end,
		type = "range",
		softMin = 0,
		softMax = 64,
		get = function(info) return s["size"] end,
		set = function(info, val)
				s["size"] = val
				if r == "LANE" then
					CDTL2:RefreshLane(i)
				elseif r == "BAR" then
					CDTL2:RefreshAllBars()
				elseif r == "ICON" then
					CDTL2:RefreshAllIcons()
				end
			end,
	}
	
	return options
end

private.GetTextColor = function(s, o, i, r)
	local options = {
		name = "Color",
		desc = "Selects the font color",
		order = o + 4.5,
		hidden = function(info)
				return not s["edit"]
			end,
		type = "color",
		hasAlpha = true,
		get = function(info)
				local t = s["color"]
				
				local r = t["r"]
				local g = t["g"]
				local b = t["b"]
				local a = t["a"]
				return r, g, b, a
			end,
		set = function(info, red, green, blue, alpha)
				s["color"] = { r = red, g = green, b = blue, a = alpha }
				if r == "LANE" then
					CDTL2:RefreshLane(i)
				elseif r == "BAR" then
					CDTL2:RefreshAllBars()
				elseif r == "ICON" then
					CDTL2:RefreshAllIcons()
				end
			end,
	}
	
	return options
end

private.GetTextShadowColor = function(s, o, i, r)
	local options = {
		name = "Shadow Color",
		desc = "Selects the shadow color",
		order = o + 4.6,
		hidden = function(info)
				return not s["edit"]
			end,
		type = "color",
		hasAlpha = true,
		get = function(info)
				local t = s["shadColor"]
				
				local r = t["r"]
				local g = t["g"]
				local b = t["b"]
				local a = t["a"]
				return r, g, b, a
			end,
		set = function(info, red, green, blue, alpha)
				s["shadColor"] = { r = red, g = green, b = blue, a = alpha }
				if r == "LANE" then
					CDTL2:RefreshLane(i)
				elseif r == "BAR" then
					CDTL2:RefreshAllBars()
				elseif r == "ICON" then
					CDTL2:RefreshAllIcons()
				end
			end,
	}
	
	return options
end

private.GetTextShadowXOffset = function(s, o, i, r)
	local options = {
		name = "Shadow x Offset",
		desc = "Sets the text shadow x offset",
		order = o + 4.8,
		hidden = function(info)
				return not s["edit"]
			end,
		type = "range",
		softMin = -5,
		softMax = 5,
		get = function(info) return s["shadX"] end,
		set = function(info, val)
				s["shadX"] = val
				if r == "LANE" then
					CDTL2:RefreshLane(i)
				elseif r == "BAR" then
					CDTL2:RefreshAllBars()
				elseif r == "ICON" then
					CDTL2:RefreshAllIcons()
				end
			end,
	}
	
	return options
end

private.GetTextShadowYOffset = function(s, o, i, r)
	local options = {
		name = "Shadow y Offset",
		desc = "Sets the text shadow y offset",
		order = o + 4.9,
		hidden = function(info)
				return not s["edit"]
			end,
		type = "range",
		softMin = -5,
		softMax = 5,
		get = function(info) return s["shadY"] end,
		set = function(info, val)
				s["shadY"] = val
				if r == "LANE" then
					CDTL2:RefreshLane(i)
				elseif r == "BAR" then
					CDTL2:RefreshAllBars()
				elseif r == "ICON" then
					CDTL2:RefreshAllIcons()
				end
			end,
	}
	
	return options
end

private.GetTextAnchor = function(s, o, i, r)
	local options = {
		name = "Anchor",
		desc = "Sets the text anchor point",
		order = o + 5.2,
		hidden = function(info)
				return not s["edit"]
			end,
		type = "select",
		values = {
				["TOPLEFT"] = "TOPLEFT",
				["TOP"] = "TOP",
				["TOPRIGHT"] = "TOPRIGHT",
				["LEFT"] = "LEFT",
				["CENTER"] = "CENTER",
				["RIGHT"] = "RIGHT",
				["BOTTOMLEFT"] = "BOTTOMLEFT",
				["BOTTOM"] = "BOTTOM",
				["BOTTOMRIGHT"] = "BOTTOMRIGHT",
			},
		get = function(info) return s["anchor"] end,
		set = function(info, val)
				s["anchor"] = val
				if r == "LANE" then
					CDTL2:RefreshLane(i)
				elseif r == "BAR" then
					CDTL2:RefreshAllBars()
				elseif r == "ICON" then
					CDTL2:RefreshAllIcons()
				end
			end,
	}
	
	return options
end

private.GetTextAlign = function(s, o, i, r)
	local options = {
		name = "Align",
		desc = "Sets the text alignment",
		order = o + 5.3,
		hidden = function(info)
				return not s["edit"]
			end,
		type = "select",
		values = {
				["LEFT"] = "LEFT",
				["CENTER"] = "CENTER",
				["RIGHT"] = "RIGHT",
			},
		get = function(info) return s["align"] end,
		set = function(info, val)
				s["align"] = val
				if r == "LANE" then
					CDTL2:RefreshLane(i)
				elseif r == "BAR" then
					CDTL2:RefreshAllBars()
				elseif r == "ICON" then
					CDTL2:RefreshAllIcons()
				end
			end,
	}
	
	return options
end

private.GetTextXOffset = function(s, o, i, r)
	local options = {
		name = "x Offset",
		desc = "Sets text x offset",
		order = o + 5.5,
		hidden = function(info)
				return not s["edit"]
			end,
		type = "range",
		softMin = -5,
		softMax = 5,
		get = function(info) return s["offX"] end,
		set = function(info, val)
				s["offX"] = val
				if r == "LANE" then
					CDTL2:RefreshLane(i)
				elseif r == "BAR" then
					CDTL2:RefreshAllBars()
				elseif r == "ICON" then
					CDTL2:RefreshAllIcons()
				end
			end,
	}
	
	return options
end

private.GetTextYOffset = function(s, o, i, r)
	local options = {
		name = "y Offset",
		desc = "Sets text y offset",
		order = o + 5.6,
		hidden = function(info)
				return not s["edit"]
			end,
		type = "range",
		softMin = -5,
		softMax = 5,
		get = function(info) return s["offY"] end,
		set = function(info, val)
				s["offY"] = val
				if r == "LANE" then
					CDTL2:RefreshLane(i)
				elseif r == "BAR" then
					CDTL2:RefreshAllBars()
				elseif r == "ICON" then
					CDTL2:RefreshAllIcons()
				end
			end,
	}
	
	return options
end