local private = {}

function CDTL2:CreateReadyFrames()
	local ready1Enabled = CDTL2.db.profile.ready["ready1"]["enabled"]
	local ready2Enabled = CDTL2.db.profile.ready["ready2"]["enabled"]
	local ready3Enabled = CDTL2.db.profile.ready["ready3"]["enabled"]
	
	if ready1Enabled then
		if CDTL2_Ready_1 then
			CDTL2:RefreshReady(1)
			--CDTL2:Print("Ready 1 already created")
		else
			private.CreateReady(1)
			--CDTL2:Print("Created Ready 1")
		end
	end
	
	if ready2Enabled then
		if CDTL2_Ready_2 then
			CDTL2:RefreshReady(2)
			--CDTL2:Print("Ready 2 already created")
		else
			private.CreateReady(2)
			--CDTL2:Print("Created Ready 2")
		end
	end
	
	if ready3Enabled then
		if CDTL2_Ready_3 then
			CDTL2:RefreshReady(3)
			--CDTL2:Print("Ready 3 already created")
		else
			private.CreateReady(3)
			--CDTL2:Print("Created Ready 3")
		end
	end
end

private.CreateReady = function(frameNumber)
	local frameName = "CDTL2_Ready_"..frameNumber
	local f = CreateFrame("Frame", frameName, UIParent, BackdropTemplateMixin and "BackdropTemplate" or nil)

	f.number = frameNumber
	f.childCount = 0

	f.mf = CreateFrame("Frame", frameName.."_MF", UIParent, BackdropTemplateMixin and "BackdropTemplate" or nil)
	f.mf:SetParent(f)
	f.mf.bg = f.mf:CreateTexture(nil, "BACKGROUND")
	f.mf.bd = CreateFrame("Frame", frameName.."_BD", UIParent, BackdropTemplateMixin and "BackdropTemplate" or nil)
	f.mf.bd:SetParent(f.mf)

	f.bg = f:CreateTexture(nil, "BACKGROUND")
	--f.bd = CreateFrame("Frame", frameName.."_BD", UIParent, BackdropTemplateMixin and "BackdropTemplate" or nil)
	--f.bd:SetParent(f)
	
	f.db = CreateFrame("Frame", frameName.."_DB", UIParent, BackdropTemplateMixin and "BackdropTemplate" or nil)
	f.db:SetParent(f)
	f.db.bg = f.db:CreateTexture(nil, "BACKGROUND")
	f.db.text = f.db:CreateFontString(nil,"ARTWORK")
	
	-- ON UPDATE
	f:HookScript("OnUpdate", function(self, elapsed)
		private.ReadyUpdate(self, elapsed)
	end)
	
	-- DRAG AND DROP MOVEMENT
	f:RegisterForDrag("LeftButton")
	f:SetScript("OnDragStart", f.StartMoving)
	f:SetScript("OnDragStop", f.StopMovingOrSizing)
	
	CDTL2:RefreshReady(frameNumber)
	table.insert(CDTL2.readyFrames, f)
end

private.ReadyUpdate = function(f, elapsed)
	local s = nil
	if f.number == 1 then
		s = CDTL2.db.profile.ready["ready1"]
	elseif f.number == 2 then
		s = CDTL2.db.profile.ready["ready2"]
	elseif f.number == 3 then
		s = CDTL2.db.profile.ready["ready3"]
	end

	local children = { f.mf:GetChildren() }
	local validChildren = {}
	
	local count = 0
	for _, child in ipairs(children) do
		if child.valid then
			if child:GetAlpha() ~= 0 then
				count = count + 1
				table.insert(validChildren, child)
			end
		end
	end
	
	-- POSITION/SORT CHILDREN
	--[[for k, child in ipairs(validChildren) do
		local x = s["icons"]["xPadding"] * (k - 1)
		local y = s["icons"]["size"] * (k - 1)
		
		child:ClearAllPoints()
		if s["grow"] == "DOWN" then
			y = -y - (s["icons"]["yPadding"] * (k - 1)) - s["padding"]
			child:SetPoint("TOP", x, y)
		else
			y = y + (s["icons"]["yPadding"] * (k - 1)) + s["padding"]
			child:SetPoint("BOTTOM", x, y)
		end
	end]]--
	
	-- POSITION/SORT FRAME
	--[[local w = s["icons"]["size"] + (2 * s["padding"])
	local h = s["icons"]["size"] + (2 * s["padding"])
	local xPos = s["posX"]
	local yPos = s["posY"]
	
	if count > 1 then
		w = s["icons"]["size"] + (2 * s["padding"]) + ((count - 1) * s["icons"]["xPadding"])
		h = (s["icons"]["size"] * count) + (2 * s["padding"]) + ((count - 1) * s["icons"]["yPadding"])
	end
	
	local anchor = "CENTER"
	if s["grow"] == "DOWN" then
		anchor = "TOP"
		yPos = yPos + (h / 2)
	elseif  s["grow"] == "UP" then
		anchor = "BOTTOM"
		yPos = yPos - (h / 2)
	end]]--
	
	local bWidth = s["icons"]["size"]
	local bHeight = s["icons"]["size"]
	local bPadX = s["icons"]["xPadding"]
	local bPadY = s["icons"]["yPadding"]
	local fPad = s["padding"]
	
	local mfSizeX = bWidth + (fPad * 2)
	local mfSizeY = bHeight + (fPad * 2)
	
	local mfPosX = 0
	local mfPosY = 0
	
	local xInset = 0
	local yInset = 0
	
	local xOffset = 0
	local yOffset = 0
	
	local xMod = 0
	local yMod = 0
	
	if count == 1 then
		validChildren[1]:ClearAllPoints()	
		validChildren[1]:SetPoint("CENTER", 0, 0)
	elseif count > 1 then
		local anchor = "TOP"
		
		if s["grow"] == "UP" then
			yInset = -fPad
			xMod = 0
			yMod = -1
			
			mfSizeY = (bHeight * count) + (fPad * 2) + (bPadY + (count - 1))
			yOffset = (mfSizeY - (fPad * 2)) / 2
			yOffset = yOffset - (bHeight / 2)
		
		elseif s["grow"] == "DOWN" then
			yInset = -fPad
			xMod = 0
			yMod = -1
			
			mfSizeY = (bHeight * count) + (fPad * 2) + (bPadY + (count - 1))
			yOffset = (mfSizeY - (fPad * 2)) / -2
			yOffset = yOffset + (bHeight / 2)
		
		elseif s["grow"] == "LEFT" then
			anchor = "LEFT"
			xInset = fPad
			xMod = 1
			yMod = 0
			
			mfSizeX = (bWidth * count) + (fPad * 2) + (bPadX + (count - 1))
			xOffset = (mfSizeX - (fPad * 2)) / -2
			xOffset = xOffset + (bWidth / 2)
		
		elseif s["grow"] == "RIGHT" then
			anchor = "LEFT"
			xInset = fPad
			xMod = 1
			yMod = 0
			
			mfSizeX = (bWidth * count) + (fPad * 2) + (bPadX + (count - 1))
			xOffset = (mfSizeX - (fPad * 2)) / 2
			xOffset = xOffset - (bWidth / 2)
		
		elseif s["grow"] == "CENTER_V" then
			yInset = -fPad
			xMod = 0
			yMod = -1
			
			mfSizeY = (bHeight * count) + (fPad * 2) + (bPadY + (count - 1))
		
		elseif s["grow"] == "CENTER_H" then
			anchor = "LEFT"
			xInset = fPad
			xMod = 1
			yMod = 0
			
			mfSizeX = (bWidth * count) + (fPad * 2) + (bPadX + (count - 1))
		end
		
		for k, child in ipairs(validChildren) do
			local x = (bWidth * (k - 1) * xMod) + (bPadX * (k - 1))
			local y = (bHeight * (k - 1) * yMod) - (bPadY * (k - 1))
						
			child:ClearAllPoints()		
			child:SetPoint(anchor, x + xInset, y + yInset)
		end
	end
	
	f.mf:SetSize(mfSizeX, mfSizeY)
	f.mf:ClearAllPoints()
	f.mf:SetPoint("CENTER", mfPosX + xOffset, mfPosY + yOffset)
	
	-- FRAME (UN)LOCKING
	if CDTL2.db.profile.global["unlockFrames"] then
		local _, _, relativeTo, xOfs, yOfs = f:GetPoint()

		if f.number == 1 then
			CDTL2.db.profile.ready["ready1"]["relativeTo"] = relativeTo
			CDTL2.db.profile.ready["ready1"]["posX"] = xOfs
			CDTL2.db.profile.ready["ready1"]["posY"] = yOfs
		elseif f.number == 2 then
			CDTL2.db.profile.ready["ready2"]["relativeTo"] = relativeTo
			CDTL2.db.profile.ready["ready2"]["posX"] = xOfs
			CDTL2.db.profile.ready["ready2"]["posY"] = yOfs
		end
	else
		--f:SetSize(w, h)
		--f:ClearAllPoints()
		--f:SetPoint(anchor, UIParent, s["relativeTo"], xPos, yPos)
	end
	
	f.childCount = count	
	--CDTL2:Autohide(f)
	
	if s["enabled"] then
		CDTL2:Autohide(f)
	else
		f:SetAlpha(0)
	end
end

function CDTL2:RefreshReady(i)
	local f = nil
	local s = nil
	if i == 1 then
		f = CDTL2_Ready_1
		s = CDTL2.db.profile.ready["ready1"]
	elseif i == 2 then
		f = CDTL2_Ready_2
		s = CDTL2.db.profile.ready["ready2"]
	elseif i == 3 then
		f = CDTL2_Ready_3
		s = CDTL2.db.profile.ready["ready3"]
	end
	
	--CDTL2:Print(tostring(i))
	
	f.name = s["name"]

	f:ClearAllPoints()
	f:SetPoint(s["relativeTo"], s["posX"], s["posY"])
	f:SetSize(s["icons"]["size"], s["icons"]["size"])
	
	f.mf:ClearAllPoints()
	f.mf:SetPoint("CENTER", 0, 0)
	f.mf:SetSize(s["icons"]["size"], s["icons"]["size"])
	
	-- BACKGROUND
	f.bg:SetTexture(CDTL2.LSM:Fetch("statusbar", s["bgTexture"]))
	f.bg:SetAllPoints(true)
	f.bg:SetVertexColor(
		s["bgTextureColor"]["r"],
		s["bgTextureColor"]["g"],
		s["bgTextureColor"]["b"],
		s["bgTextureColor"]["a"]
	)
	
	-- MF BACKGROUND
	f.mf.bg:SetTexture(CDTL2.LSM:Fetch("statusbar", s["bgTexture"]))
	f.mf.bg:SetAllPoints(true)
	f.mf.bg:SetVertexColor(
		s["bgTextureColor"]["r"],
		s["bgTextureColor"]["g"],
		s["bgTextureColor"]["b"],
		s["bgTextureColor"]["a"]
	)
	
	-- BORDER
	CDTL2:SetBorder(f.mf.bd, s["border"])
	f.mf.bd:SetFrameLevel(f.mf:GetFrameLevel() + 1)
	
	-- DEBUG/UNLOCK
	f.db:ClearAllPoints()
	f.db:SetPoint("CENTER", 0, 0)
	f.db.text:SetFont(CDTL2.LSM:Fetch("font", "Fira Sans Condensed"), 12, "NONE")
	f.db.text:ClearAllPoints()
	f.db.text:SetPoint("CENTER", 0, 0)
	f.db.text:SetText(f.name)
	f.db:SetSize(s["icons"]["size"], s["icons"]["size"])
	f.db.bg:SetAllPoints(true)
	f.db.bg:SetColorTexture( 
		CDTL2.colors["db"]["r"],
		CDTL2.colors["db"]["g"],
		CDTL2.colors["db"]["b"],
		CDTL2.colors["db"]["a"]
	)
	
	if CDTL2.db.profile.global["unlockFrames"] then
		CDTL2:FrameUnlock(f)
	else
		CDTL2:FrameLock(f)
	end
	
	if CDTL2.db.profile.global["debugMode"] then
		CDTL2:DebugOn(f)
	else
		CDTL2:DebugOff(f)
	end
end